@@ -39,6 +39,7 @@
 #include "cairo-dock-task.h"
 #include "cairo-dock-log.h"
 #include "cairo-dock-packages.h"
+#include "cairo-dock-keybinder.h"
 #include "cairo-dock-X-manager.h"
 #include "cairo-dock-X-utilities.h"
 #include "cairo-dock-themes-manager.h"
@@ -46,20 +47,24 @@
 #include "cairo-dock-gui-backend.h"
 #include "cairo-dock-gui-commons.h"
 
+#define CAIRO_DOCK_PLUGINS_EXTRAS_URL "http://extras.glx-dock.org"
+
 static CairoDockTask *s_pImportTask = NULL;
 
 extern gchar *g_cThemesDirPath;
 extern gchar *g_cConfFile;
 extern CairoContainer *g_pPrimaryContainer;
 extern CairoDockDesktopGeometry g_desktopGeometry;
+extern int g_iMajorVersion, g_iMinorVersion, g_iMicroVersion;
 
-static void _cairo_dock_fill_model_with_themes (const gchar *cThemeName, CairoDockPackage *pTheme, GtkListStore *pModele)
+
+static void _cairo_dock_fill_model_with_themes (const gchar *cThemeName, CairoDockPackage *pTheme, GtkListStore *pModel)
 {
 	GtkTreeIter iter;
 	memset (&iter, 0, sizeof (GtkTreeIter));
-	gtk_list_store_append (GTK_LIST_STORE (pModele), &iter);
+	gtk_list_store_append (GTK_LIST_STORE (pModel), &iter);
 	
-	gtk_list_store_set (GTK_LIST_STORE (pModele), &iter,
+	gtk_list_store_set (GTK_LIST_STORE (pModel), &iter,
 		CAIRO_DOCK_MODEL_NAME, pTheme->cDisplayedName,
 		CAIRO_DOCK_MODEL_RESULT, cThemeName, -1);
 }
@@ -96,7 +101,7 @@ void cairo_dock_make_tree_view_for_delete_themes (GtkWidget *pWindow)
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
 	
 	GtkWidget *pScrolledWindow = gtk_scrolled_window_new (NULL, NULL);
-	gtk_widget_set (pScrolledWindow, "height-request", 160, NULL);  // environ 8 lignes.
+	g_object_set (pScrolledWindow, "height-request", 160, NULL);  // environ 8 lignes.
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (pScrolledWindow), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
 	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (pScrolledWindow), pOneWidget);
 	
@@ -235,8 +240,8 @@ gboolean cairo_dock_load_theme (GKeyFile* pKeyFile, GFunc pCallback, GtkWidget *
 		gtk_window_set_skip_pager_hint (GTK_WINDOW (pWaitingDialog), TRUE);
 		gtk_window_set_transient_for (GTK_WINDOW (pWaitingDialog), GTK_WINDOW (pMainWindow));
 		gtk_window_set_modal (GTK_WINDOW (pWaitingDialog), TRUE);
-		
-		GtkWidget *pMainVBox = gtk_vbox_new (FALSE, CAIRO_DOCK_FRAME_MARGIN);
+
+		GtkWidget *pMainVBox = _gtk_vbox_new (CAIRO_DOCK_FRAME_MARGIN);
 		gtk_container_add (GTK_CONTAINER (pWaitingDialog), pMainVBox);
 		
 		GtkWidget *pLabel = gtk_label_new (_("Please wait while importing the theme..."));
@@ -276,7 +281,7 @@ gboolean cairo_dock_load_theme (GKeyFile* pKeyFile, GFunc pCallback, GtkWidget *
 }
 
 
-gboolean cairo_dock_add_module_to_modele (gchar *cModuleName, CairoDockModule *pModule, GtkListStore *pModele)
+gboolean cairo_dock_add_module_to_modele (gchar *cModuleName, CairoDockModule *pModule, GtkListStore *pModel)
 {
 	if (pModule->pVisitCard->iCategory != CAIRO_DOCK_CATEGORY_BEHAVIOR && pModule->pVisitCard->iCategory != CAIRO_DOCK_CATEGORY_THEME && ! cairo_dock_module_is_auto_loaded (pModule))
 	{
@@ -284,8 +289,8 @@ gboolean cairo_dock_add_module_to_modele (gchar *cModuleName, CairoDockModule *p
 		GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file_at_size (pModule->pVisitCard->cIconFilePath, 32, 32, NULL);
 		GtkTreeIter iter;
 		memset (&iter, 0, sizeof (GtkTreeIter));
-		gtk_list_store_append (GTK_LIST_STORE (pModele), &iter);
-		gtk_list_store_set (GTK_LIST_STORE (pModele), &iter,
+		gtk_list_store_append (GTK_LIST_STORE (pModel), &iter);
+		gtk_list_store_set (GTK_LIST_STORE (pModel), &iter,
 			CAIRO_DOCK_MODEL_NAME, pModule->pVisitCard->cTitle,
 			CAIRO_DOCK_MODEL_RESULT, cModuleName,
 			CAIRO_DOCK_MODEL_DESCRIPTION_FILE, dgettext (pModule->pVisitCard->cGettextDomain, pModule->pVisitCard->cDescription),
@@ -336,7 +341,7 @@ static void _cairo_dock_initiate_config_module (GtkMenuItem *pMenuItem, CairoDoc
 	else
 		cairo_dock_show_module_gui (pModule->pVisitCard->cModuleName);
 }
-static void _on_click_module_tree_view (GtkTreeView *pTreeView, GdkEventButton* pButton, gpointer data)
+static gboolean _on_click_module_tree_view (GtkTreeView *pTreeView, GdkEventButton* pButton, gpointer data)
 {
 	if ((pButton->button == 3 && pButton->type == GDK_BUTTON_RELEASE)  // right click
 	|| (pButton->button == 1 && pButton->type == GDK_2BUTTON_PRESS))  // double click
@@ -345,23 +350,23 @@ static void _on_click_module_tree_view (GtkTreeView *pTreeView, GdkEventButton*
 		GtkTreeModel *pModel;
 		GtkTreeIter iter;
 		if (! gtk_tree_selection_get_selected (pSelection, &pModel, &iter))
-			return ;
+			return FALSE;
 		
 		gchar *cModuleName = NULL;
 		gtk_tree_model_get (pModel, &iter,
 			CAIRO_DOCK_MODEL_RESULT, &cModuleName, -1);
 		CairoDockModule *pModule = cairo_dock_find_module_from_name (cModuleName);
 		if (pModule == NULL)
-			return ;
+			return FALSE;
 		
 		if (pModule->pInstancesList == NULL)  // on ne gere pas la config d'un module no nactif, donc inutile de presenter le menu dans ce cas-la.
-			return ;
+			return FALSE;
 		
 		if (pButton->button == 3)
 		{
 			GtkWidget *pMenu = gtk_menu_new ();
 			
-			cairo_dock_add_in_menu_with_stock_and_data (_("Configure this applet"), GTK_STOCK_PROPERTIES, (GFunc)_cairo_dock_initiate_config_module, pMenu, pModule);
+			cairo_dock_add_in_menu_with_stock_and_data (_("Configure this applet"), GTK_STOCK_PROPERTIES, G_CALLBACK (_cairo_dock_initiate_config_module), pMenu, pModule);
 			
 			gtk_widget_show_all (pMenu);
 			gtk_menu_popup (GTK_MENU (pMenu),
@@ -377,7 +382,7 @@ static void _on_click_module_tree_view (GtkTreeView *pTreeView, GdkEventButton*
 			_cairo_dock_initiate_config_module (NULL, pModule);
 		}
 	}
-	return ;
+	return FALSE;
 }
 static void _cairo_dock_render_module_name (GtkTreeViewColumn *tree_column, GtkCellRenderer *cell, GtkTreeModel *model,GtkTreeIter *iter, gpointer data)
 {
@@ -443,6 +448,7 @@ GtkWidget *cairo_dock_build_modules_treeview (void)
 	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (pOneWidget), TRUE);
 	gtk_tree_view_set_headers_clickable (GTK_TREE_VIEW (pOneWidget), TRUE);
 	g_signal_connect (G_OBJECT (pOneWidget), "button-release-event", G_CALLBACK (_on_click_module_tree_view), NULL);  // pour le menu du clic droit
+	g_signal_connect (G_OBJECT (pOneWidget), "button-press-event", G_CALLBACK (_on_click_module_tree_view), NULL);  // pour le menu du clic droit
 	
 	GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (pOneWidget));
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
@@ -480,6 +486,215 @@ GtkWidget *cairo_dock_build_modules_treeview (void)
 }
 
 
+typedef enum {
+	CD_SHORTKEY_MODEL_NAME = 0,  // demander
+	CD_SHORTKEY_MODEL_DESCRIPTION,  // description
+	CD_SHORTKEY_MODEL_PIXBUF,  // icon image
+	CD_SHORTKEY_MODEL_SHORTKEY,  // shortkey
+	CD_SHORTKEY_MODEL_STATE,  // grabbed or not
+	CD_SHORTKEY_MODEL_BINDING,  // the binding
+	CD_SHORTKEY_MODEL_NB_COLUMNS
+	} CDModelColumns;
+static void _on_key_grab_cb (GtkWidget *pInputDialog, GdkEventKey *event, GtkTreeView *pTreeView)
+{
+	gchar *key;
+	cd_debug ("key pressed");
+	if (gtk_accelerator_valid (event->keyval, event->state))
+	{
+		/* This lets us ignore all ignorable modifier keys, including
+		* NumLock and many others. :)
+		*
+		* The logic is: keep only the important modifiers that were pressed
+		* for this event. */
+		event->state &= gtk_accelerator_get_default_mod_mask();
+
+		/* Generate the correct name for this key */
+		key = gtk_accelerator_name (event->keyval, event->state);
+
+		cd_debug ("KEY GRABBED: '%s'", key);
+
+		// get the key binding
+		GtkTreeSelection *pSelection = gtk_tree_view_get_selection (pTreeView);
+		GtkTreeModel *pModel;
+		GtkTreeIter iter;
+		if (gtk_tree_selection_get_selected (pSelection, &pModel, &iter))  // we retrieve it from the current selection, this way if the binding has been destroyed meanwhile (very unlikely), we're safe.
+		{
+			// Bind the new shortkey
+			CairoKeyBinding *binding = NULL;
+			gtk_tree_model_get (pModel, &iter,
+				CD_SHORTKEY_MODEL_BINDING, &binding, -1);
+			
+			cd_keybinder_rebind (binding, key, NULL);
+			
+			// update the model
+			gtk_list_store_set (GTK_LIST_STORE (pModel), &iter,
+				CD_SHORTKEY_MODEL_SHORTKEY, binding->keystring,
+				CD_SHORTKEY_MODEL_STATE, binding->bSuccess, -1);
+			
+			// store the new shortkey
+			if (binding->cConfFilePath != NULL)
+			{
+				cairo_dock_update_conf_file (binding->cConfFilePath,
+					G_TYPE_STRING, binding->cGroupName, binding->cKeyName, key,
+					G_TYPE_INVALID);
+			}
+		}
+			
+		// Close the window
+		g_free (key);
+		gtk_widget_destroy (pInputDialog);
+	}
+}
+static void on_cancel_shortkey (GtkButton *button, GtkWidget *pInputDialog)
+{
+	gtk_widget_destroy (pInputDialog);
+}
+static void _cairo_dock_initiate_change_shortkey (GtkMenuItem *pMenuItem, GtkTreeView *pTreeView)
+{
+	// ensure a line is selected
+	GtkTreeSelection *pSelection = gtk_tree_view_get_selection (pTreeView);
+	GtkTreeModel *pModel;
+	GtkTreeIter iter;
+	if (! gtk_tree_selection_get_selected (pSelection, &pModel, &iter))
+		return ;
+
+	// build a small modal input dialog, mainly to prevent any other interaction.
+	GtkWidget *pInputDialog = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+	gtk_window_set_decorated (GTK_WINDOW (pInputDialog), FALSE);
+	gtk_window_set_skip_taskbar_hint (GTK_WINDOW (pInputDialog), TRUE);
+	gtk_window_set_skip_pager_hint (GTK_WINDOW (pInputDialog), TRUE);
+	//gtk_window_set_transient_for (GTK_WINDOW (pInputDialog), GTK_WINDOW (pMainWindow));
+	gtk_window_set_modal (GTK_WINDOW (pInputDialog), TRUE);
+
+	gtk_widget_add_events (pInputDialog, GDK_KEY_PRESS_MASK);
+	g_signal_connect (GTK_WIDGET(pInputDialog), "key-press-event", G_CALLBACK(_on_key_grab_cb), pTreeView);
+
+	GtkWidget *pMainVBox = _gtk_vbox_new (CAIRO_DOCK_FRAME_MARGIN);
+	gtk_container_add (GTK_CONTAINER (pInputDialog), pMainVBox);
+
+	GtkWidget *pLabel = gtk_label_new (_("Press the shortkey"));
+	gtk_box_pack_start(GTK_BOX (pMainVBox), pLabel, FALSE, FALSE, 0);
+
+	GtkWidget *pCancelButton = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
+	g_signal_connect (G_OBJECT (pCancelButton), "clicked", G_CALLBACK(on_cancel_shortkey), pInputDialog);
+	gtk_box_pack_start (GTK_BOX (pMainVBox), pCancelButton, FALSE, FALSE, 0);
+
+	gtk_widget_show_all (pInputDialog);
+}
+static gboolean _on_click_shortkey_tree_view (GtkTreeView *pTreeView, GdkEventButton* pButton, gpointer data)
+{
+	GtkWidget *pMainWindow = NULL;
+	if ((pButton->button == 3 && pButton->type == GDK_BUTTON_RELEASE)  // right click
+	|| (pButton->button == 1 && pButton->type == GDK_2BUTTON_PRESS))  // double click
+	{
+		if (pButton->button == 3)
+		{
+			GtkWidget *pMenu = gtk_menu_new ();
+			
+			cairo_dock_add_in_menu_with_stock_and_data (_("Change the shortkey"), GTK_STOCK_PROPERTIES, G_CALLBACK (_cairo_dock_initiate_change_shortkey), pMenu, pTreeView);
+			
+			gtk_widget_show_all (pMenu);
+			gtk_menu_popup (GTK_MENU (pMenu),
+				NULL,
+				NULL,
+				NULL,
+				NULL,
+				1,
+				gtk_get_current_event_time ());
+		}
+		else
+		{
+			_cairo_dock_initiate_change_shortkey (NULL, pTreeView);
+		}
+	}
+	return FALSE;
+}
+static void _cairo_dock_render_shortkey (GtkTreeViewColumn *tree_column, GtkCellRenderer *cell, GtkTreeModel *model,GtkTreeIter *iter, gpointer data)
+{
+	gboolean bActive = FALSE;
+	gchar *cShortkey = NULL;
+	gtk_tree_model_get (model, iter,
+		CD_SHORTKEY_MODEL_STATE, &bActive,
+		CD_SHORTKEY_MODEL_SHORTKEY, &cShortkey, -1);
+	
+	if (bActive)
+	{
+		g_object_set (cell, "foreground", "#108000", NULL);  // vert
+		g_object_set (cell, "foreground-set", TRUE, NULL);
+	}
+	else if (cShortkey != NULL)
+	{
+		g_object_set (cell, "foreground", "#B00000", NULL);  // rouge
+		g_object_set (cell, "foreground-set", TRUE, NULL);
+	}
+	else
+	{
+		g_object_set (cell, "foreground-set", FALSE, NULL);
+	}
+	g_free (cShortkey);
+}
+void cairo_dock_add_shortkey_to_model (CairoKeyBinding *binding, GtkListStore *pModel)
+{
+	//g_print (" + %s\n",  pModule->pVisitCard->cIconFilePath);
+	GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file_at_size (binding->cIconFilePath, 32, 32, NULL);
+	GtkTreeIter iter;
+	memset (&iter, 0, sizeof (GtkTreeIter));
+	gtk_list_store_append (GTK_LIST_STORE (pModel), &iter);
+	cd_debug (" + %s (%s, %d)", binding->keystring, binding->cDemander, binding->bSuccess);
+	gtk_list_store_set (GTK_LIST_STORE (pModel), &iter,
+		CD_SHORTKEY_MODEL_NAME, binding->cDemander,
+		CD_SHORTKEY_MODEL_SHORTKEY, binding->keystring,
+		CD_SHORTKEY_MODEL_DESCRIPTION, binding->cDescription,
+		CD_SHORTKEY_MODEL_PIXBUF, pixbuf,
+		CD_SHORTKEY_MODEL_STATE, binding->bSuccess,
+		CD_SHORTKEY_MODEL_BINDING, binding, -1);
+	g_object_unref (pixbuf);
+}
+GtkWidget *cairo_dock_build_shortkeys_widget (void)
+{
+	// fill the model
+	GtkListStore *pModel = gtk_list_store_new (CD_SHORTKEY_MODEL_NB_COLUMNS,
+		G_TYPE_STRING,  // demander
+		G_TYPE_STRING,  // description
+		GDK_TYPE_PIXBUF,  // icon
+		G_TYPE_STRING,  // shortkey
+		G_TYPE_BOOLEAN,  // grabbed or not
+		G_TYPE_POINTER);  // binding
+	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (pModel), CD_SHORTKEY_MODEL_NAME, GTK_SORT_ASCENDING);
+	cd_keybinder_foreach ((GFunc) cairo_dock_add_shortkey_to_model, pModel);
+	
+	// make the treeview
+	GtkWidget *pOneWidget = gtk_tree_view_new_with_model (GTK_TREE_MODEL (pModel));
+	g_object_unref (pModel);
+	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (pOneWidget), TRUE);
+	g_signal_connect (G_OBJECT (pOneWidget), "button-press-event", G_CALLBACK (_on_click_shortkey_tree_view), NULL);
+	g_signal_connect (G_OBJECT (pOneWidget), "button-release-event", G_CALLBACK (_on_click_shortkey_tree_view), NULL);
+	
+	// define the rendering of the treeview
+	GtkTreeViewColumn* col;
+	GtkCellRenderer *rend;
+	// icon
+	rend = gtk_cell_renderer_pixbuf_new ();
+	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (pOneWidget), -1, NULL, rend, "pixbuf", CD_SHORTKEY_MODEL_PIXBUF, NULL);
+	// demander
+	rend = gtk_cell_renderer_text_new ();
+	col = gtk_tree_view_column_new_with_attributes (_("Origin"), rend, "text", CD_SHORTKEY_MODEL_NAME, NULL);
+	gtk_tree_view_column_set_sort_column_id (col, CD_SHORTKEY_MODEL_NAME);
+	gtk_tree_view_append_column (GTK_TREE_VIEW (pOneWidget), col);
+	// action description
+	rend = gtk_cell_renderer_text_new ();
+	col = gtk_tree_view_column_new_with_attributes (_("Action"), rend, "text", CD_SHORTKEY_MODEL_DESCRIPTION, NULL);
+	gtk_tree_view_append_column (GTK_TREE_VIEW (pOneWidget), col);
+	// shortkey
+	rend = gtk_cell_renderer_text_new ();
+	col = gtk_tree_view_column_new_with_attributes (_("Shortkey"), rend, "text", CD_SHORTKEY_MODEL_SHORTKEY, NULL);
+	gtk_tree_view_column_set_cell_data_func (col, rend, (GtkTreeCellDataFunc)_cairo_dock_render_shortkey, NULL, NULL);
+	gtk_tree_view_column_set_sort_column_id (col, CD_SHORTKEY_MODEL_SHORTKEY);
+	gtk_tree_view_append_column (GTK_TREE_VIEW (pOneWidget), col);
+	
+	return pOneWidget;
+}
+
 void cairo_dock_update_desklet_widgets (CairoDesklet *pDesklet, GSList *pWidgetList)
 {
 	CairoDockGroupKeyWidget *pGroupKeyWidget;
@@ -562,3 +777,9 @@ void cairo_dock_update_is_detached_widget (gboolean bIsDetached, GSList *pWidget
 	pOneWidget = pGroupKeyWidget->pSubWidgetList->data;
 	gtk_toggle_button_set_active  (GTK_TOGGLE_BUTTON (pOneWidget), bIsDetached);
 }
+
+
+gchar *cairo_dock_get_third_party_applets_link (void)
+{
+	return g_strdup_printf (CAIRO_DOCK_PLUGINS_EXTRAS_URL"/%d.%d.%d", g_iMajorVersion, g_iMinorVersion, g_iMicroVersion);
+}
