@@ -25,6 +25,7 @@
 #include <signal.h>
 
 #include <cairo.h>
+#include <gdk/gdkx.h>
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
 #include <X11/Xutil.h>
@@ -61,6 +62,7 @@ static Atom s_aNetActiveWindow;
 static Atom s_aNetCurrentDesktop;
 static Atom s_aNetDesktopViewport;
 static Atom s_aNetDesktopGeometry;
+static Atom s_aNetWorkarea;
 static Atom s_aNetShowingDesktop;
 static Atom s_aRootMapID;
 static Atom s_aNetNbDesktops;
@@ -103,16 +105,19 @@ static void _on_change_nb_desktops (void)
 
 static void _on_change_desktop_geometry (void)
 {
-	if (cairo_dock_update_screen_geometry ())  // modification de la resolution.
+	// check if the resolution has changed
+	if (cairo_dock_update_screen_geometry ())  // resolution has changed => replace the docks.
 	{
 		cd_message ("resolution alteree");
 		
 		cairo_dock_reposition_root_docks (FALSE);  // main dock compris. Se charge de Xinerama.
 	}
 	
+	// check if the number of viewports has changed.
 	cairo_dock_get_nb_viewports (&g_desktopGeometry.iNbViewportX, &g_desktopGeometry.iNbViewportY);
 	_cairo_dock_retrieve_current_desktop_and_viewport ();  // au cas ou on enleve le viewport courant.
 	
+	// notify everybody
 	cairo_dock_notify_on_object (&myDesktopMgr, NOTIFICATION_SCREEN_GEOMETRY_ALTERED);
 }
 
@@ -130,7 +135,7 @@ static gboolean _cairo_dock_unstack_Xevents (gpointer data)
 	while (XCheckMaskEvent (s_XDisplay, event_mask, &event))
 	{
 		Xid = event.xany.window;
-		//g_print ("  type : %d; atom : %s; window : %d\n", event.type, gdk_x11_get_xatom_name (event.xproperty.atom), Xid);
+		//g_print ("  type : %d; atom : %s; window : %d\n", event.type, XGetAtomName (s_XDisplay, event.xproperty.atom), Xid);
 		//if (event.type == ClientMessage)
 		//cd_message ("\n\n\n >>>>>>>>>>>< event.type : %d\n\n", event.type);
 		if (Xid == root)
@@ -154,7 +159,7 @@ static gboolean _cairo_dock_unstack_Xevents (gpointer data)
 				{
 					_on_change_nb_desktops ();  // -> NOTIFICATION_SCREEN_GEOMETRY_ALTERED
 				}
-				else if (event.xproperty.atom == s_aNetDesktopGeometry)
+				else if (event.xproperty.atom == s_aNetDesktopGeometry || event.xproperty.atom == s_aNetWorkarea)  // check s_aNetWorkarea too, to workaround a bug in Compiz (or X?) : when down-sizing the screen, the _NET_DESKTOP_GEOMETRY atom is not received  (up-sizing is ok though, and changing the viewport makes the atom to be received); but _NET_WORKAREA is correctly sent; since it's only sent when the resolution is changed, or the dock's height (if space is reserved), it's not a big overload to check it too.
 				{
 					_on_change_desktop_geometry ();  // -> NOTIFICATION_SCREEN_GEOMETRY_ALTERED
 				}
@@ -213,21 +218,6 @@ static gboolean _cairo_dock_unstack_Xevents (gpointer data)
 	return TRUE;
 }
 
-static void cairo_dock_initialize_X_manager (Display *pDisplay)
-{
-	s_XDisplay = pDisplay;
-
-	s_aNetClientList		= XInternAtom (s_XDisplay, "_NET_CLIENT_LIST_STACKING", False);
-	s_aNetActiveWindow		= XInternAtom (s_XDisplay, "_NET_ACTIVE_WINDOW", False);
-	s_aNetCurrentDesktop	= XInternAtom (s_XDisplay, "_NET_CURRENT_DESKTOP", False);
-	s_aNetDesktopViewport	= XInternAtom (s_XDisplay, "_NET_DESKTOP_VIEWPORT", False);
-	s_aNetDesktopGeometry	= XInternAtom (s_XDisplay, "_NET_DESKTOP_GEOMETRY", False);
-	s_aNetShowingDesktop 	= XInternAtom (s_XDisplay, "_NET_SHOWING_DESKTOP", False);
-	s_aRootMapID			= XInternAtom (s_XDisplay, "_XROOTPMAP_ID", False);
-	s_aNetNbDesktops		= XInternAtom (s_XDisplay, "_NET_NUMBER_OF_DESKTOPS", False);
-	s_aXKlavierState		= XInternAtom (s_XDisplay, "XKLAVIER_STATE", False);
-}
-
 
   ////////////////////////
  /// X desktop access ///
@@ -247,7 +237,7 @@ void cairo_dock_get_current_desktop_and_viewport (int *iCurrentDesktop, int *iCu
 
 static gboolean _set_desklets_on_widget_layer (CairoDesklet *pDesklet, gpointer data)
 {
-	Window Xid = GDK_WINDOW_XID (pDesklet->container.pWidget->window);
+	Window Xid = gldi_container_get_Xid (CAIRO_CONTAINER (pDesklet));
 	if (pDesklet->iVisibility == CAIRO_DESKLET_ON_WIDGET_LAYER)
 		cairo_dock_wm_set_on_widget_layer (Xid, TRUE);
 	return FALSE;  // continue
@@ -340,12 +330,11 @@ gboolean cairo_dock_wm_can_set_on_widget_layer (void)
   //////////////////
  /// DESKTOP BG ///
 //////////////////
-
 static cairo_surface_t *_cairo_dock_create_surface_from_desktop_bg (void)  // attention : fonction lourde.
 {
 	//g_print ("+++ %s ()\n", __func__);
 	Pixmap iRootPixmapID = cairo_dock_get_window_background_pixmap (cairo_dock_get_root_id ());
-	g_return_val_if_fail (iRootPixmapID != 0, NULL);
+	g_return_val_if_fail (iRootPixmapID != 0, NULL);  // Note: depending on the WM, iRootPixmapID might be 0, and a window of type 'Desktop' might be used instead (covering the whole screen). We don't handle this case, as I've never encounterd it yet.
 	
 	cairo_surface_t *pDesktopBgSurface = NULL;
 	GdkPixbuf *pBgPixbuf = cairo_dock_get_pixbuf_from_pixmap (iRootPixmapID, FALSE);  // FALSE <=> on n'y ajoute pas de transparence.
@@ -540,8 +529,9 @@ static void init (void)
 	s_aNetCurrentDesktop	= XInternAtom (s_XDisplay, "_NET_CURRENT_DESKTOP", False);
 	s_aNetDesktopViewport	= XInternAtom (s_XDisplay, "_NET_DESKTOP_VIEWPORT", False);
 	s_aNetDesktopGeometry	= XInternAtom (s_XDisplay, "_NET_DESKTOP_GEOMETRY", False);
+	s_aNetWorkarea			= XInternAtom (s_XDisplay, "_NET_WORKAREA", False);
 	s_aNetShowingDesktop 	= XInternAtom (s_XDisplay, "_NET_SHOWING_DESKTOP", False);
-	s_aRootMapID			= XInternAtom (s_XDisplay, "_XROOTPMAP_ID", False);
+	s_aRootMapID			= XInternAtom (s_XDisplay, "_XROOTPMAP_ID", False);  // Note: ESETROOT_PMAP_ID might be used instead. We don't handle it as it seems quite rare and somewhat deprecated.
 	s_aNetNbDesktops		= XInternAtom (s_XDisplay, "_NET_NUMBER_OF_DESKTOPS", False);
 	s_aXKlavierState		= XInternAtom (s_XDisplay, "XKLAVIER_STATE", False);
 	
