@@ -41,6 +41,10 @@
 #include "cairo-dock-X-manager.h"
 #include "cairo-dock-X-utilities.h"
 
+#if (GTK_MAJOR_VERSION >= 3)
+#include <cairo/cairo-xlib.h>  // needed for cairo_xlib_surface_create
+#endif
+
 extern CairoDockDesktopGeometry g_desktopGeometry;
 
 static gboolean s_bUseXComposite = TRUE;
@@ -160,6 +164,7 @@ gboolean cairo_dock_update_screen_geometry (void)
 		&x_return, &y_return,
 		&width_return, &height_return,
 		&border_width_return, &depth_return);
+	cd_debug (">>>>>   screen resolution: %dx%d -> %dx%d", g_desktopGeometry.iXScreenWidth[CAIRO_DOCK_HORIZONTAL], g_desktopGeometry.iXScreenHeight[CAIRO_DOCK_HORIZONTAL], width_return, height_return);
 	if ((int)width_return != g_desktopGeometry.iXScreenWidth[CAIRO_DOCK_HORIZONTAL] || (int)height_return != g_desktopGeometry.iXScreenHeight[CAIRO_DOCK_HORIZONTAL])  // on n'utilise pas WidthOfScreen() et HeightOfScreen() car leurs valeurs ne sont pas mises a jour immediatement apres les changements de resolution.
 	{
 		g_desktopGeometry.iXScreenWidth[CAIRO_DOCK_HORIZONTAL] = width_return;
@@ -426,6 +431,7 @@ GdkPixbuf *cairo_dock_get_pixbuf_from_pixmap (int XPixmapID, gboolean bAddAlpha)
 	//g_print ("%s (%d) : %ux%ux%u (%d;%d)\n", __func__, XPixmapID, iWidth, iHeight, iDepth, x, y);
 
 	//\__________________ On recupere le drawable associe.
+	#if (GTK_MAJOR_VERSION < 3)
 	GdkDrawable *pGdkDrawable = gdk_xid_table_lookup (XPixmapID);
 	if (pGdkDrawable)
 	{
@@ -461,6 +467,19 @@ GdkPixbuf *cairo_dock_get_pixbuf_from_pixmap (int XPixmapID, gboolean bAddAlpha)
 		iWidth,
 		iHeight);
 	g_object_unref (G_OBJECT (pGdkDrawable));
+	#else
+	cairo_surface_t *surface = cairo_xlib_surface_create (s_XDisplay,
+		XPixmapID,
+		DefaultVisual(s_XDisplay, 0),
+		iWidth,
+		iHeight);
+	GdkPixbuf *pIconPixbuf = gdk_pixbuf_get_from_surface(surface,
+		0,
+		0,
+		iWidth,
+		iHeight);
+	cairo_surface_destroy(surface);
+	#endif
 	g_return_val_if_fail (pIconPixbuf != NULL, NULL);
 
 	//\__________________ On lui ajoute un canal alpha si necessaire.
@@ -792,6 +811,12 @@ void cairo_dock_minimize_xwindow (Window Xid)
 	//cairo_dock_set_xwindow_timestamp (Xid, cairo_dock_get_xwindow_timestamp (root));
 }
 
+void cairo_dock_lower_xwindow (Window Xid)
+{
+	g_return_if_fail (Xid > 0);
+	XLowerWindow (s_XDisplay, Xid);
+}
+
 
 
 static void _cairo_dock_change_window_state (Window Xid, gulong iNewValue, Atom iProperty1, Atom iProperty2)
@@ -929,6 +954,75 @@ gchar *cairo_dock_get_xwindow_name (Window Xid, gboolean bSearchWmName)
 	return cName;
 }
 
+gboolean cairo_dock_remove_version_from_string (gchar *cString)
+{
+	if (cString == NULL)
+		return FALSE;
+	int n = strlen (cString);
+	gchar *str = cString + n - 1;
+	do
+	{
+		if (g_ascii_isdigit(*str) || *str == '.')
+		{
+			str --;
+			continue;
+		}
+		if (*str == '-' || *str == ' ')  // 'Glade-2', 'OpenOffice 3.1'
+		{
+			*str = '\0';
+			return TRUE;
+		}
+		else
+			return FALSE;
+	}
+	while (str != cString);
+	return FALSE;
+}
+
+gchar *cairo_dock_get_xwindow_class (Window Xid, gchar **cWMClass)
+{
+	XClassHint *pClassHint = XAllocClassHint ();
+	gchar *cClass = NULL, *cWmClass = NULL;
+	if (XGetClassHint (s_XDisplay, Xid, pClassHint) != 0 && pClassHint->res_class)
+	{
+		cWmClass = g_strdup (pClassHint->res_class);
+		
+		cd_debug ("  res_name : %s(%x); res_class : %s(%x)", pClassHint->res_name, pClassHint->res_name, pClassHint->res_class, pClassHint->res_class);
+		if (strcmp (pClassHint->res_class, "Wine") == 0 && pClassHint->res_name && g_str_has_suffix (pClassHint->res_name, ".exe"))
+		{
+			cd_debug ("  wine application detected, changing the class '%s' to '%s'", pClassHint->res_class, pClassHint->res_name);
+			cClass = g_ascii_strdown (pClassHint->res_name, -1);
+		}
+		else if (*pClassHint->res_class == '/' && g_str_has_suffix (pClassHint->res_class, ".exe"))  // cas des applications Mono telles que tomboy ...
+		{
+			gchar *str = strrchr (pClassHint->res_class, '/');
+			if (str)
+				str ++;
+			else
+				str = pClassHint->res_class;
+			cClass = g_ascii_strdown (str, -1);
+			cClass[strlen (cClass) - 4] = '\0';
+		}
+		else
+		{
+			cClass = g_ascii_strdown (pClassHint->res_class, -1);  // on la passe en minuscule, car certaines applis ont la bonne idee de donner des classes avec une majuscule ou non suivant les fenetres.
+		}
+		
+		cairo_dock_remove_version_from_string (cClass);  // on enleve les numeros de version (Openoffice.org-3.1)
+		
+		gchar *str = strchr (cClass, '.');  // on vire les .xxx, sinon on ne sait pas detecter l'absence d'extension quand on cherche l'icone (openoffice.org), ou tout simplement ca empeche de trouver l'icone (jbrout.py).
+		if (str != NULL)
+			*str = '\0';
+		cd_debug ("got an application with class '%s'", cClass);
+		
+		XFree (pClassHint->res_name);
+		XFree (pClassHint->res_class);
+		XFree (pClassHint);
+	}
+	*cWMClass = cWmClass;
+	return cClass;
+}
+
 gboolean cairo_dock_xwindow_is_maximized (Window Xid)
 {
 	g_return_val_if_fail (Xid > 0, FALSE);
@@ -1131,6 +1225,7 @@ int cairo_dock_get_xwindow_desktop (Window Xid)
 
 void cairo_dock_get_xwindow_geometry (Window Xid, int *iLocalPositionX, int *iLocalPositionY, int *iWidthExtent, int *iHeightExtent)  // renvoie les coordonnees du coin haut gauche dans le referentiel du viewport actuel. // sous KDE, x et y sont toujours nuls ! (meme avec XGetWindowAttributes).
 {
+	// get the geometry from X.
 	Window root_return;
 	int x_return=1, y_return=1;
 	unsigned int width_return, height_return, border_width_return, depth_return;
@@ -1140,11 +1235,31 @@ void cairo_dock_get_xwindow_geometry (Window Xid, int *iLocalPositionX, int *iLo
 		&width_return, &height_return,
 		&border_width_return, &depth_return);  // renvoie les coordonnees du coin haut gauche dans le referentiel du viewport actuel.
 	
-	*iLocalPositionX = x_return;  // on pourrait tenir compte de border_width_return...
-	*iLocalPositionY = y_return;  // idem.
-	*iWidthExtent = width_return;  // idem.
-	*iHeightExtent = height_return;  // idem.
-	//g_print ("%s () -> %d;%d %dx%d / %d,%d\n", __func__, x_return, y_return, *iWidthExtent, *iHeightExtent, border_width_return, depth_return);
+	// make another round trip to the server to query the coordinates of the window relatively to the root window (which basically gives us the (x,y) of the window); we need to do this to workaround a strange X bug: x_return and y_return are wrong (0,0, modulo the borders) (on Ubuntu 11.10 + Compiz 0.9/Metacity, not on Debian 6 + Compiz 0.8).
+	int dest_x_return, dest_y_return;
+	Window child_return;
+	XTranslateCoordinates (s_XDisplay, Xid, root_return, 0, 0, &dest_x_return, &dest_y_return, &child_return);  // translate into the coordinate space of the root window. we need to do this, because (x_return,;y_return) is always (0;0)
+	//g_print (" %d;%d %dx%d\n", x_return, y_return, width_return, height_return);
+	//g_print (" -> %d;%d\n", dest_x_return, dest_y_return);
+	
+	// take into account the window borders
+	int left=0, right=0, top=0, bottom=0;
+	gulong iLeftBytes, iBufferNbElements = 0;
+	Atom aReturnedType = 0;
+	int aReturnedFormat = 0;
+	gulong *pBuffer = NULL;
+	XGetWindowProperty (s_XDisplay, Xid, XInternAtom (s_XDisplay, "_NET_FRAME_EXTENTS", False), 0, G_MAXULONG, False, XA_CARDINAL, &aReturnedType, &aReturnedFormat, &iBufferNbElements, &iLeftBytes, (guchar **)&pBuffer);
+	if (iBufferNbElements > 3)
+	{
+		left=pBuffer[0], right=pBuffer[1], top=pBuffer[2], bottom=pBuffer[3];
+	}
+	if (pBuffer)
+		XFree (pBuffer);
+	
+	*iLocalPositionX = dest_x_return - left;
+	*iLocalPositionY = dest_y_return - top;
+	*iWidthExtent = width_return + left + right;  // unfortunately border_width_return is always 0, so we can't use it here :-/
+	*iHeightExtent = height_return + top + bottom;
 }
 
 void cairo_dock_get_xwindow_position_on_its_viewport (Window Xid, int *iRelativePositionX, int *iRelativePositionY)
@@ -1166,7 +1281,7 @@ void cairo_dock_get_xwindow_position_on_its_viewport (Window Xid, int *iRelative
 	//cd_debug ("position relative : (%d;%d) taille : %dx%d", *iRelativePositionX, *iRelativePositionY, iWidthExtent, iHeightExtent);
 }
 
-gboolean cairo_dock_xwindow_is_on_current_desktop (Window Xid)
+/**gboolean cairo_dock_xwindow_is_on_current_desktop (Window Xid)
 {
 	int iWindowDesktopNumber, iLocalPositionX, iLocalPositionY, iWidthExtent, iHeightExtent;  // coordonnees du coin haut gauche dans le referentiel du viewport actuel.
 	iWindowDesktopNumber = cairo_dock_get_xwindow_desktop (Xid);
@@ -1178,7 +1293,7 @@ gboolean cairo_dock_xwindow_is_on_current_desktop (Window Xid)
 		iLocalPositionX < g_desktopGeometry.iXScreenWidth[CAIRO_DOCK_HORIZONTAL] &&
 		iLocalPositionY + iHeightExtent > 0 &&
 		iLocalPositionY < g_desktopGeometry.iXScreenHeight[CAIRO_DOCK_HORIZONTAL] );  // -1 <=> 0xFFFFFFFF en unsigned.
-}
+}*/
 
 
 Window *cairo_dock_get_windows_list (gulong *iNbWindows, gboolean bStackOrder)
