@@ -88,6 +88,7 @@ void cairo_dock_close_xwindow (Window Xid);
 void cairo_dock_kill_xwindow (Window Xid);
 void cairo_dock_show_xwindow (Window Xid);
 void cairo_dock_minimize_xwindow (Window Xid);
+void cairo_dock_lower_xwindow (Window Xid);
 void cairo_dock_maximize_xwindow (Window Xid, gboolean bMaximize);
 void cairo_dock_set_xwindow_fullscreen (Window Xid, gboolean bFullScreen);
 void cairo_dock_set_xwindow_above (Window Xid, gboolean bAbove);
@@ -97,6 +98,14 @@ void cairo_dock_move_xwindow_to_absolute_position (Window Xid, int iDesktopNumbe
 // GET //
 gulong cairo_dock_get_xwindow_timestamp (Window Xid);
 gchar *cairo_dock_get_xwindow_name (Window Xid, gboolean bSearchWmName);
+
+/** Remove the version number from a string. Directly modifies the string.
+ * @param cString a string.
+ * @return TRUE if a version has been removed.
+ */
+gboolean cairo_dock_remove_version_from_string (gchar *cString);
+
+gchar *cairo_dock_get_xwindow_class (Window Xid, gchar **cWMClass);
 gboolean cairo_dock_xwindow_is_maximized (Window Xid);
 gboolean cairo_dock_xwindow_is_fullscreen (Window Xid);
 gboolean cairo_dock_xwindow_skip_taskbar (Window Xid);
@@ -109,7 +118,7 @@ int cairo_dock_get_xwindow_desktop (Window Xid);
 void cairo_dock_get_xwindow_geometry (Window Xid, int *iLocalPositionX, int *iLocalPositionY, int *iWidthExtent, int *iHeightExtent);
 void cairo_dock_get_xwindow_position_on_its_viewport (Window Xid, int *iRelativePositionX, int *iRelativePositionY);
 
-gboolean cairo_dock_xwindow_is_on_current_desktop (Window Xid);
+///gboolean cairo_dock_xwindow_is_on_current_desktop (Window Xid);
 
 Window *cairo_dock_get_windows_list (gulong *iNbWindows, gboolean bStackOrder);
 Window cairo_dock_get_active_xwindow (void);
