@@ -50,7 +50,7 @@ extern CairoDockImageBuffer g_pIconBackgroundBuffer;
 extern gboolean g_bUseOpenGL;
 
 
-void cairo_dock_set_icon_surface_full (cairo_t *pIconContext, cairo_surface_t *pSurface, double fScale, double fAlpha, Icon *pIcon, CairoContainer *pContainer)
+void cairo_dock_set_icon_surface_full (cairo_t *pIconContext, cairo_surface_t *pSurface, double fScale, double fAlpha, Icon *pIcon)
 {
 	g_return_if_fail (cairo_status (pIconContext) == CAIRO_STATUS_SUCCESS);
 	
@@ -110,9 +110,9 @@ void cairo_dock_set_icon_surface_full (cairo_t *pIconContext, cairo_surface_t *p
 
 void cairo_dock_set_icon_surface_with_reflect (cairo_t *pIconContext, cairo_surface_t *pSurface, Icon *pIcon, CairoContainer *pContainer)
 {
-	cairo_dock_set_icon_surface_full (pIconContext, pSurface, 1., 1., pIcon, pContainer);
+	cairo_dock_set_icon_surface (pIconContext, pSurface, pIcon);
 	
-	cairo_dock_add_reflection_to_icon (pIcon, pContainer);
+	///cairo_dock_add_reflection_to_icon (pIcon, pContainer);
 }
 
 
@@ -177,6 +177,8 @@ void cairo_dock_draw_bar_on_icon (cairo_t *pIconContext, double fValue, Icon *pI
 {
 	int iWidth = pIcon->iImageWidth;
 	int iHeight = pIcon->iImageHeight;
+	double fMaxScale = (pIcon->fHeight != 0 ? (pIcon->pContainer && pIcon->pContainer->bIsHorizontal ? pIcon->iImageHeight : pIcon->iImageWidth) / pIcon->fHeight : 1.);
+	double lw = 2 * fMaxScale;  // line demi-width
 	
 	cairo_pattern_t *pGradationPattern = cairo_pattern_create_linear (0.,
 		0.,
@@ -201,11 +203,11 @@ void cairo_dock_draw_bar_on_icon (cairo_t *pIconContext, double fValue, Icon *pI
 	cairo_save (pIconContext);
 	cairo_set_operator (pIconContext, CAIRO_OPERATOR_OVER);
 	
-	cairo_set_line_width (pIconContext, 6);
+	cairo_set_line_width (pIconContext, 2 * lw);
 	cairo_set_line_cap (pIconContext, CAIRO_LINE_CAP_ROUND);
 	
-	cairo_move_to (pIconContext, 3, iHeight - 3);
-	cairo_rel_line_to (pIconContext, (iWidth - 6) * fabs (fValue), 0);
+	cairo_move_to (pIconContext, lw, iHeight - lw);
+	cairo_rel_line_to (pIconContext, (iWidth - 2*lw) * fabs (fValue), 0);
 	
 	cairo_set_source (pIconContext, pGradationPattern);
 	cairo_stroke (pIconContext);
@@ -228,7 +230,7 @@ void cairo_dock_set_minutes_secondes_as_quick_info (Icon *pIcon, CairoContainer
 {
 	int minutes = iTimeInSeconds / 60;
 	int secondes = iTimeInSeconds % 60;
-	cd_debug ("%s (%d:%d)\n", __func__, minutes, secondes);
+	//cd_debug ("%s (%d:%d)\n", __func__, minutes, secondes);
 	if (minutes != 0)
 		cairo_dock_set_quick_info_printf (pIcon, pContainer, "%d:%02d", minutes, abs (secondes));
 	else
@@ -430,7 +432,7 @@ void cairo_dock_insert_icons_in_applet (CairoDockModuleInstance *pInstance, GLis
 				cairo_dock_set_icon_name (pInstance->pModule->pVisitCard->cModuleName, pIcon, pContainer);
 			if (cairo_dock_check_unique_subdock_name (pIcon))
 				cairo_dock_set_icon_name (pIcon->cName, pIcon, pContainer);
-			pIcon->pSubDock = cairo_dock_create_subdock_from_scratch (pIconsList, pIcon->cName, pInstance->pDock);
+			pIcon->pSubDock = cairo_dock_create_subdock (pIcon->cName, cDockRenderer, pInstance->pDock, pIconsList);
 			if (pIcon->pSubDock)
 				pIcon->pSubDock->bPreventDraggingIcons = TRUE;  // par defaut pour toutes les applets on empeche de pouvoir deplacer/supprimer les icones a la souris.
 		}
@@ -441,14 +443,15 @@ void cairo_dock_insert_icons_in_applet (CairoDockModuleInstance *pInstance, GLis
 			for (ic = pIconsList; ic != NULL; ic = ic->next)
 			{
 				pOneIcon = ic->data;
-				cairo_dock_insert_icon_in_dock (pOneIcon, pIcon->pSubDock, ! CAIRO_DOCK_UPDATE_DOCK_SIZE, ! CAIRO_DOCK_ANIMATE_ICON);
+				cairo_dock_insert_icon_in_dock (pOneIcon, pIcon->pSubDock, ! CAIRO_DOCK_ANIMATE_ICON);
 				pOneIcon->cParentDockName = g_strdup (pIcon->cName);
-				cairo_dock_trigger_load_icon_buffers (pOneIcon, CAIRO_CONTAINER (pIcon->pSubDock));
+				cairo_dock_trigger_load_icon_buffers (pOneIcon);
 			}
 			g_list_free (pIconsList);
+			
+			cairo_dock_set_renderer (pIcon->pSubDock, cDockRenderer);
+			cairo_dock_update_dock_size (pIcon->pSubDock);
 		}
-		cairo_dock_set_renderer (pIcon->pSubDock, cDockRenderer);
-		cairo_dock_update_dock_size (pIcon->pSubDock);
 		
 		if (pIcon->iSubdockViewType != 0)  // trigger the redraw after the icons are loaded.
 			cairo_dock_trigger_redraw_subdock_content_on_icon (pIcon);
@@ -460,6 +463,13 @@ void cairo_dock_insert_icons_in_applet (CairoDockModuleInstance *pInstance, GLis
 			cairo_dock_destroy_dock (pIcon->pSubDock, pIcon->cName);
 			pIcon->pSubDock = NULL;
 		}
+		Icon *pOneIcon;
+		GList *ic;
+		for (ic = pIconsList; ic != NULL; ic = ic->next)
+		{
+			pOneIcon = ic->data;
+			cairo_dock_set_icon_container (pOneIcon, pInstance->pDesklet);
+		}
 		pInstance->pDesklet->icons = g_list_concat (pInstance->pDesklet->icons, pIconsList);
 		cairo_dock_set_desklet_renderer_by_name (pInstance->pDesklet, cDeskletRenderer, (CairoDeskletRendererConfigPtr) pDeskletRendererData);
 		cairo_dock_redraw_container (pInstance->pContainer);
@@ -486,7 +496,7 @@ void cairo_dock_insert_icon_in_applet (CairoDockModuleInstance *pInstance, Icon
 				cairo_dock_set_icon_name (pInstance->pModule->pVisitCard->cModuleName, pIcon, pContainer);
 			if (cairo_dock_check_unique_subdock_name (pIcon))
 				cairo_dock_set_icon_name (pIcon->cName, pIcon, pContainer);
-			pIcon->pSubDock = cairo_dock_create_subdock_from_scratch (NULL, pIcon->cName, pInstance->pDock);
+			pIcon->pSubDock = cairo_dock_create_subdock (pIcon->cName, NULL, pInstance->pDock, NULL);
 			if (pIcon->pSubDock)
 				pIcon->pSubDock->bPreventDraggingIcons = TRUE;  // par defaut pour toutes les applets on empeche de pouvoir deplacer/supprimer les icones a la souris.
 		}
@@ -495,9 +505,11 @@ void cairo_dock_insert_icon_in_applet (CairoDockModuleInstance *pInstance, Icon
 			Icon *pLastIcon = cairo_dock_get_last_icon (pIcon->pSubDock->icons);
 			pOneIcon->fOrder = (pLastIcon ? pLastIcon->fOrder + 1 : 0);
 		}
-		cairo_dock_trigger_load_icon_buffers (pOneIcon, CAIRO_CONTAINER (pIcon->pSubDock));
-		cairo_dock_insert_icon_in_dock (pOneIcon, pIcon->pSubDock, CAIRO_DOCK_UPDATE_DOCK_SIZE, ! CAIRO_DOCK_ANIMATE_ICON);
+		cairo_dock_insert_icon_in_dock (pOneIcon, pIcon->pSubDock, ! CAIRO_DOCK_ANIMATE_ICON);
 		pOneIcon->cParentDockName = g_strdup (pIcon->cName);
+		cairo_dock_trigger_load_icon_buffers (pOneIcon);
+		if (pIcon->iSubdockViewType != 0)
+			cairo_dock_trigger_redraw_subdock_content_on_icon (pIcon);
 	}
 	else if (pInstance->pDesklet)
 	{
@@ -533,7 +545,7 @@ gboolean cairo_dock_detach_icon_from_applet (CairoDockModuleInstance *pInstance,
 	{
 		if (pIcon->pSubDock != NULL)
 		{
-			bRemoved = cairo_dock_detach_icon_from_dock (pOneIcon, pIcon->pSubDock, FALSE);
+			bRemoved = cairo_dock_detach_icon_from_dock_full (pOneIcon, pIcon->pSubDock, FALSE);
 			cairo_dock_update_dock_size (pIcon->pSubDock);
 		}
 	}
@@ -576,7 +588,6 @@ void cairo_dock_remove_all_icons_from_applet (CairoDockModuleInstance *pInstance
 			g_list_foreach (pIcon->pSubDock->icons, (GFunc) cairo_dock_free_icon, NULL);
 			g_list_free (pIcon->pSubDock->icons);
 			pIcon->pSubDock->icons = NULL;
-			pIcon->pSubDock->pFirstDrawnElement = NULL;
 		}
 		else  // precaution pas chere
 		{
@@ -588,7 +599,6 @@ void cairo_dock_remove_all_icons_from_applet (CairoDockModuleInstance *pInstance
 }
 
 
-
 void cairo_dock_resize_applet (CairoDockModuleInstance *pInstance, int w, int h)
 {
 	Icon *pIcon = pInstance->pIcon;
@@ -599,29 +609,16 @@ void cairo_dock_resize_applet (CairoDockModuleInstance *pInstance, int w, int h)
 	
 	if (pInstance->pDock)
 	{
-		double fMaxScale = cairo_dock_get_max_scale (pContainer);
-		pIcon->fWidth = w / fMaxScale;
-		pIcon->fHeight = h / fMaxScale;  // set the height too, because at the moment it takes into account the dock's ratio.
-		pIcon->iImageWidth = 0;  // will be updated when the icon is reloaded.
-		pIcon->iImageHeight = 0;  // will be updated when the icon is reloaded.
-		cairo_dock_load_icon_image (pIcon, pContainer);
-		
-		/**if (pInstance->pDrawContext)
-		{
-			cairo_destroy (pInstance->pDrawContext);
-			pInstance->pDrawContext = NULL;
-		}
-		if (pIcon->pIconBuffer)
-			pInstance->pDrawContext = cairo_create (pIcon->pIconBuffer);
-		if (cairo_status (pInstance->pDrawContext) != CAIRO_STATUS_SUCCESS)
-			pInstance->pDrawContext = NULL;*/
-		
-		cairo_dock_update_dock_size (pInstance->pDock);
+		pIcon->iImageWidth = w;  // request the buffer size, this is the one we care to draw stuff on the icon.
+		pIcon->iImageHeight = h;
+		pIcon->fWidth = 0;
+		pIcon->fHeight = 0;
+		cairo_dock_resize_icon_in_dock (pIcon, pInstance->pDock);
 	}
 	else  // in desklet mode, just resize the desklet, it will trigger the reload of the applet when the 'configure' event is received.
 	{
 		gtk_window_resize (GTK_WINDOW (pContainer->pWidget),
 			w,
-			h);
+			h);  /// TODO: actually, the renderer should handle this, because except with the 'Simple' view, we can't know the actual size of the icon.
 	}
 }
