@@ -36,18 +36,18 @@ G_BEGIN_DECLS
 /** Apply a surface on a context, with a zoom and a transparency factor. The context is cleared beforehand with the default icon background.
 *@param pIconContext the drawing context; is not altered by the function.
 *@param pSurface the surface to apply.
-*@param fScale zoom fastor.
+*@param fScale zoom factor.
 *@param fAlpha transparency in [0,1].
 *@param pIcon the icon.
-*@param pContainer the container of the icon.
 */
-void cairo_dock_set_icon_surface_full (cairo_t *pIconContext, cairo_surface_t *pSurface, double fScale, double fAlpha, Icon *pIcon, CairoContainer *pContainer);
+void cairo_dock_set_icon_surface_full (cairo_t *pIconContext, cairo_surface_t *pSurface, double fScale, double fAlpha, Icon *pIcon);
 
 /** Apply a surface on a context. The context is cleared beforehand with the default icon background..
 *@param pIconContext the drawing context; is not altered by the function.
 *@param pSurface the surface to apply.
+*@param pIcon the icon.
 */
-#define cairo_dock_set_icon_surface(pIconContext, pSurface) cairo_dock_set_icon_surface_full (pIconContext, pSurface, 1, 1, NULL, NULL)
+#define cairo_dock_set_icon_surface(pIconContext, pSurface, pIcon) cairo_dock_set_icon_surface_full (pIconContext, pSurface, 1, 1, pIcon)
 
 /** Draw a bar at the bottom of an Icon, with a gradation from red to green and a given length.
 *@param pIconContext the drawing context; is not altered by the function.
@@ -322,12 +322,6 @@ cairo_dock_get_integer_list_key_value (pKeyFile, cGroupName, cKeyName, &bFlushCo
 *@return the sub-menu, newly created and attached to the menu.
 */
 #define CD_APPLET_ADD_SUB_MENU(cLabel, pMenu) CD_APPLET_ADD_SUB_MENU_WITH_IMAGE(cLabel, pMenu, NULL)
-#define CD_APPLET_CREATE_AND_ADD_SUB_MENU CD_APPLET_ADD_SUB_MENU
-
-/** Create and add the default sub-menu of an applet to the main menu. This sub-menu is named according to the name of the applet, and is represented by the default icon of the applet.
-*@return the sub-menu, newly created and attached to the main menu.
-*/
-#define CD_APPLET_CREATE_MY_SUB_MENU(...) (myDocksParam.bLockAll ? CD_APPLET_MY_MENU : CD_APPLET_ADD_SUB_MENU_WITH_IMAGE (myApplet->pModule->pVisitCard->cTitle, CD_APPLET_MY_MENU, MY_APPLET_SHARE_DATA_DIR"/"MY_APPLET_ICON_FILE))
 
 /** Create and add an entry to a menu, with an icon.
 *@param cLabel name of the entry.
@@ -336,7 +330,7 @@ cairo_dock_get_integer_list_key_value (pKeyFile, cGroupName, cKeyName, &bFlushCo
 *@param pMenu menu to add the entry to.
 *@param pData data passed as parameter of the callback.
 */
-#define CD_APPLET_ADD_IN_MENU_WITH_STOCK_AND_DATA(cLabel, gtkStock, pCallBack, pMenu, pData) cairo_dock_add_in_menu_with_stock_and_data (cLabel, gtkStock, (GFunc)pCallBack, pMenu, pData)
+#define CD_APPLET_ADD_IN_MENU_WITH_STOCK_AND_DATA(cLabel, gtkStock, pCallBack, pMenu, pData) cairo_dock_add_in_menu_with_stock_and_data (cLabel, gtkStock, G_CALLBACK(pCallBack), pMenu, pData)
 
 /** Create and add an entry to a menu.
 *@param cLabel name of the entry.
@@ -366,12 +360,13 @@ cairo_dock_get_integer_list_key_value (pKeyFile, cGroupName, cKeyName, &bFlushCo
 #define CD_APPLET_ADD_SEPARATOR_IN_MENU(pMenu) do { \
 	pMenuItem = gtk_separator_menu_item_new (); \
 	gtk_menu_shell_append(GTK_MENU_SHELL (pMenu), pMenuItem); } while (0)
-#define CD_APPLET_ADD_SEPARATOR CD_APPLET_ADD_SEPARATOR_IN_MENU
 
-/** Create and add an entry to a menu for the 'about' function.
-*@param pMenu menu to add the entry to.
-*/
-#define CD_APPLET_ADD_ABOUT_IN_MENU(pMenu) CD_APPLET_ADD_IN_MENU_WITH_STOCK (_("Applet's handbook"), GTK_STOCK_ABOUT, cairo_dock_pop_up_about_applet, pMenu)
+/** Create and add the first separator to the main menu.
+ */
+#define CD_APPLET_ADD_FIRST_SEPARATOR_IN_MENU do { \
+	if (pClickedIcon == myIcon || (pClickedContainer == CAIRO_CONTAINER (myDesklet) && pClickedIcon == NULL)) \
+		CD_APPLET_ADD_SEPARATOR_IN_MENU (CD_APPLET_MY_MENU); } while (0)
+
 
 /** Pop-up a menu on the applet's icon.
 *@param pMenu menu to show
@@ -453,6 +448,16 @@ cairo_dock_get_integer_list_key_value (pKeyFile, cGroupName, cKeyName, &bFlushCo
 */
 #define CD_APPLET_SCROLL_DOWN (CD_APPLET_SCROLL_DIRECTION == GDK_SCROLL_DOWN)
 
+/** Bind a shortkey to an action. Unbind it when you don't want it anymore. myApplet is passed as the callback data.
+ * @param cShortKey a shortkey.
+ * @param cDescription a short description of the action
+ * @param cGroupName group name where it's stored in the applet's conf file
+ * @param cKeyName key name where it's stored in the applet's conf file
+ * @param handler function called when the shortkey is pressed by the user
+ * @return the key binding
+*/
+#define CD_APPLET_BIND_KEY(cShortKey, cDescription, cGroupName, cKeyName, handler) \
+cd_keybinder_bind (cShortKey, myApplet->pModule->pVisitCard->cTitle, cDescription, myApplet->pModule->pVisitCard->cIconFilePath, myApplet->cConfFilePath, cGroupName, cKeyName, handler, myApplet)
 
   /////////////////////
  // DRAWING SURFACE //
@@ -466,17 +471,13 @@ cairo_dock_get_integer_list_key_value (pKeyFile, cGroupName, cKeyName, &bFlushCo
 */
 #define CAIRO_DOCK_REDRAW_MY_CONTAINER \
 	cairo_dock_redraw_container (myContainer)
-/** Reload the reflect of the applet's icon (do nothing in OpenGL mode).
-*/
-#define CD_APPLET_UPDATE_REFLECT_ON_MY_ICON \
-	if (myContainer->bUseReflect) cairo_dock_add_reflection_to_icon (myIcon, myContainer)
 
 /** Load an image into a surface, at the same size as the applet's icon. If the image is given by its sole name, it is searched inside the current theme root folder. 
 *@param cImagePath path or name of an image.
 *@return the newly allocated surface.
 */
 #define CD_APPLET_LOAD_SURFACE_FOR_MY_APPLET(cImagePath) \
-	cairo_dock_create_surface_from_image_simple (cImagePath, myIcon->fWidth * (myDock ? (1 + myIconsParam.fAmplitude) / myDock->container.fRatio : 1), myIcon->fHeight* (myDock ? (1 + myIconsParam.fAmplitude) / myDock->container.fRatio : 1))
+	cairo_dock_create_surface_from_image_simple (cImagePath, myIcon->iImageWidth, myIcon->iImageHeight)
 
 /** Load a user image into a surface, at the same size as the applet's icon, or a default image taken in the installed folder of the applet if the first one is NULL. If the user image is given by its sole name, it is searched inside the current theme root folder.
 *@param cUserImageName name or path of an user image.
@@ -506,8 +507,7 @@ cairo_dock_get_integer_list_key_value (pKeyFile, cGroupName, cKeyName, &bFlushCo
 *@param fScale zoom factor (at 1 the surface will fill all the icon).
 */
 #define CD_APPLET_SET_SURFACE_ON_MY_ICON_WITH_ZOOM(pSurface, fScale) do { \
-	cairo_dock_set_icon_surface_full (myDrawContext, pSurface, fScale, 1., myIcon, myContainer); \
-	cairo_dock_add_reflection_to_icon (myIcon, myContainer); \
+	cairo_dock_set_icon_surface_full (myDrawContext, pSurface, fScale, 1., myIcon); \
 	cairo_dock_redraw_icon (myIcon, myContainer); } while (0)
 
 /** Apply a surface on the applet's icon with a transparency factor, and redraw it.
@@ -515,8 +515,7 @@ cairo_dock_get_integer_list_key_value (pKeyFile, cGroupName, cKeyName, &bFlushCo
 *@param fAlpha transparency (in [0,1]).
 */
 #define CD_APPLET_SET_SURFACE_ON_MY_ICON_WITH_ALPHA(pSurface, fAlpha) do { \
-	cairo_dock_set_icon_surface_full (myDrawContext, pSurface, 1., fAlpha, myIcon, myContainer); \
-	cairo_dock_add_reflection_to_icon (myIcon, myContainer); \
+	cairo_dock_set_icon_surface_full (myDrawContext, pSurface, 1., fAlpha, myIcon); \
 	cairo_dock_redraw_icon (myIcon, myContainer); } while (0)
 
 /** Apply a surface on the applet's icon with add a bar at the bottom, and redraw it. The bar is drawn at the bottom of the icon with a gradation from red to green and a given length.
@@ -525,7 +524,6 @@ cairo_dock_get_integer_list_key_value (pKeyFile, cGroupName, cKeyName, &bFlushCo
 */
 #define CD_APPLET_SET_SURFACE_ON_MY_ICON_WITH_BAR(pSurface, fValue) do { \
 	cairo_dock_set_icon_surface_with_bar (myDrawContext, pSurface, fValue, myIcon); \
-	cairo_dock_add_reflection_to_icon (myIcon, myContainer); \
 	cairo_dock_redraw_icon (myIcon, myContainer); } while (0)
 
 /** Apply an image on the applet's icon. The image is resized at the same size as the icon. Does not trigger the icon refresh.
@@ -601,7 +599,10 @@ cairo_dock_get_integer_list_key_value (pKeyFile, cGroupName, cKeyName, &bFlushCo
 ///////////////
 /** Prevent the applet's icon to be animated when the mouse hovers it (call it once at init).
 */
-#define CD_APPLET_SET_STATIC_ICON cairo_dock_set_icon_static (myIcon)
+#define CD_APPLET_SET_STATIC_ICON cairo_dock_set_icon_static (myIcon, TRUE)
+/** Prevent the applet's icon to be animated when the mouse hovers it (call it once at init).
+*/
+#define CD_APPLET_UNSET_STATIC_ICON cairo_dock_set_icon_static (myIcon, FALSE)
 
 /** Make the applet's icon always visible, even when the dock is hidden.
 */
@@ -612,9 +613,7 @@ cairo_dock_get_integer_list_key_value (pKeyFile, cGroupName, cKeyName, &bFlushCo
 *@param iAnimationLength number of rounds the animation should be played.
 */
 #define CD_APPLET_ANIMATE_MY_ICON(cAnimationName, iAnimationLength) \
-	do {\
-		if (myDock) \
-			cairo_dock_request_icon_animation (myIcon, myDock, cAnimationName, iAnimationLength); } while (0)
+	cairo_dock_request_icon_animation (myIcon, myContainer, cAnimationName, iAnimationLength)
 
 /** Stop any animation on the applet's icon.
 */
@@ -665,12 +664,19 @@ cairo_dock_get_integer_list_key_value (pKeyFile, cGroupName, cKeyName, &bFlushCo
  *@param iPosition position where to display the overlay
  *@return TRUE if the overlay has been successfuly added.
  */
-#define CD_APPLET_ADD_OVERLAY(cImageFile, iPosition) cairo_dock_add_overlay_from_image (myIcon, cImageFile, iPosition)
+#define CD_APPLET_ADD_OVERLAY_ON_MY_ICON(cImageFile, iPosition) cairo_dock_add_overlay_from_image (myIcon, cImageFile, iPosition)
+
+/** Print an overlay from an image on the applet's icon (it can't be removed without erasing the icon).
+ *@param cImageFile an image (if it's not a path, it is searched amongst the current theme's images)
+ *@param iPosition position where to display the overlay
+ *@return TRUE if the overlay has been successfuly added.
+ */
+#define CD_APPLET_PRINT_OVERLAY_ON_MY_ICON(cImageFile, iPosition) cairo_dock_print_overlay_on_icon_from_image (myIcon, myContainer, cImageFile, iPosition)
 
 /** Remove an overlay the applet's icon, given its position (there is only one overlay at a given position).
  *@param iPosition position of the overlay
  */
-#define CD_APPLET_REMOVE_OVERLAY(iPosition) cairo_dock_remove_overlay_at_position (iPosition)
+#define CD_APPLET_REMOVE_OVERLAY_ON_MY_ICON(iPosition) cairo_dock_remove_overlay_at_position (myIcon, iPosition)
 
 
 /** Make an emblem from an image. If the image is given by its sole name, it is looked up inside the root theme folder. Free it with cairo_dock_free_emblem.
@@ -684,12 +690,6 @@ cairo_dock_get_integer_list_key_value (pKeyFile, cGroupName, cKeyName, &bFlushCo
 */
 #define CD_APPLET_DRAW_EMBLEM_ON_MY_ICON(pEmblem) cairo_dock_draw_emblem_on_icon (pEmblem, myIcon, myContainer)
 
-#define CD_APPLET_SET_EMBLEM_ON_MY_ICON(cImageFile, iPosition) do {\
-	CairoEmblem *pEmblem = cairo_dock_make_emblem (cImageFile, myIcon);\
-	cairo_dock_set_emblem_position (pEmblem, iPosition);\
-	cairo_dock_draw_emblem_on_icon (pEmblem, myIcon, myContainer);\
-	cairo_dock_free_emblem (pEmblem); } while (0)
-
 
 /** Add a Data Renderer the applet's icon.
 *@param pAttr the attributes of the Data Renderer. They allow you to define its properties.
