@@ -52,7 +52,7 @@
 #include "cairo-dock-keyfile-utilities.h"  // cairo_dock_open_key_file
 #include "cairo-dock-application-facility.h"
 #include "cairo-dock-X-manager.h"
-#include "cairo-dock-emblem.h"
+#include "cairo-dock-overlay.h"
 #define _MANAGER_DEF_
 #include "cairo-dock-applications-manager.h"
 
@@ -74,6 +74,7 @@ static Display *s_XDisplay = NULL;
 static GHashTable *s_hXWindowTable = NULL;  // table des fenetres X affichees dans le dock.
 static int s_bAppliManagerIsRunning = FALSE;
 static int s_iTime = 1;  // on peut aller jusqu'a 2^31, soit 17 ans a 4Hz.
+static int s_iNumWindow = 1;  // used to order appli icons by age (=creation date).
 static Window s_iCurrentActiveWindow = 0;
 static Atom s_aNetWmState;
 static Atom s_aNetWmDesktop;
@@ -87,6 +88,7 @@ static Atom s_aWmHints;
 static void cairo_dock_blacklist_appli (Window Xid);
 static Icon * cairo_dock_create_icon_from_xwindow (Window Xid, CairoDock *pDock);
 
+
   //////////////////////////
  // Appli manager : core //
 //////////////////////////
@@ -102,13 +104,16 @@ static void _cairo_dock_hide_show_windows_on_other_desktops (Window *Xid, Icon *
 			cd_debug (" => est sur le bureau actuel.");
 			if (icon->cParentDockName == NULL)
 			{
-				pParentDock = cairo_dock_insert_appli_in_dock (icon, pMainDock, CAIRO_DOCK_UPDATE_DOCK_SIZE, ! CAIRO_DOCK_ANIMATE_ICON);
+				pParentDock = cairo_dock_insert_appli_in_dock (icon, pMainDock, ! CAIRO_DOCK_ANIMATE_ICON);
 			}
 		}
 		else
 		{
 			cd_debug (" => n'est pas sur le bureau actuel.");
-			pParentDock = cairo_dock_detach_appli (icon);
+			if (icon->cParentDockName != NULL)  // if in a dock, detach it
+				pParentDock = cairo_dock_detach_appli (icon);
+			else  // else if inhibited, detach from the inhibitor
+				cairo_dock_detach_Xid_from_inhibitors (icon->Xid, icon->cClass);
 		}
 		if (pParentDock != NULL)
 			gtk_widget_queue_draw (pParentDock->container.pWidget);
@@ -136,8 +141,7 @@ static void _hide_if_any_overlap (CairoDock *pDock, gpointer data)
 	{
 		if (cairo_dock_search_window_overlapping_dock (pDock) != NULL)
 		{
-			if (!cairo_dock_is_temporary_hidden (pDock))
-				cairo_dock_activate_temporary_auto_hide (pDock);
+			cairo_dock_activate_temporary_auto_hide (pDock);
 		}
 	}
 }
@@ -241,9 +245,12 @@ static gboolean _cairo_dock_remove_old_applis (Window *Xid, Icon *icon, gpointer
 	//g_print ("%s (%s(%ld) %d / %d)\n", __func__, icon->cName, icon->Xid, icon->iLastCheckTime, iTime);
 	if (icon->iLastCheckTime >= 0 && icon->iLastCheckTime < iTime && ! cairo_dock_icon_is_being_removed (icon))
 	{
-		cd_message ("cette fenetre (%ld(%ld), %s) est trop vieille (%d / %d)", *Xid, icon->Xid, icon->cName, icon->iLastCheckTime, iTime);
+		cd_message ("cette fenetre (%ld(%ld), %s) est trop vieille (%d / %d, %s)", *Xid, icon->Xid, icon->cName, icon->iLastCheckTime, iTime, icon->cParentDockName);
 		if (CAIRO_DOCK_IS_APPLI (icon))
 		{
+			// notify everybody
+			cairo_dock_notify_on_object (&myTaskbarMgr, NOTIFICATION_APPLI_DESTROYED, icon);
+			
 			CairoDock *pParentDock = cairo_dock_search_dock_from_name (icon->cParentDockName);
 			if (pParentDock != NULL)
 			{
@@ -279,15 +286,13 @@ static void _on_update_applis_list (CairoDock *pDock)
 		return;
 	s_iTime ++;
 	gulong i, iNbWindows = 0;
-	Window *pXWindowsList = cairo_dock_get_windows_list (&iNbWindows, TRUE);
+	Window *pXWindowsList = cairo_dock_get_windows_list (&iNbWindows, TRUE);  // TRUE => ordered by z-stack.
 	
 	Window Xid;
 	Icon *icon;
 	int iStackOrder = 0;
 	gpointer pOriginalXid;
 	gboolean bAppliAlreadyRegistered;
-	gboolean bUpdateMainDockSize = FALSE;
-	CairoDock *pParentDock;
 	
 	for (i = 0; i < iNbWindows; i ++)
 	{
@@ -304,26 +309,15 @@ static void _on_update_applis_list (CairoDock *pDock)
 				icon->iStackOrder = iStackOrder ++;
 				if (myTaskbarParam.bShowAppli)
 				{
-					///if ((! myTaskbarParam.bAppliOnCurrentDesktopOnly || cairo_dock_appli_is_on_current_desktop (icon)))  // bHideVisibleApplis est gere lors de l'insertion.
-					///{
-						cd_message (" insertion de %s ... (%d)", icon->cName, icon->iLastCheckTime);
-						pParentDock = cairo_dock_insert_appli_in_dock (icon, pDock, ! CAIRO_DOCK_UPDATE_DOCK_SIZE, CAIRO_DOCK_ANIMATE_ICON);
-						if (pParentDock != NULL)
-						{
-							if (pParentDock->bIsMainDock)  // update the main dock at once in the end.
-								bUpdateMainDockSize = TRUE;
-							else
-								cairo_dock_update_dock_size (pParentDock);
-						}
-					/**}
-					else if (myTaskbarParam.bMixLauncherAppli)  // on met tout de meme l'indicateur sur le lanceur.
-					{
-						cairo_dock_prevent_inhibited_class (icon);
-					}*/
+					cd_message (" insertion de %s ... (%d)", icon->cName, icon->iLastCheckTime);
+					cairo_dock_insert_appli_in_dock (icon, pDock, CAIRO_DOCK_ANIMATE_ICON);
 				}
 				
 				// visibilite
 				cairo_dock_foreach_root_docks ((GFunc)_hide_if_overlap, icon);
+				
+				// notify everybody
+				cairo_dock_notify_on_object (&myTaskbarMgr, NOTIFICATION_APPLI_CREATED, icon);
 			}
 			else
 				cairo_dock_blacklist_appli (Xid);
@@ -337,9 +331,6 @@ static void _on_update_applis_list (CairoDock *pDock)
 	}
 	
 	g_hash_table_foreach_remove (s_hXWindowTable, (GHRFunc) _cairo_dock_remove_old_applis, GINT_TO_POINTER (s_iTime));
-	
-	if (bUpdateMainDockSize)  // will stay at FALSE if pDock is NULL.
-		cairo_dock_update_dock_size (pDock);
 
 	XFree (pXWindowsList);
 }
@@ -464,28 +455,28 @@ static void _on_change_window_state (Icon *icon)
 		}
 	}*/
 	
-	// masquage du dock.
-	if (Xid == s_iCurrentActiveWindow)  // c'est la fenetre courante qui a change d'etat.
-	{
-		if ((bIsHidden != icon->bIsHidden) || (bIsFullScreen != icon->bIsFullScreen))  // si c'est l'etat maximise qui a change, on le verra au changement de dimensions.
-		{
-			icon->bIsFullScreen = bIsFullScreen;
-			icon->bIsHidden = bIsHidden;
-			cairo_dock_foreach_root_docks ((GFunc)_hide_show_if_on_our_way, icon);
-		}
-	}
+	gboolean bHiddenChanged 	= (bIsHidden != icon->bIsHidden);
+	gboolean bMaximizedChanged 	= (bIsMaximized != icon->bIsMaximized);
+	gboolean bFullScreenChanged 	= (bIsFullScreen != icon->bIsFullScreen);
 	
 	icon->bIsMaximized = bIsMaximized;
 	icon->bIsFullScreen = bIsFullScreen;
 	icon->bIsHidden = bIsHidden;
 	
+	// masquage du dock.
+	if (Xid == s_iCurrentActiveWindow)  // c'est la fenetre courante qui a change d'etat.
+	{
+		if (bHiddenChanged || bFullScreenChanged)  // si c'est l'etat maximise qui a change, on le verra au changement de dimensions.
+		{
+			cairo_dock_foreach_root_docks ((GFunc)_hide_show_if_on_our_way, icon);
+		}
+	}
+	
 	// on gere le cachage/apparition de l'icone (transparence ou miniature, applis minimisees seulement).
 	CairoDock *pParentDock = cairo_dock_search_dock_from_name (icon->cParentDockName);
-	if (bIsHidden != bPrevHidden)
+	if (bHiddenChanged)
 	{
 		cd_message ("  changement de visibilite -> %d", bIsHidden);
-		icon->bIsHidden = bIsHidden;
-		
 		// visibilite
 		if (!icon->bIsHidden)  // la fenetre reapparait.
 			cairo_dock_foreach_root_docks ((GFunc)_hide_if_overlap, icon);
@@ -495,10 +486,9 @@ static void _on_change_window_state (Icon *icon)
 		// affichage des applis minimisees.
 		if (g_bUseOpenGL && myTaskbarParam.iMinimizedWindowRenderType == 2)
 		{
-			CairoDock *pDock = cairo_dock_search_dock_from_name (icon->cParentDockName);
-			if (pDock != NULL)
+			if (pParentDock != NULL)
 			{
-				cairo_dock_draw_hidden_appli_icon (icon, CAIRO_CONTAINER (pDock), TRUE);
+				cairo_dock_draw_hidden_appli_icon (icon, CAIRO_CONTAINER (pParentDock), TRUE);
 			}
 		}
 		else if (myTaskbarParam.iMinimizedWindowRenderType == 0)
@@ -515,7 +505,7 @@ static void _on_change_window_state (Icon *icon)
 				cd_message (" => se cache");
 				///if (! myTaskbarParam.bAppliOnCurrentDesktopOnly || cairo_dock_appli_is_on_current_desktop (icon))
 				///{
-					pParentDock = cairo_dock_insert_appli_in_dock (icon, g_pMainDock, CAIRO_DOCK_UPDATE_DOCK_SIZE, CAIRO_DOCK_ANIMATE_ICON);
+					pParentDock = cairo_dock_insert_appli_in_dock (icon, g_pMainDock, CAIRO_DOCK_ANIMATE_ICON);
 					if (pParentDock != NULL)
 					{
 						if (g_bUseOpenGL && myTaskbarParam.iMinimizedWindowRenderType == 2)  // quand on est passe dans ce cas tout a l'heure l'icone n'etait pas encore dans son dock.
@@ -561,6 +551,8 @@ static void _on_change_window_state (Icon *icon)
 		}
 		#endif
 	}
+	// notify everybody
+	cairo_dock_notify_on_object (&myTaskbarMgr, NOTIFICATION_APPLI_STATE_CHANGED, icon, bHiddenChanged, bMaximizedChanged, bFullScreenChanged);
 }
 
 static void _on_change_window_desktop (Icon *icon)
@@ -588,14 +580,18 @@ static void _on_change_window_desktop (Icon *icon)
 	{
 		cairo_dock_foreach_root_docks ((GFunc)_show_if_no_overlapping_window, NULL);
 	}
+	// notify everybody
+	cairo_dock_notify_on_object (&myTaskbarMgr, NOTIFICATION_APPLI_SIZE_POSITION_CHANGED, icon);
 }
 
 static void _on_change_window_size_position (Icon *icon, XConfigureEvent *e)
 {
 	Window Xid = icon->Xid;
+	int x = e->x, y = e->y;
+	int w = e->width, h = e->height;
 	
 	#ifdef HAVE_XEXTEND
-	if (e->width != icon->windowGeometry.width || e->height != icon->windowGeometry.height)
+	if (w != icon->windowGeometry.width || h != icon->windowGeometry.height)
 	{
 		if (icon->iBackingPixmap != 0)
 			XFreePixmap (s_XDisplay, icon->iBackingPixmap);
@@ -608,22 +604,49 @@ static void _on_change_window_size_position (Icon *icon, XConfigureEvent *e)
 			icon->iBackingPixmap = 0;
 	}
 	#endif
-	icon->windowGeometry.width = e->width;
-	icon->windowGeometry.height = e->height;
-	icon->windowGeometry.x = e->x;
-	icon->windowGeometry.y = e->y;
-	icon->iViewPortX = e->x / g_desktopGeometry.iXScreenWidth[CAIRO_DOCK_HORIZONTAL] + g_desktopGeometry.iCurrentViewportX;
-	icon->iViewPortY = e->y / g_desktopGeometry.iXScreenHeight[CAIRO_DOCK_HORIZONTAL] + g_desktopGeometry.iCurrentViewportY;
+	
+	// get the correct coordinates (same reason as in cairo_dock_get_xwindow_geometry(), this is a workaround to a bug in X; we didn't need that before Ubuntu 11.10).
+	// actually, the XConfigureEvent has more precise coordinates during a move (updated more often than the ones from XTranslateCoordinates), but it has nil coordinates during a resize (and we have no way to distinguish this case).
+	Window child, root = DefaultRootWindow (s_XDisplay);
+	XTranslateCoordinates (s_XDisplay, Xid, root, 0, 0, &x, &y, &child);
+	//g_print (" :: %s ((%d;%d) <> (%d;%d), %dx%d)\n", __func__, x, y, e->x, e->y, w, h);
+	
+	// take into account the window borders (Note: we don't monitor the _NET_FRAME_EXTENTS property, so let's retrieve theme now; maybe we should monitor them...)
+	int left=0, right=0, top=0, bottom=0;
+	gulong iLeftBytes, iBufferNbElements = 0;
+	Atom aReturnedType = 0;
+	int aReturnedFormat = 0;
+	gulong *pBuffer = NULL;
+	XGetWindowProperty (s_XDisplay, Xid, XInternAtom (s_XDisplay, "_NET_FRAME_EXTENTS", False), 0, G_MAXULONG, False, XA_CARDINAL, &aReturnedType, &aReturnedFormat, &iBufferNbElements, &iLeftBytes, (guchar **)&pBuffer);
+	if (iBufferNbElements > 3)
+	{
+		left=pBuffer[0], right=pBuffer[1], top=pBuffer[2], bottom=pBuffer[3];
+	}
+	if (pBuffer)
+		XFree (pBuffer);
+	
+	icon->windowGeometry.width = w + left + right;
+	icon->windowGeometry.height = h + top + bottom;
+	icon->windowGeometry.x = x - left;
+	icon->windowGeometry.y = y - top;
+	icon->iViewPortX = x / g_desktopGeometry.iXScreenWidth[CAIRO_DOCK_HORIZONTAL] + g_desktopGeometry.iCurrentViewportX;
+	icon->iViewPortY = y / g_desktopGeometry.iXScreenHeight[CAIRO_DOCK_HORIZONTAL] + g_desktopGeometry.iCurrentViewportY;
+	//g_print (" :: (%d;%d) %dx%d)\n", icon->windowGeometry.x, icon->windowGeometry.y, icon->windowGeometry.width, icon->windowGeometry.height);
 	
 	// on regarde si l'appli est sur le viewport courant.
-	if (e->x + e->width <= 0 || e->x >= g_desktopGeometry.iXScreenWidth[CAIRO_DOCK_HORIZONTAL] || e->y + e->height <= 0 || e->y >= g_desktopGeometry.iXScreenHeight[CAIRO_DOCK_HORIZONTAL])  // en fait il faudrait faire ca modulo le nombre de viewports * la largeur d'un bureau, car avec une fenetre a droite, elle peut revenir sur le bureau par la gauche si elle est tres large...
+	if (x + w <= 0 || x >= g_desktopGeometry.iXScreenWidth[CAIRO_DOCK_HORIZONTAL] || y + h <= 0 || y >= g_desktopGeometry.iXScreenHeight[CAIRO_DOCK_HORIZONTAL])  // not on this desktop (actually, it may be only a little bit on this desktop ... maybe we should use a % of the width, or a margin ...)
 	{
 		// applis du bureau courant seulement.
-		if (myTaskbarParam.bAppliOnCurrentDesktopOnly && icon->cParentDockName != NULL)
+		if (myTaskbarParam.bAppliOnCurrentDesktopOnly)
 		{
-			CairoDock *pParentDock = cairo_dock_detach_appli (icon);
-			if (pParentDock)
-				gtk_widget_queue_draw (pParentDock->container.pWidget);
+			if (icon->cParentDockName != NULL)
+			{
+				CairoDock *pParentDock = cairo_dock_detach_appli (icon);
+				if (pParentDock)
+					gtk_widget_queue_draw (pParentDock->container.pWidget);
+			}
+			else
+				cairo_dock_detach_Xid_from_inhibitors (icon->Xid, icon->cClass);
 		}
 		
 		// visibilite
@@ -634,10 +657,8 @@ static void _on_change_window_size_position (Icon *icon, XConfigureEvent *e)
 		// applis du bureau courant seulement.
 		if (myTaskbarParam.bAppliOnCurrentDesktopOnly && icon->cParentDockName == NULL && myTaskbarParam.bShowAppli)
 		{
-			cd_message ("cette fenetre est sur le bureau courant (%d;%d)", e->x, e->y);
-			///gboolean bInsideDock = (icon->cParentDockName != NULL);
-			///if (! bInsideDock)
-				cairo_dock_insert_appli_in_dock (icon, g_pMainDock, CAIRO_DOCK_UPDATE_DOCK_SIZE, ! CAIRO_DOCK_ANIMATE_ICON);
+			//cd_message ("cette fenetre est sur le bureau courant (%d;%d)", x, y);
+			cairo_dock_insert_appli_in_dock (icon, g_pMainDock, ! CAIRO_DOCK_ANIMATE_ICON);  // the icon might be on this desktop and yet not in a dock (inhibited), in which case this function does nothing.
 		}
 		
 		// visibilite
@@ -649,6 +670,9 @@ static void _on_change_window_size_position (Icon *icon, XConfigureEvent *e)
 	{
 		cairo_dock_foreach_root_docks ((GFunc)_hide_show_if_on_our_way, icon);
 	}
+	
+	// notify everybody
+	cairo_dock_notify_on_object (&myTaskbarMgr, NOTIFICATION_APPLI_SIZE_POSITION_CHANGED, icon);
 }
 
 static gboolean _on_window_configured_notification (gpointer data, Window Xid, XConfigureEvent *e)
@@ -675,6 +699,9 @@ static void _on_change_window_name (Icon *icon, CairoDock *pDock, gboolean bSear
 			cairo_dock_set_icon_name (cName, icon, NULL);
 			
 			cairo_dock_update_name_on_inhibitors (icon->cClass, icon->Xid, cName);
+			
+			// notify everybody
+			cairo_dock_notify_on_object (&myTaskbarMgr, NOTIFICATION_APPLI_NAME_CHANGED, icon);
 		}
 		g_free (cName);
 	}
@@ -688,6 +715,8 @@ static void _on_change_window_icon (Icon *icon, CairoDock *pDock)
 		if (pDock->iRefCount != 0)
 			cairo_dock_trigger_redraw_subdock_content (pDock);
 		cairo_dock_redraw_icon (icon, CAIRO_CONTAINER (pDock));
+		// notify everybody
+		cairo_dock_notify_on_object (&myTaskbarMgr, NOTIFICATION_APPLI_ICON_CHANGED, icon);
 	}
 }
 
@@ -715,6 +744,8 @@ static void _on_change_window_hints (Icon *icon, CairoDock *pDock, int iState)
 				if (pDock->iRefCount != 0)
 					cairo_dock_trigger_redraw_subdock_content (pDock);
 				cairo_dock_redraw_icon (icon, CAIRO_CONTAINER (pDock));
+				// notify everybody
+				cairo_dock_notify_on_object (&myTaskbarMgr, NOTIFICATION_APPLI_ICON_CHANGED, icon);
 			}
 		}
 		XFree (pWMHints);  // "When finished with the data, free the space used for it by calling XFree()."
@@ -726,6 +757,51 @@ static void _on_change_window_hints (Icon *icon, CairoDock *pDock, int iState)
 	}
 }
 
+static void _on_change_window_class (Icon *icon, CairoDock *pDock)
+{
+	//g_print ("WM_CLASS changed (%p, %s)!\n", icon->pMimeTypes, icon->cCommand);
+	// retrieve the new class
+	gchar *cClass = NULL, *cWmClass = NULL;
+	cClass = cairo_dock_get_xwindow_class (icon->Xid, &cWmClass);
+	if (! cairo_dock_strings_differ (cClass, icon->cClass) || ! cClass)  // cClass can be NULL (libreoffice when closing a window).
+	{
+		//g_print ("fausse alerte\n");
+		g_free (cClass);
+		g_free (cWmClass);
+		return;
+	}
+	//g_print (" %s -> %s\n", icon->cClass, cClass);
+	
+	// remove the icon from the dock, and then from its class
+	CairoDock *pParentDock = NULL;
+	if (icon->cParentDockName != NULL)  // if in a dock, detach it
+		pParentDock = cairo_dock_detach_appli (icon);
+	else  // else if inhibited, detach from the inhibitor
+		cairo_dock_detach_Xid_from_inhibitors (icon->Xid, icon->cClass);
+	cairo_dock_remove_appli_from_class (icon);
+	
+	// set the new class
+	g_free (icon->cClass);
+	icon->cClass = cClass;
+	g_free (icon->cWmClass);
+	icon->cWmClass = cWmClass;
+	cairo_dock_add_appli_to_class (icon);
+	
+	// re-insert the icon
+	pParentDock = cairo_dock_insert_appli_in_dock (icon, g_pMainDock, ! CAIRO_DOCK_ANIMATE_ICON);
+	if (pParentDock != NULL)
+		gtk_widget_queue_draw (pParentDock->container.pWidget);
+	
+	// reload the icon
+	g_strfreev (icon->pMimeTypes);
+	icon->pMimeTypes = g_strdupv ((gchar**)cairo_dock_get_class_mimetypes (icon->cClass));
+	g_free (icon->cCommand);
+	icon->cCommand = g_strdup (cairo_dock_get_class_command (icon->cClass));
+	cairo_dock_reload_icon_image (icon, CAIRO_CONTAINER (pParentDock));
+	// notify everybody
+	cairo_dock_notify_on_object (&myTaskbarMgr, NOTIFICATION_APPLI_ICON_CHANGED, icon);
+}
+
 static gboolean _on_property_changed_notification (gpointer data, Window Xid, Atom aProperty, int iState)
 {
 	Icon *icon = g_hash_table_lookup (s_hXWindowTable, &Xid);
@@ -734,7 +810,7 @@ static gboolean _on_property_changed_notification (gpointer data, Window Xid, At
 		if (! cairo_dock_xwindow_skip_taskbar (Xid))
 		{
 			//g_print ("Special case : this appli (%ld) should not be ignored any more!\n", Xid);
-			g_hash_table_remove (s_hXWindowTable, &Xid);
+			g_hash_table_remove (s_hXWindowTable, &Xid);  // will be detected by the XEvent loop.
 			g_free (icon);
 		}
 		return CAIRO_DOCK_INTERCEPT_NOTIFICATION;
@@ -765,6 +841,10 @@ static gboolean _on_property_changed_notification (gpointer data, Window Xid, At
 		{
 			_on_change_window_hints (icon, pParentDock, iState);
 		}
+		else if (aProperty == XInternAtom (s_XDisplay, "WM_CLASS", False))
+		{
+			_on_change_window_class (icon, pParentDock);
+		}
 	}
 	return CAIRO_DOCK_LET_PASS_NOTIFICATION;
 }
@@ -819,7 +899,7 @@ void cairo_dock_unregister_appli (Icon *icon)
 		
 		// remove from class
 		cairo_dock_remove_appli_from_class (icon);  // n'efface pas sa classe (on peut en avoir besoin encore).
-		cairo_dock_update_Xid_on_inhibitors (icon->Xid, icon->cClass);
+		cairo_dock_detach_Xid_from_inhibitors (icon->Xid, icon->cClass);
 		
 		// stop watching events
 		cairo_dock_set_xwindow_mask (icon->Xid, None);
@@ -844,17 +924,12 @@ void cairo_dock_start_applications_manager (CairoDock *pDock)
 	
 	//\__________________ On recupere l'ensemble des fenetres presentes.
 	gulong i, iNbWindows = 0;
-	Window *pXWindowsList = cairo_dock_get_windows_list (&iNbWindows, FALSE);  // on recupere les fenetres par ordre de creation, de facon a ce que si on redemarre la barre des taches, les lanceurs soient lies aux memes fenetres. Au prochain update, la liste sera recuperee par z-order, ce qui remettra le z-order de chaque icone a jour.
+	Window *pXWindowsList = cairo_dock_get_windows_list (&iNbWindows, FALSE);  // ordered by creation date; this allows us to set the correct age to the icon, which is constant. On the next updates, the z-order (which is dynamic) will be set.
 	
 	if (s_iCurrentActiveWindow == 0)
 		s_iCurrentActiveWindow = cairo_dock_get_active_xwindow ();
 	
 	//\__________________ On cree les icones de toutes ces applis.
-	CairoDock *pParentDock;
-	gboolean bUpdateMainDockSize = FALSE;
-	
-	/*Icon * pLastAppli = cairo_dock_get_last_appli (pDock->icons);
-	int iOrder = (pLastAppli != NULL ? pLastAppli->fOrder + 1 : 1);*/
 	Window Xid;
 	Icon *pIcon;
 	for (i = 0; i < iNbWindows; i ++)
@@ -862,28 +937,12 @@ void cairo_dock_start_applications_manager (CairoDock *pDock)
 		Xid = pXWindowsList[i];
 		pIcon = cairo_dock_create_icon_from_xwindow (Xid, pDock);
 		
-		if (pIcon != NULL)
+		if (pIcon != NULL)  // authorised window.
 		{
-			//pIcon->fOrder = iOrder++;
 			pIcon->iLastCheckTime = s_iTime;
 			if (myTaskbarParam.bShowAppli && pDock)
 			{
-				///if (! myTaskbarParam.bAppliOnCurrentDesktopOnly || cairo_dock_appli_is_on_current_desktop (pIcon))  // le filtre 'bHideVisibleApplis' est gere dans la fonction d'insertion.
-				///{
-					pParentDock = cairo_dock_insert_appli_in_dock (pIcon, pDock, ! CAIRO_DOCK_UPDATE_DOCK_SIZE, ! CAIRO_DOCK_ANIMATE_ICON);
-					//g_print (">>>>>>>>>>>> Xid : %d\n", Xid);
-					if (pParentDock != NULL)
-					{
-						if (pParentDock->bIsMainDock)
-							bUpdateMainDockSize = TRUE;
-						else
-							cairo_dock_update_dock_size (pParentDock);
-					}
-				/**}
-				else if (myTaskbarParam.bMixLauncherAppli)  // on met tout de meme l'indicateur sur le lanceur.
-				{
-					cairo_dock_prevent_inhibited_class (pIcon);
-				}*/
+				cairo_dock_insert_appli_in_dock (pIcon, pDock, ! CAIRO_DOCK_ANIMATE_ICON);
 			}
 		}
 		else
@@ -892,9 +951,6 @@ void cairo_dock_start_applications_manager (CairoDock *pDock)
 	if (pXWindowsList != NULL)
 		XFree (pXWindowsList);
 	
-	if (bUpdateMainDockSize)
-		cairo_dock_update_dock_size (pDock);
-	
 	// masquage du dock, une fois que sa taille est correcte.
 	Icon *pActiveAppli = cairo_dock_get_current_active_icon ();
 	cairo_dock_foreach_root_docks ((GFunc)_hide_show_if_on_our_way, pActiveAppli);
@@ -919,7 +975,7 @@ static gboolean _cairo_dock_remove_one_appli (Window *pXid, Icon *pIcon, gpointe
 	{
 		gchar *cParentDockName = pIcon->cParentDockName;
 		pIcon->cParentDockName = NULL;  // astuce.
-		cairo_dock_detach_icon_from_dock (pIcon, pDock, myIconsParam.iSeparateIcons);
+		cairo_dock_detach_icon_from_dock (pIcon, pDock);
 		if (! pDock->bIsMainDock)  // la taille du main dock est mis a jour 1 fois a la fin.
 		{
 			if (pDock->icons == NULL)  // le dock degage, le fake aussi.
@@ -929,7 +985,7 @@ static gboolean _cairo_dock_remove_one_appli (Window *pXid, Icon *pIcon, gpointe
 				if (CAIRO_DOCK_ICON_TYPE_IS_CLASS_CONTAINER (pFakeClassIcon)) // fake launcher
 				{
 					cd_debug ("on degage le fake qui pointe sur %s", cParentDockName);
-					cairo_dock_detach_icon_from_dock (pFakeClassIcon, pFakeClassParentDock, myIconsParam.iSeparateIcons);
+					cairo_dock_detach_icon_from_dock (pFakeClassIcon, pFakeClassParentDock);
 					cairo_dock_free_icon (pFakeClassIcon);
 					if (! pFakeClassParentDock->bIsMainDock)
 						cairo_dock_update_dock_size (pFakeClassParentDock);
@@ -937,8 +993,8 @@ static gboolean _cairo_dock_remove_one_appli (Window *pXid, Icon *pIcon, gpointe
 				
 				cairo_dock_destroy_dock (pDock, cParentDockName);
 			}
-			else
-				cairo_dock_update_dock_size (pDock);
+			///else
+			///	cairo_dock_update_dock_size (pDock);
 		}
 		g_free (cParentDockName);
 	}
@@ -961,7 +1017,7 @@ static void _cairo_dock_stop_application_manager (void)
 	cairo_dock_remove_all_applis_from_class_table ();  // enleve aussi les indicateurs.
 	
 	g_hash_table_foreach_remove (s_hXWindowTable, (GHRFunc) _cairo_dock_remove_one_appli, NULL);  // libere toutes les icones d'appli.
-	cairo_dock_update_dock_size (g_pMainDock);
+	///cairo_dock_update_dock_size (g_pMainDock);
 	
 	cairo_dock_foreach_root_docks ((GFunc)_unhide_all_docks, NULL);
 }
@@ -982,7 +1038,7 @@ static void _cairo_dock_stop_application_manager (void)
 	CairoDock *pDock = cairo_dock_search_dock_from_name (pIcon->cParentDockName);
 	if (pDock != NULL)
 	{
-		cairo_dock_detach_icon_from_dock (pIcon, pDock, FALSE);
+		cairo_dock_detach_icon_from_dock_full (pIcon, pDock, FALSE);
 	}
 	
 	// make it an invalid appli, and free it.
@@ -1128,7 +1184,7 @@ void cairo_dock_set_icons_geometry_for_window_manager (CairoDock *pDock)
 {
 	if (! s_bAppliManagerIsRunning)
 		return ;
-	cd_debug ("%s (main:%d)", __func__, pDock->bIsMainDock);
+	//g_print ("%s (main:%d, ref:%d)\n", __func__, pDock->bIsMainDock, pDock->iRefCount);
 	
 	/*long *data = g_new0 (long, 1+6*g_list_length (pDock->icons));
 	int i = 0;*/
@@ -1314,7 +1370,7 @@ static void _load_appli (Icon *icon)
 		// create the thumbnail (window preview).
 		if (g_bUseOpenGL)
 		{
-			icon->iIconTexture = cairo_dock_texture_from_pixmap (icon->Xid, icon->iBackingPixmap);
+			///icon->iIconTexture = cairo_dock_texture_from_pixmap (icon->Xid, icon->iBackingPixmap);  // doesn't work any more since Compiz 0.9 :-(
 		}
 		if (icon->iIconTexture == 0)
 		{
@@ -1328,28 +1384,30 @@ static void _load_appli (Icon *icon)
 		if (icon->iIconTexture != 0 && iPrevTexture != 0)
 		{
 			CairoDock *pParentDock = cairo_dock_search_dock_from_name (icon->cParentDockName);
-			CairoEmblem *e = cairo_dock_make_emblem_from_texture (iPrevTexture,icon);
+			cairo_dock_print_overlay_on_icon_from_texture (icon, CAIRO_CONTAINER (pParentDock), iPrevTexture, CAIRO_OVERLAY_LOWER_LEFT);
+			/**CairoEmblem *e = cairo_dock_make_emblem_from_texture (iPrevTexture,icon);
 			cairo_dock_set_emblem_position (e, CAIRO_DOCK_EMBLEM_LOWER_LEFT);
 			cairo_dock_draw_emblem_on_icon (e, icon, CAIRO_CONTAINER (pParentDock));
-			g_free (e);  // on n'utilise pas cairo_dock_free_emblem pour ne pas detruire la texture avec.
+			g_free (e);  // on n'utilise pas cairo_dock_free_emblem pour ne pas detruire la texture avec.*/
 		}
 		else if (icon->pIconBuffer != NULL && pPrevSurface != NULL)
 		{
 			CairoDock *pParentDock = cairo_dock_search_dock_from_name (icon->cParentDockName);
-			CairoEmblem *e = cairo_dock_make_emblem_from_surface (pPrevSurface, 0, 0, icon);
+			cairo_dock_print_overlay_on_icon_from_surface (icon, CAIRO_CONTAINER (pParentDock), pPrevSurface, 0, 0, CAIRO_OVERLAY_LOWER_LEFT);
+			/**CairoEmblem *e = cairo_dock_make_emblem_from_surface (pPrevSurface, 0, 0, icon);
 			cairo_dock_set_emblem_position (e, CAIRO_DOCK_EMBLEM_LOWER_LEFT);
 			cairo_dock_draw_emblem_on_icon (e, icon, CAIRO_CONTAINER (pParentDock));
-			g_free (e);  // meme remarque.
+			g_free (e);  // meme remarque.*/
 		}
 	}
 	// or use the class icon
-	if (icon->pIconBuffer == NULL && myTaskbarParam.bOverWriteXIcons && ! cairo_dock_class_is_using_xicon (icon->cClass))
+	if (icon->iIconTexture == 0 && icon->pIconBuffer == NULL && myTaskbarParam.bOverWriteXIcons && ! cairo_dock_class_is_using_xicon (icon->cClass))
 		icon->pIconBuffer = cairo_dock_create_surface_from_class (icon->cClass, iWidth, iHeight);
 	// or use the X icon
-	if (icon->pIconBuffer == NULL)
+	if (icon->iIconTexture == 0 && icon->pIconBuffer == NULL)
 		icon->pIconBuffer = cairo_dock_create_surface_from_xwindow (icon->Xid, iWidth, iHeight);
 	// or use a default image
-	if (icon->pIconBuffer == NULL)  // certaines applis comme xterm ne definissent pas d'icone, on en met une par defaut.
+	if (icon->iIconTexture == 0 && icon->pIconBuffer == NULL)  // certaines applis comme xterm ne definissent pas d'icone, on en met une par defaut.
 	{
 		cd_debug ("%s (%ld) doesn't define any icon, we set the default one.", icon->cName, icon->Xid);
 		gchar *cIconPath = cairo_dock_search_image_s_path (CAIRO_DOCK_DEFAULT_APPLI_ICON_NAME);
@@ -1420,15 +1478,16 @@ static Icon * cairo_dock_create_icon_from_xwindow (Window Xid, CairoDock *pDock)
 		}
 		#endif
 		
-		if (pDock)
+		/**if (pDock)
 		{
-			cairo_dock_trigger_load_icon_buffers (icon, CAIRO_CONTAINER (pDock));
-		}
+			cairo_dock_trigger_load_icon_buffers (icon);
+		}*/
 	}
 	
 	//\____________ On enregistre l'appli et on commence a la surveiller.
+	icon->iAge = s_iNumWindow ++;
 	cairo_dock_register_appli (icon);
-
+	
 	return icon;
 }
 
@@ -1461,6 +1520,8 @@ static gboolean get_config (GKeyFile *pKeyFile, CairoTaskbarParam *pTaskBar)
 		
 		pTaskBar->bHideVisibleApplis = cairo_dock_get_boolean_key_value (pKeyFile, "TaskBar", "hide visible", &bFlushConfFileNeeded, FALSE, "Applications", NULL);
 		
+		pTaskBar->iIconPlacement = cairo_dock_get_integer_key_value (pKeyFile, "TaskBar", "place icons", &bFlushConfFileNeeded, CAIRO_APPLI_AFTER_LAST_LAUNCHER, NULL, NULL);  // after the last launcher by default.
+		pTaskBar->cRelativeIconName = cairo_dock_get_string_key_value (pKeyFile, "TaskBar", "relative icon", &bFlushConfFileNeeded, NULL, NULL, NULL);
 		
 		// representation
 		pTaskBar->bOverWriteXIcons = cairo_dock_get_boolean_key_value (pKeyFile, "TaskBar", "overwrite xicon", &bFlushConfFileNeeded, TRUE, NULL, NULL);
@@ -1527,6 +1588,7 @@ static void reset_config (CairoTaskbarParam *pTaskBar)
 	g_free (pTaskBar->cOverwriteException);
 	g_free (pTaskBar->cGroupException);
 	g_free (pTaskBar->cForceDemandsAttention);
+	g_free (pTaskBar->cRelativeIconName);
 }
 
   ////////////
@@ -1548,36 +1610,32 @@ static void reload (CairoTaskbarParam *pPrevTaskBar, CairoTaskbarParam *pTaskBar
 	CairoDock *pDock = g_pMainDock;
 	
 	gboolean bUpdateSize = FALSE;
-	if (pPrevTaskBar->bGroupAppliByClass != pTaskBar->bGroupAppliByClass ||
-		pPrevTaskBar->bHideVisibleApplis != pTaskBar->bHideVisibleApplis ||
-		pPrevTaskBar->bAppliOnCurrentDesktopOnly != pTaskBar->bAppliOnCurrentDesktopOnly ||
-		pPrevTaskBar->bMixLauncherAppli != pTaskBar->bMixLauncherAppli ||
-		pPrevTaskBar->bOverWriteXIcons != pTaskBar->bOverWriteXIcons ||
-		pPrevTaskBar->iMinimizedWindowRenderType != pTaskBar->iMinimizedWindowRenderType ||
-		pPrevTaskBar->iAppliMaxNameLength != pTaskBar->iAppliMaxNameLength ||
-		cairo_dock_strings_differ (pPrevTaskBar->cGroupException, pTaskBar->cGroupException) ||
-		cairo_dock_strings_differ (pPrevTaskBar->cOverwriteException, pTaskBar->cOverwriteException) ||
-		pPrevTaskBar->bShowAppli != pTaskBar->bShowAppli)
+	if (pPrevTaskBar->bGroupAppliByClass != pTaskBar->bGroupAppliByClass
+		|| pPrevTaskBar->bHideVisibleApplis != pTaskBar->bHideVisibleApplis
+		|| pPrevTaskBar->bAppliOnCurrentDesktopOnly != pTaskBar->bAppliOnCurrentDesktopOnly
+		|| pPrevTaskBar->bMixLauncherAppli != pTaskBar->bMixLauncherAppli
+		|| pPrevTaskBar->bOverWriteXIcons != pTaskBar->bOverWriteXIcons
+		|| pPrevTaskBar->iMinimizedWindowRenderType != pTaskBar->iMinimizedWindowRenderType
+		|| pPrevTaskBar->iAppliMaxNameLength != pTaskBar->iAppliMaxNameLength
+		|| cairo_dock_strings_differ (pPrevTaskBar->cGroupException, pTaskBar->cGroupException)
+		|| cairo_dock_strings_differ (pPrevTaskBar->cOverwriteException, pTaskBar->cOverwriteException)
+		|| pPrevTaskBar->bShowAppli != pTaskBar->bShowAppli
+		|| pPrevTaskBar->iIconPlacement != pTaskBar->iIconPlacement
+		|| cairo_dock_strings_differ (pPrevTaskBar->cRelativeIconName, pTaskBar->cRelativeIconName))
 	{
 		_cairo_dock_stop_application_manager ();
-		bUpdateSize = TRUE;
-	}
-	
-	if (! s_bAppliManagerIsRunning)  // maintenant on veut voir les applis !
-	{
-		cairo_dock_start_applications_manager (pDock);  // va inserer le separateur si necessaire.
-		bUpdateSize = TRUE;
-	}
-	else
-		gtk_widget_queue_draw (pDock->container.pWidget);  // pour le fVisibleAlpha
-	
-	if (bUpdateSize)
-	{
+		
+		cairo_dock_start_applications_manager (pDock);
+		
 		cairo_dock_calculate_dock_icons (pDock);
 		gtk_widget_queue_draw (pDock->container.pWidget);  // le 'gdk_window_move_resize' ci-dessous ne provoquera pas le redessin si la taille n'a pas change.
 		
 		cairo_dock_move_resize_dock (pDock);
 	}
+	else
+	{
+		gtk_widget_queue_draw (pDock->container.pWidget);  // pour le fVisibleAlpha
+	}
 }
 
 
@@ -1624,6 +1682,8 @@ static void unload (void)
 	
 	// empty the applis table.
 	g_hash_table_foreach_remove (s_hXWindowTable, (GHRFunc) _remove_appli, NULL);
+	
+	s_bAppliManagerIsRunning = FALSE;
 }
 
 
