@@ -20,6 +20,7 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "cairo-dock-notifications.h"
 #include "cairo-dock-draw.h"
 #include "cairo-dock-icon-factory.h"
 #include "cairo-dock-callbacks.h"
@@ -273,7 +274,7 @@ void cairo_dock_set_renderer (CairoDock *pDock, const gchar *cRendererName)
 		pDock->container.iSidGLAnimation = 0;
 		cairo_dock_launch_animation (CAIRO_CONTAINER (pDock));
 	}
-	if (/**cRendererName != NULL && */pDock->cRendererName != cRendererName)  // NULL ecrase le nom de l'ancienne vue depuis le 12/11/2009
+	if (pDock->cRendererName != cRendererName)  // NULL ecrase le nom de l'ancienne vue.
 	{
 		g_free (pDock->cRendererName);
 		pDock->cRendererName = g_strdup (cRendererName);
@@ -312,7 +313,7 @@ void cairo_dock_set_desklet_renderer (CairoDesklet *pDesklet, CairoDeskletRender
 		
 		Icon* pIcon = pDesklet->pIcon;
 		if (pIcon)
-			cairo_dock_load_icon_buffers (pIcon, CAIRO_CONTAINER (pDesklet));
+			cairo_dock_load_icon_buffers (pIcon, CAIRO_CONTAINER (pDesklet));  // immediately, because the applet may need it to draw its stuff.
 		
 		GList* ic;
 		for (ic = pDesklet->icons; ic != NULL; ic = ic->next)
@@ -320,7 +321,7 @@ void cairo_dock_set_desklet_renderer (CairoDesklet *pDesklet, CairoDeskletRender
 			pIcon = ic->data;
 			pIcon->iImageWidth = pIcon->fWidth;
 			pIcon->iImageHeight = pIcon->fHeight;
-			cairo_dock_trigger_load_icon_buffers (pIcon, CAIRO_CONTAINER (pDesklet));
+			cairo_dock_trigger_load_icon_buffers (pIcon);
 		}
 		
 		if (pRenderer->load_data != NULL)
