@@ -24,14 +24,9 @@
 
 #include <cairo.h>
 
-#ifdef HAVE_GLITZ
-#include <glitz-glx.h>
-#include <cairo-glitz.h>
-#endif
-
-#include <gtk/gtkgl.h>
 #include <GL/gl.h> 
 
+#include "cairo-dock-notifications.h"
 #include "cairo-dock-icon-facility.h"  // cairo_dock_compute_icon_area
 #include "cairo-dock-dock-facility.h"  // cairo_dock_is_hidden
 #include "cairo-dock-module-factory.h"  // cairo_dock_search_container_from_icon
@@ -64,12 +59,6 @@ extern CairoDockDesktopGeometry g_desktopGeometry;  // _place_menu_on_icon
 static gboolean s_bSticky = TRUE;
 
 
-static gboolean _cairo_dock_on_delete (GtkWidget *pWidget, GdkEvent *event, gpointer data)
-{
-	cd_debug ("pas de alt+f4");
-	return TRUE;  // on empeche les ALT+F4 malheureux.
-}
-
 void cairo_dock_set_containers_non_sticky (void)
 {
 	if (g_pPrimaryContainer != NULL)
@@ -80,41 +69,129 @@ void cairo_dock_set_containers_non_sticky (void)
 	s_bSticky = FALSE;
 }
 
+
+static gboolean _prevent_delete (GtkWidget *pWidget, GdkEvent *event, gpointer data)
+{
+	cd_debug ("pas de alt+f4");
+	return TRUE;  // on empeche les ALT+F4 malheureux.
+}
+
+static void cairo_dock_set_default_rgba_visual (GtkWidget *pWidget)
+{
+	GdkScreen* pScreen = gtk_widget_get_screen (pWidget);
+	
+	#if (GTK_MAJOR_VERSION < 3)
+	GdkColormap* pColormap = gdk_screen_get_rgba_colormap (pScreen);
+	if (!pColormap)
+		pColormap = gdk_screen_get_rgb_colormap (pScreen);
+	
+	gtk_widget_set_colormap (pWidget, pColormap);
+	#else
+	GdkVisual *pGdkVisual = gdk_screen_get_rgba_visual (pScreen);
+	if (pGdkVisual == NULL)
+		pGdkVisual = gdk_screen_get_system_visual (pScreen);
+	
+	gtk_widget_set_visual (pWidget, pGdkVisual);
+	#endif
+}
+
+static gboolean _cairo_default_container_animation_loop (CairoContainer *pContainer)
+{
+	gboolean bContinue = FALSE;
+	
+	gboolean bUpdateSlowAnimation = FALSE;
+	pContainer->iAnimationStep ++;
+	if (pContainer->iAnimationStep * pContainer->iAnimationDeltaT >= CAIRO_DOCK_MIN_SLOW_DELTA_T)
+	{
+		bUpdateSlowAnimation = TRUE;
+		pContainer->iAnimationStep = 0;
+		pContainer->bKeepSlowAnimation = FALSE;
+	}
+	
+	if (bUpdateSlowAnimation)
+	{
+		cairo_dock_notify_on_object (pContainer, NOTIFICATION_UPDATE_SLOW, pContainer, &pContainer->bKeepSlowAnimation);
+	}
+	
+	cairo_dock_notify_on_object (pContainer, NOTIFICATION_UPDATE, pContainer, &bContinue);
+	
+	if (! bContinue && ! pContainer->bKeepSlowAnimation)
+	{
+		pContainer->iSidGLAnimation = 0;
+		return FALSE;
+	}
+	else
+		return TRUE;
+}
+
+static gboolean _set_opacity (GtkWidget *pWidget,
+#if (GTK_MAJOR_VERSION < 3)
+	GdkEventExpose *pExpose,
+#else
+	cairo_t *ctx,
+#endif
+	CairoContainer *pContainer)
+{
+	if (pContainer->iWidth != 1 ||pContainer->iHeight != 1)
+	{
+		g_signal_handlers_disconnect_by_func (pWidget, _set_opacity, pContainer);  // we'll never need to pass here any more, so simply disconnect ourselves.
+		//g_print ("____OPACITY 1 (%dx%d)\n", pContainer->iWidth, pContainer->iHeight);
+		gtk_window_set_opacity (GTK_WINDOW (pWidget), 1.);
+	}
+	return FALSE ;
+}
 GtkWidget *cairo_dock_init_container_full (CairoContainer *pContainer, gboolean bOpenGLWindow)
 {
 	GtkWidget* pWindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+	pContainer->pWidget = pWindow;
 	
-	if (s_bSticky)
-		gtk_window_stick (GTK_WINDOW (pWindow));
+	if (!pContainer->iface.animation_loop)
+		pContainer->iface.animation_loop = _cairo_default_container_animation_loop;
+	
+	if (g_bUseOpenGL && bOpenGLWindow)
+	{
+		gldi_glx_init_container (pContainer);
+		pContainer->iAnimationDeltaT = myContainersParam.iGLAnimationDeltaT;
+	}
+	else
+	{
+		cairo_dock_set_default_rgba_visual (pWindow);
+		pContainer->iAnimationDeltaT = myContainersParam.iCairoAnimationDeltaT;
+	}
+	if (pContainer->iAnimationDeltaT == 0)
+		pContainer->iAnimationDeltaT = 30;
+	pContainer->fRatio = 1;
+	
+	g_signal_connect (G_OBJECT (pWindow),
+		"delete-event",
+		G_CALLBACK (_prevent_delete),
+		NULL);
+	
+	gtk_window_set_default_size (GTK_WINDOW (pWindow), 1, 1);  // this should prevent having grey rectangles during the loading, when the window is mapped and rendered by the WM but not yet by us.
+	gtk_window_resize (GTK_WINDOW (pWindow), 1, 1);
+	gtk_widget_set_app_paintable (pWindow, TRUE);
+	gtk_window_set_decorated (GTK_WINDOW (pWindow), FALSE);
 	gtk_window_set_skip_pager_hint (GTK_WINDOW(pWindow), TRUE);
 	gtk_window_set_skip_taskbar_hint (GTK_WINDOW(pWindow), TRUE);
-
+	if (s_bSticky)
+		gtk_window_stick (GTK_WINDOW (pWindow));
+	// set the opacity to 0 to avoid seeing grey rectangles until the window is ready to be painted by us.
+	gtk_window_set_opacity (GTK_WINDOW (pWindow), 0.);
+	g_signal_connect (G_OBJECT (pWindow),
+		#if (GTK_MAJOR_VERSION < 3)
+		"expose-event",
+		#else
+		"draw",
+		#endif
+		G_CALLBACK (_set_opacity),
+		pContainer);  // the callback will be removed once it has done its job.
+	
 	// needed since gtk+-3.0 but it's possible that this resize grip has been backported to gtk+-2.0 (e.g. in Ubuntu Natty...)
 	#if (GTK_MAJOR_VERSION >= 3 || ENABLE_GTK_GRIP == 1)
 	gtk_window_set_has_resize_grip (GTK_WINDOW(pWindow), FALSE);
 	#endif
 	
-	cairo_dock_set_colormap_for_window (pWindow);
-	if (g_bUseOpenGL && bOpenGLWindow)
-	{
-		cairo_dock_set_gl_capabilities (pWindow);
-		pContainer->iAnimationDeltaT = myContainersParam.iGLAnimationDeltaT;
-	}
-	else
-		pContainer->iAnimationDeltaT = myContainersParam.iCairoAnimationDeltaT;
-	if (pContainer->iAnimationDeltaT == 0)
-		pContainer->iAnimationDeltaT = 30;
-	
-	g_signal_connect (G_OBJECT (pWindow),
-		"delete-event",
-		G_CALLBACK (_cairo_dock_on_delete),
-		NULL);
-	
-	gtk_widget_set_app_paintable (pWindow, TRUE);
-	gtk_window_set_decorated (GTK_WINDOW (pWindow), FALSE);
-	pContainer->pWidget = pWindow;
-	
-	cairo_dock_install_notifications_on_object (pContainer, NB_NOTIFICATIONS_CONTAINER);  // l'implementation du container installera par-dessus ses notifications.
+	gldi_object_set_manager (GLDI_OBJECT (pContainer), GLDI_MANAGER (&myContainersMgr));  // the implementation of the container will set its manager on top of this one.
 	
 	if (g_pPrimaryContainer == NULL)
 	{
@@ -125,6 +202,8 @@ GtkWidget *cairo_dock_init_container_full (CairoContainer *pContainer, gboolean
 
 void cairo_dock_finish_container (CairoContainer *pContainer)
 {
+	gldi_glx_finish_container (pContainer);
+	
 	gtk_widget_destroy (pContainer->pWidget);  // enleve les signaux.
 	pContainer->pWidget = NULL;
 	if (pContainer->iSidGLAnimation != 0)
@@ -132,101 +211,13 @@ void cairo_dock_finish_container (CairoContainer *pContainer)
 		g_source_remove (pContainer->iSidGLAnimation);
 		pContainer->iSidGLAnimation = 0;
 	}
+	cairo_dock_notify_on_object (pContainer, NOTIFICATION_DESTROY, pContainer);
 	cairo_dock_clear_notifications_on_object (pContainer);
-	pContainer->pNotificationsTab = NULL;
+	
 	if (g_pPrimaryContainer == pContainer)
 		g_pPrimaryContainer = NULL;
-	else if (g_bUseOpenGL && g_pPrimaryContainer != NULL)
-		cairo_dock_set_default_gl_context ();
 }
 
-/* Apply the scren colormap to a window, providing it transparency.
-*@param pWidget a GTK window.
-*/
-void cairo_dock_set_colormap_for_window (GtkWidget *pWidget)
-{
-	GdkScreen* pScreen = gtk_widget_get_screen (pWidget);
-	GdkColormap* pColormap = gdk_screen_get_rgba_colormap (pScreen);
-	if (!pColormap)
-		pColormap = gdk_screen_get_rgb_colormap (pScreen);
-	
-	/// est-ce que ca vaut le coup de plutot faire ca avec le visual obtenu pour l'openGL ?...
-	//GdkVisual *visual = gdkx_visual_get (pVisInfo->visualid);
-	//pColormap = gdk_colormap_new (visual, TRUE);
-
-	gtk_widget_set_colormap (pWidget, pColormap);
-}
-
-/* Apply the scren colormap to a container, providing it transparency, and activate Glitz if possible.
-* @param pContainer the container.
-*/
-void cairo_dock_set_colormap (CairoContainer *pContainer)
-{
-	GdkColormap* pColormap;
-#ifdef HAVE_GLITZ
-	if (g_bUseGlitz)
-	{
-		glitz_drawable_format_t templ, *format;
-		unsigned long	    mask = GLITZ_FORMAT_DOUBLEBUFFER_MASK;
-		XVisualInfo		    *vinfo = NULL;
-		int			    screen = 0;
-		GdkVisual		    *visual;
-		GdkDisplay		    *gdkdisplay;
-		Display		    *xdisplay;
-
-		templ.doublebuffer = 1;
-		gdkdisplay = gtk_widget_get_display (pContainer->pWidget);
-		xdisplay   = gdk_x11_display_get_xdisplay (gdkdisplay);
-
-		int i = 0;
-		do
-		{
-			format = glitz_glx_find_window_format (xdisplay,
-				screen,
-				mask,
-				&templ,
-				i++);
-			if (format)
-			{
-				vinfo = glitz_glx_get_visual_info_from_format (xdisplay,
-					screen,
-					format);
-				if (vinfo->depth == 32)
-				{
-					pContainer->pDrawFormat = format;
-					break;
-				}
-				else if (!pContainer->pDrawFormat)
-				{
-					pContainer->pDrawFormat = format;
-				}
-			}
-		} while (format);
-
-		if (! pContainer->pDrawFormat)
-		{
-			cd_warning ("no double buffered GLX visual");
-		}
-		else
-		{
-			vinfo = glitz_glx_get_visual_info_from_format (xdisplay,
-				screen,
-				pContainer->pDrawFormat);
-
-			visual = gdkx_visual_get (vinfo->visualid);
-			pColormap = gdk_colormap_new (visual, TRUE);
-
-			gtk_widget_set_colormap (pContainer->pWidget, pColormap);
-			gtk_widget_set_double_buffered (pContainer->pWidget, FALSE);
-			return ;
-		}
-	}
-#endif
-	
-	cairo_dock_set_colormap_for_window (pContainer->pWidget);
-}
-
-
 
 void cairo_dock_redraw_container (CairoContainer *pContainer)
 {
@@ -243,7 +234,7 @@ void cairo_dock_redraw_container (CairoContainer *pContainer)
 static inline void _redraw_container_area (CairoContainer *pContainer, GdkRectangle *pArea)
 {
 	g_return_if_fail (pContainer != NULL);
-	if (! GTK_WIDGET_VISIBLE (pContainer->pWidget))
+	if (! gldi_container_is_visible (pContainer))
 		return ;
 	
 	if (pArea->y < 0)
@@ -254,7 +245,7 @@ static inline void _redraw_container_area (CairoContainer *pContainer, GdkRectan
 		pArea->width = pContainer->iHeight - pArea->x;
 	
 	if (pArea->width > 0 && pArea->height > 0)
-		gdk_window_invalidate_rect (pContainer->pWidget->window, pArea, FALSE);
+		gdk_window_invalidate_rect (gldi_container_get_gdk_window (pContainer), pArea, FALSE);
 }
 
 void cairo_dock_redraw_container_area (CairoContainer *pContainer, GdkRectangle *pArea)
@@ -272,7 +263,7 @@ void cairo_dock_redraw_icon (Icon *icon, CairoContainer *pContainer)
 	
 	if (CAIRO_DOCK_IS_DOCK (pContainer) &&
 		( (cairo_dock_is_hidden (CAIRO_DOCK (pContainer)) && ! icon->bIsDemandingAttention && ! icon->bAlwaysVisible)
-		|| (CAIRO_DOCK (pContainer)->iRefCount != 0 && ! GTK_WIDGET_VISIBLE (pContainer->pWidget)) ) )  // inutile de redessiner.
+		|| (CAIRO_DOCK (pContainer)->iRefCount != 0 && ! gldi_container_is_visible (pContainer)) ) )  // inutile de redessiner.
 		return ;
 	_redraw_container_area (pContainer, &rect);
 }
@@ -343,9 +334,9 @@ void cairo_dock_allow_widget_to_receive_data (GtkWidget *pWidget, GCallback pCal
 		data);
 }
 
-void cairo_dock_disallow_widget_to_receive_data (GtkWidget *pWidget)
+void gldi_container_disable_drop (CairoContainer *pContainer)
 {
-	gtk_drag_dest_set_target_list (pWidget, NULL);
+	gtk_drag_dest_set_target_list (pContainer->pWidget, NULL);
 }
 
 gboolean cairo_dock_string_is_adress (const gchar *cString)
@@ -407,7 +398,6 @@ void cairo_dock_notify_drop_data (gchar *cReceivedData, Icon *pPointedIcon, doub
 		
 		cData = sArg->str;
 		cd_debug (" notification de drop '%s'", cData);
-		cairo_dock_notify_on_object (&myContainersMgr, NOTIFICATION_DROP_DATA, cData, pPointedIcon, fOrder, pContainer);
 		cairo_dock_notify_on_object (pContainer, NOTIFICATION_DROP_DATA, cData, pPointedIcon, fOrder, pContainer);
 	}
 	
@@ -425,10 +415,7 @@ gboolean cairo_dock_emit_signal_on_container (CairoContainer *pContainer, const
 gboolean cairo_dock_emit_leave_signal (CairoContainer *pContainer)
 {
 	// actualize the coordinates of the pointer, since they are most probably out-dated (because the mouse has left the dock, or because a right-click generates an event with (0;0) coordinates)
-	if (pContainer->bIsHorizontal)
-		gdk_window_get_pointer (pContainer->pWidget->window, &pContainer->iMouseX, &pContainer->iMouseY, NULL);
-	else
-		gdk_window_get_pointer (pContainer->pWidget->window, &pContainer->iMouseY, &pContainer->iMouseX, NULL);
+	gldi_container_update_mouse_position (pContainer);
 	return cairo_dock_emit_signal_on_container (pContainer, "leave-notify-event");
 }
 gboolean cairo_dock_emit_enter_signal (CairoContainer *pContainer)
@@ -437,14 +424,6 @@ gboolean cairo_dock_emit_enter_signal (CairoContainer *pContainer)
 }
 
 
-static void _cairo_dock_delete_menu (GtkMenuShell *menu, CairoDock *pDock)
-{
-	g_return_if_fail (CAIRO_DOCK_IS_DOCK (pDock));
-	pDock->bMenuVisible = FALSE;
-	
-	///pDock->container.bInside = TRUE;
-	cairo_dock_emit_leave_signal (CAIRO_CONTAINER (pDock));
-}
 static void _place_menu_on_icon (GtkMenu *menu, gint *x, gint *y, gboolean *push_in, gpointer *data)
 {
 	*push_in = TRUE;
@@ -455,7 +434,11 @@ static void _place_menu_on_icon (GtkMenu *menu, gint *x, gint *y, gboolean *push
 	
 	int w, h;  // taille menu
 	GtkRequisition requisition;
+	#if (GTK_MAJOR_VERSION < 3)
 	gtk_widget_size_request (GTK_WIDGET (menu), &requisition);
+	#else
+	gtk_widget_get_preferred_size (GTK_WIDGET (menu), &requisition, NULL);
+	#endif
 	w = requisition.width;
 	h = requisition.height;
 	
@@ -479,37 +462,20 @@ static void _place_menu_on_icon (GtkMenu *menu, gint *x, gint *y, gboolean *push
 }
 void cairo_dock_popup_menu_on_icon (GtkWidget *menu, Icon *pIcon, CairoContainer *pContainer)
 {
-	static gpointer *data = NULL;  // 1 seul menu a la fois, donc on peut la faire statique.
+	static gpointer data[2];  // 1 seul menu a la fois, donc on peut la faire statique.
 	
 	if (menu == NULL)
 		return;
-	GtkMenuPositionFunc place_menu = NULL;
+	GtkMenuPositionFunc place_menu = NULL;  // if 'place_menu' is NULL, then 'data' is ignored.
 	if (pIcon != NULL && pContainer != NULL)
 	{
 		place_menu = (GtkMenuPositionFunc)_place_menu_on_icon;
-		if (data == NULL)
-			data = g_new0 (gpointer, 2);
 		data[0] = pIcon;
 		data[1] = pContainer;
 	}
 	
-	if (CAIRO_DOCK_IS_DOCK (pContainer))
-	{
-		if (g_signal_handler_find (menu,
-			G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA,
-			0,
-			0,
-			NULL,
-			_cairo_dock_delete_menu,
-			pContainer) == 0)  // on evite de connecter 2 fois ce signal, donc la fonction est appelable plusieurs fois sur un meme menu.
-		{
-			g_signal_connect (G_OBJECT (menu),
-				"deactivate",
-				G_CALLBACK (_cairo_dock_delete_menu),
-				pContainer);
-		}
-		CAIRO_DOCK (pContainer)->bMenuVisible = TRUE;
-	}
+	if (pContainer->iface.setup_menu)
+		pContainer->iface.setup_menu (pContainer, pIcon, menu);
 	
 	gtk_widget_show_all (GTK_WIDGET (menu));
 	
@@ -523,7 +489,7 @@ void cairo_dock_popup_menu_on_icon (GtkWidget *menu, Icon *pIcon, CairoContainer
 }
 
 
-GtkWidget *cairo_dock_add_in_menu_with_stock_and_data (const gchar *cLabel, const gchar *gtkStock, GFunc pFunction, GtkWidget *pMenu, gpointer pData)
+GtkWidget *cairo_dock_add_in_menu_with_stock_and_data (const gchar *cLabel, const gchar *gtkStock, GCallback pFunction, GtkWidget *pMenu, gpointer pData)
 {
 	GtkWidget *pMenuItem = gtk_image_menu_item_new_with_label (cLabel);
 	if (gtkStock)
@@ -555,7 +521,7 @@ GtkWidget *cairo_dock_create_sub_menu (const gchar *cLabel, GtkWidget *pMenu, co
 	GtkWidget *pMenuItem, *image, *pSubMenu = gtk_menu_new ();
 	if (cImage == NULL)
 	{
-		pMenuItem = gtk_menu_item_new_with_label (cLabel);
+		pMenuItem = gtk_image_menu_item_new_with_label (cLabel);
 	}
 	else
 	{
@@ -581,20 +547,13 @@ GtkWidget *cairo_dock_create_sub_menu (const gchar *cLabel, GtkWidget *pMenu, co
 }
 
 
-static GtkWidget *s_pMenu = NULL;
-static gboolean _on_destroy_menu (GtkWidget *widget, GdkEvent  *event, gpointer   user_data)
-{
-	cd_debug ("*** menu destroyed");
-	s_pMenu = NULL;
-	return FALSE;
-}
+static GtkWidget *s_pMenu = NULL;  // right-click menu
 GtkWidget *cairo_dock_build_menu (Icon *icon, CairoContainer *pContainer)
 {
 	if (s_pMenu != NULL)
 	{
-		cd_debug ("previous menu still alive");
-		gtk_widget_destroy (GTK_WIDGET (s_pMenu));
-		s_pMenu = NULL;
+		//g_print ("previous menu still alive\n");
+		gtk_widget_destroy (GTK_WIDGET (s_pMenu));  // -> 's_pMenu' becomes NULL thanks to the weak pointer.
 	}
 	g_return_val_if_fail (pContainer != NULL, NULL);
 	
@@ -603,7 +562,6 @@ GtkWidget *cairo_dock_build_menu (Icon *icon, CairoContainer *pContainer)
 	
 	//\_________________________ On passe la main a ceux qui veulent y rajouter des choses.
 	gboolean bDiscardMenu = FALSE;
-	cairo_dock_notify_on_object (&myContainersMgr, NOTIFICATION_BUILD_CONTAINER_MENU, icon, pContainer, menu, &bDiscardMenu);
 	cairo_dock_notify_on_object (pContainer, NOTIFICATION_BUILD_CONTAINER_MENU, icon, pContainer, menu, &bDiscardMenu);
 	if (bDiscardMenu)
 	{
@@ -611,15 +569,57 @@ GtkWidget *cairo_dock_build_menu (Icon *icon, CairoContainer *pContainer)
 		return NULL;
 	}
 	
-	cairo_dock_notify_on_object (&myContainersMgr, NOTIFICATION_BUILD_ICON_MENU, icon, pContainer, menu);
 	cairo_dock_notify_on_object (pContainer, NOTIFICATION_BUILD_ICON_MENU, icon, pContainer, menu);
-	g_signal_connect (G_OBJECT (menu), "destroy", G_CALLBACK (_on_destroy_menu), NULL);  // apparemment inutile.
 	
 	s_pMenu = menu;
+	g_object_add_weak_pointer (G_OBJECT (menu), (gpointer*)&s_pMenu);  // will nullify 's_pMenu' as soon as the menu is destroyed.
 	return menu;
 }
 
 
+GldiShape *gldi_container_create_input_shape (CairoContainer *pContainer, int x, int y, int w, int h)
+{
+	if (pContainer->iWidth == 0 || pContainer->iHeight == 0)  // very unlikely to happen, but anyway avoid this case.
+		return NULL;
+	
+	#if (GTK_MAJOR_VERSION < 3)
+	int W, H;
+	if (pContainer->bIsHorizontal)
+	{
+		W = pContainer->iWidth;
+		H = pContainer->iHeight;
+	}
+	else
+	{
+		W = pContainer->iHeight;
+		H = pContainer->iWidth;
+	}
+	GdkBitmap *pShapeBitmap = (GdkBitmap*) gdk_pixmap_new (NULL,
+		W,
+		H,
+		1);
+
+	cairo_t *pCairoContext = gdk_cairo_create (pShapeBitmap);
+	g_return_val_if_fail (pCairoContext != NULL, NULL);  // if no context, abort (https://bugs.launchpad.net/cairo-dock-plug-ins/+bug/861725)
+	cairo_set_source_rgba (pCairoContext, 0.0f, 0.0f, 0.0f, 0.0f);
+	cairo_set_operator (pCairoContext, CAIRO_OPERATOR_SOURCE);
+	cairo_paint (pCairoContext);
+	
+	cairo_set_source_rgba (pCairoContext, 1., 1., 1., 1.);
+	cairo_rectangle (pCairoContext,
+		x,
+		y,
+		w,
+		h);
+	cairo_fill (pCairoContext);
+	cairo_destroy (pCairoContext);
+	#else
+	cairo_rectangle_int_t rect = {x, y, w, h};
+	cairo_region_t *pShapeBitmap = cairo_region_create_rectangle (&rect);  // for a more complex shape, we would need to draw it on a cairo_surface_t, and then make it a region with gdk_cairo_region_from_surface().
+	#endif
+	return pShapeBitmap;
+}
+
 
   //////////////////
  /// GET CONFIG ///
@@ -638,7 +638,6 @@ static gboolean get_config (GKeyFile *pKeyFile, CairoContainersParam *pContainer
 	pContainersParam->iCairoAnimationDeltaT = 1000. / iRefreshFrequency;
 	//g_print ("iGLAnimationDeltaT: %d\n", pContainersParam->iGLAnimationDeltaT);
 	
-	//pContainersParam->bConfigPanelTransparency = cairo_dock_get_boolean_key_value (pKeyFile, "System", "config transparency", &bFlushConfFileNeeded, TRUE, NULL, NULL);
 	return bFlushConfFileNeeded;
 }
 
