@@ -36,6 +36,15 @@ G_BEGIN_DECLS
 * To remove the Data Renderer from an icon, use /ref cairo_dock_remove_data_renderer_on_icon.
 */
 
+typedef enum _RendererRotateTheme {
+	CD_RENDERER_ROTATE_NO=0,
+	CD_RENDERER_ROTATE_WITH_CONTAINER,
+	CD_RENDERER_ROTATE_YES,
+	CD_RENDERER_NB_ROTATE
+	} RendererRotateTheme; 
+
+#define CAIRO_DATA_RENDERER_UNDEF_VALUE ((double)-1.e9)
+
 //
 // Structures
 //
@@ -66,6 +75,8 @@ struct _CairoDataRendererAttribute {
 	gboolean bUpdateMinMax;
 	/// whether to write the values on the icon. [false by default].
 	gboolean bWriteValues;
+	/// an option to rotate applet, no, automatic or always.
+	RendererRotateTheme iRotateTheme;
 	/// time needed to update to the new values. The update is smooth in OpenGL mode. [0 by default]
 	gint iLatencyTime;
 	/// a function used to format the values into a string. Only useful if you make te DataRenderer write the values [optionnal, by default the values are formatted with 2 decimals].
@@ -156,6 +167,10 @@ struct _CairoDataRenderer {
 	gboolean bCanRenderValueAsText;
 	/// set to TRUE <=> the drawing will be rotated if the container is vertical.
 	gboolean bRotateWithContainer;
+	/// an option to rotate applet, no, automatic or always.
+	RendererRotateTheme iRotateTheme;
+	/// set to TRUE <=> the theme images are rotated 90° clockwise.
+	gboolean bisRotate;
 	/// an optionnal list of labels to be displayed on the Data Renderer to indicate the nature of each value. Same size as the set of values.
 	CairoDataRendererText *pLabels;
 	/// an optionnal list of emblems to be displayed on the Data Renderer to indicate the nature of each value. Same size as the set of values.
@@ -167,6 +182,7 @@ struct _CairoDataRenderer {
 	gint iSmoothAnimationStep;
 	/// latency due to the smooth movement (0 means the displayed value is the current one, 1 the previous)
 	gdouble fLatency;
+	guint iSidRenderIdle;  // source ID to delay the rendering in OpenGL until the container is fully resized
 };
 
 
@@ -220,6 +236,7 @@ void cairo_dock_render_overlays_to_context (CairoDataRenderer *pRenderer, int iN
 
 void cairo_dock_render_overlays_to_texture (CairoDataRenderer *pRenderer, int iNumValue);
 
+void cairo_data_renderer_get_size (CairoDataRenderer *pRenderer, gint *iWidth, gint *iHeight);
 
 ///
 /// Structure Access
@@ -288,7 +305,13 @@ void cairo_dock_render_overlays_to_texture (CairoDataRenderer *pRenderer, int iN
 *@param i the number of the value
 *@param t the time (in number of steps)
 *@return a double in [0,1]*/
-#define cairo_data_renderer_get_normalized_value(pRenderer, i, t) MAX (0, MIN (1, (cairo_data_renderer_get_value (pRenderer, i, t) - cairo_data_renderer_get_min_value (pRenderer, i)) / (cairo_data_renderer_get_max_value (pRenderer, i) - cairo_data_renderer_get_min_value (pRenderer, i))))
+#define cairo_data_renderer_get_normalized_value(pRenderer, i, t) \
+__extension__ ({\
+	double _x = cairo_data_renderer_get_value (pRenderer, i, t);\
+	if ( _x > CAIRO_DATA_RENDERER_UNDEF_VALUE+1) {\
+		_x = MAX (0, MIN (1, (_x - cairo_data_renderer_get_min_value (pRenderer, i)) / (cairo_data_renderer_get_max_value (pRenderer, i) - cairo_data_renderer_get_min_value (pRenderer, i)))); }\
+	_x; })
+
 /**Get the normalized current i-th value (between 0 and 1).
 *@param pRenderer a data renderer
 *@param i the number of the value
