@@ -31,9 +31,11 @@ G_BEGIN_DECLS
 * DBus is used to communicate and interact with other running applications.
 */ 
 
-#define G_VALUE_INIT {0,{{0}}}  // bonne idée d'un dev de GTK, pour eviter les warnings de gcc.
+#if (GLIB_MAJOR_VERSION == 2 && GLIB_MINOR_VERSION < 29) // not needed before => http://developer.gnome.org/gobject/unstable/gobject-Generic-values.html#G-VALUE-INIT:CAPS
+#define G_VALUE_INIT {0,{{0}}}  // bonne idee d'un dev de GTK, pour eviter les warnings de gcc.
+#endif
 
-typedef void (*CairoDockDbusNameOwnerChangedFunc) (gboolean bOwned, gpointer data);
+typedef void (*CairoDockDbusNameOwnerChangedFunc) (const gchar *cName, gboolean bOwned, gpointer data);
 
 /** Get the connection to the 'session' Bus.
 *@return the connection to the bus.
