@@ -40,6 +40,34 @@ extern CairoDocksParam myDocksParam;
 extern CairoDocksManager myDocksMgr;
 #endif
 
+typedef enum {
+	CAIRO_HIT_SCREEN_BORDER,
+	CAIRO_HIT_DOCK_PLACE,
+	CAIRO_HIT_SCREEN_CORNER,
+	CAIRO_HIT_ZONE,
+	CAIRO_HIT_NB_METHODS
+} CairoCallbackMethod;
+
+typedef enum {
+	ICON_DEFAULT,  // same as main dock
+	ICON_TINY,
+	ICON_VERY_SMALL,
+	ICON_SMALL,
+	ICON_MEDIUM,
+	ICON_BIG,
+	ICON_HUGE
+	} GldiIconSizeEnum;
+
+/// TODO: harmonize the values with the simple config -> make some public functions...
+typedef enum {
+	ICON_SIZE_TINY = 28,
+	ICON_SIZE_VERY_SMALL = 36,
+	ICON_SIZE_SMALL = 42,
+	ICON_SIZE_MEDIUM = 48,
+	ICON_SIZE_BIG = 56,
+	ICON_SIZE_HUGE = 64
+	} GldiIconSize;
+
 // params
 struct _CairoDocksParam {
 	// frame
@@ -57,6 +85,7 @@ struct _CairoDocksParam {
 	gdouble fStripesColorBright[4];
 	gdouble fStripesColorDark[4];
 	gdouble fStripesAngle;
+	gdouble fHiddenBg[4];
 	// position
 	gint iGapX, iGapY;
 	CairoDockPositionType iScreenBorder;
@@ -66,7 +95,7 @@ struct _CairoDocksParam {
 	// Root dock visibility
 	CairoDockVisibility iVisibility;
 	gchar *cHideEffect;
-	gint iCallbackMethod;
+	CairoCallbackMethod iCallbackMethod;
 	gint iZoneWidth, iZoneHeight;
 	gchar *cZoneImage;
 	gdouble fZoneAlpha;
@@ -88,7 +117,7 @@ struct _CairoDocksParam {
 // manager
 struct _CairoDocksManager {
 	GldiManager mgr;
-	CairoDock *(*cairo_dock_create_dock) (const gchar *cDockName, const gchar *cRendererName);
+	CairoDock *(*cairo_dock_create_dock) (const gchar *cDockName);
 	void (*destroy_dock) (CairoDock *pDock, const gchar *cDockName);
 	} ;
 
@@ -98,21 +127,13 @@ typedef enum {
 	NOTIFICATION_ENTER_DOCK = NB_NOTIFICATIONS_CONTAINER,
 	/// notification called when the mouse leave a dock.
 	NOTIFICATION_LEAVE_DOCK,
-	/// notification called when a dock is updated in the fast rendering loop.
-	NOTIFICATION_UPDATE_DOCK,
-	/// notification called when when a dock is updated in the slow rendering loop.
-	NOTIFICATION_UPDATE_DOCK_SLOW,
-	/// notification called when a dock is rendered.
-	NOTIFICATION_RENDER_DOCK,
-	/// notification called when a dock is stopped, for instance before it is destroyed.
-	NOTIFICATION_STOP_DOCK,
+	NOTIFICATION_STOP_DOCK_DEPRECATED,
 	/// notification called when an icon has just been inserted into a dock. data : {Icon, CairoDock}
 	NOTIFICATION_INSERT_ICON,
 	/// notification called when an icon is going to be removed from a dock. data : {Icon, CairoDock}
 	NOTIFICATION_REMOVE_ICON,
 	/// notification called when an icon is moved inside a dock. data : {Icon, CairoDock}
 	NOTIFICATION_ICON_MOVED,
-	/// 
 	NB_NOTIFICATIONS_DOCKS
 	} CairoDocksNotifications;
 
@@ -125,26 +146,22 @@ void cairo_dock_force_docks_above (void);
 void cairo_dock_reset_docks_table (void);
 
 /** Create a new root dock.
-* @param cDockName name of the dock, used to identify it quickly. If the name is already used, the corresponding dock is returned.
-* @param cRendererName name of a renderer. If NULL, the default renderer will be applied.
-* @return the newly allocated dock, to destroy with #cairo_dock_destroy_dock
+* @param cDockName name (= ID) of the dock. If the name is already used, the corresponding dock is returned.
+* @return the dock, to destroy with #cairo_dock_destroy_dock
 */
-CairoDock *cairo_dock_create_dock (const gchar *cDockName, const gchar *cRendererName);
-
-/** Increase by 1 the number of pointing icons. If the dock was a root dock, it becomes a sub-dock.
-* @param pDock a dock.
-* @param pParentDock its parent dock, if it becomes a sub-dock, otherwise it can be NULL.
-*/
-void cairo_dock_reference_dock (CairoDock *pDock, CairoDock *pParentDock);
-
+CairoDock *cairo_dock_create_dock (const gchar *cDockName);
 
 /** Create a new dock of type "sub-dock", and load a given list of icons inside. The list then belongs to the dock, so it must not be freeed after that. The buffers of each icon are loaded, so they just need to have an image filename and a name.
-* @param pIconList a list of icons that will be loaded and inserted into the new dock.
 * @param cDockName desired name for the new dock.
+* @param cRendererName name of a renderer. If NULL, the default renderer will be applied.
 * @param pParentDock the parent dock.
+* @param pIconList a list of icons that will be loaded and inserted into the new dock.
 * @return the newly allocated dock.
 */
-CairoDock *cairo_dock_create_subdock_from_scratch (GList *pIconList, gchar *cDockName, CairoDock *pParentDock);
+CairoDock *cairo_dock_create_subdock (const gchar *cDockName, const gchar *cRendererName, CairoDock *pParentDock, GList *pIconList);
+
+void cairo_dock_main_dock_to_sub_dock (CairoDock *pDock, CairoDock *pParentDock, const gchar *cRendererName);
+
 
 /** Destroy a dock and its icons.
 * @param pDock the dock.
@@ -220,18 +237,23 @@ void cairo_dock_hide_parent_dock (CairoDock *pDock);
 gboolean cairo_dock_hide_child_docks (CairoDock *pDock);
 
 /** (Re)load all buffers of all icons in all docks.
-*@param bReloadAppletsToo TRUE to reload applet icons too.
 */
-void cairo_dock_reload_buffers_in_all_docks (gboolean bReloadAppletsToo);
+void cairo_dock_reload_buffers_in_all_docks (gboolean bUpdateIconSize);
 
 void cairo_dock_draw_subdock_icons (void);
 
-void cairo_dock_reset_all_views (void);
+///void cairo_dock_update_all_docks_size (void);
+
+///void cairo_dock_reset_all_views (void);
 void cairo_dock_set_all_views_to_default (int iDockType);
 
 
 void cairo_dock_write_root_dock_gaps (CairoDock *pDock);
 
+int cairo_dock_convert_icon_size_to_pixels (GldiIconSizeEnum s, double *fMaxScale, double *fReflectSize, int *iIconGap);
+
+GldiIconSizeEnum cairo_dock_convert_icon_size_to_enum (int iIconSize);
+
 /** Reload the config of a root dock and update it accordingly.
 *@param cDockName name of the dock.
 *@param pDock the dock.
