@@ -32,10 +32,6 @@
 #include <librsvg/rsvg.h>
 #include <librsvg/rsvg-cairo.h>
 
-#ifdef HAVE_GLITZ
-#include <glitz-glx.h>
-#include <cairo-glitz.h>
-#endif
 
 #include <X11/extensions/Xrender.h>
 #include <GL/glx.h>
@@ -141,15 +137,17 @@ void cairo_dock_draw_icon_reflect_opengl (Icon *pIcon, CairoDock *pDock)
 		glPushMatrix ();
 		double x0, y0, x1, y1;
 		double fScale = ((myIconsParam.bConstantSeparatorSize && CAIRO_DOCK_ICON_TYPE_IS_SEPARATOR (pIcon)) ? 1. : pIcon->fScale);
-		double fReflectSize = MIN (myIconsParam.fReflectSize, pIcon->fHeight/pDock->container.fRatio*fScale);
-		double fReflectRatio = fReflectSize * pDock->container.fRatio / pIcon->fHeight / fScale  / pIcon->fHeightFactor;
-		double fOffsetY = pIcon->fHeight * fScale/2 + fReflectSize * pDock->container.fRatio/2 + pIcon->fDeltaYReflection;
+		///double fReflectSize = MIN (myIconsParam.fReflectSize, pIcon->fHeight/pDock->container.fRatio*fScale);
+		double fReflectSize = pIcon->fHeight * myIconsParam.fReflectHeightRatio * fScale;
+		///double fReflectRatio = fReflectSize * pDock->container.fRatio / pIcon->fHeight / fScale  / pIcon->fHeightFactor;
+		double fReflectRatio = myIconsParam.fReflectHeightRatio;
+		double fOffsetY = pIcon->fHeight * fScale/2 + fReflectSize/** * pDock->container.fRatio*/ / 2 + pIcon->fDeltaYReflection;
 		if (pDock->container.bIsHorizontal)
 		{
 			if (pDock->container.bDirectionUp)
 			{
 				glTranslatef (0., - fOffsetY, 0.);
-				glScalef (pIcon->fWidth * pIcon->fWidthFactor * fScale, - fReflectSize * pDock->container.fRatio, 1.);  // taille du reflet et on se retourne.
+				glScalef (pIcon->fWidth * pIcon->fWidthFactor * fScale, - fReflectSize/** * pDock->container.fRatio*/, 1.);  // taille du reflet et on se retourne.
 				x0 = 0.;
 				y0 = 1. - fReflectRatio;
 				x1 = 1.;
@@ -158,7 +156,7 @@ void cairo_dock_draw_icon_reflect_opengl (Icon *pIcon, CairoDock *pDock)
 			else
 			{
 				glTranslatef (0., fOffsetY, 0.);
-				glScalef (pIcon->fWidth * pIcon->fWidthFactor * fScale, fReflectSize * pDock->container.fRatio, 1.);
+				glScalef (pIcon->fWidth * pIcon->fWidthFactor * fScale, fReflectSize/** * pDock->container.fRatio*/, 1.);
 				x0 = 0.;
 				y0 = fReflectRatio;
 				x1 = 1.;
@@ -170,7 +168,7 @@ void cairo_dock_draw_icon_reflect_opengl (Icon *pIcon, CairoDock *pDock)
 			if (pDock->container.bDirectionUp)
 			{
 				glTranslatef (fOffsetY, 0., 0.);
-				glScalef (- fReflectSize * pDock->container.fRatio, pIcon->fWidth * pIcon->fWidthFactor * fScale, 1.);
+				glScalef (- fReflectSize/** * pDock->container.fRatio*/, pIcon->fWidth * pIcon->fWidthFactor * fScale, 1.);
 				x0 = 1. - fReflectRatio;
 				y0 = 0.;
 				x1 = 1.;
@@ -179,7 +177,7 @@ void cairo_dock_draw_icon_reflect_opengl (Icon *pIcon, CairoDock *pDock)
 			else
 			{
 				glTranslatef (- fOffsetY, 0., 0.);
-				glScalef (fReflectSize * pDock->container.fRatio, pIcon->fWidth * pIcon->fWidthFactor * fScale, 1.);
+				glScalef (fReflectSize/** * pDock->container.fRatio*/, pIcon->fWidth * pIcon->fWidthFactor * fScale, 1.);
 				x0 = fReflectRatio;
 				y0 = 0.;
 				x1 = 0.;
@@ -313,11 +311,12 @@ void cairo_dock_translate_on_icon_opengl (Icon *icon, CairoContainer *pContainer
 {
 	double fX=0, fY=0;
 	_compute_icon_coordinate (icon, pContainer, fDockMagnitude, &fX, &fY);
+	double fMaxScale = cairo_dock_get_icon_max_scale (icon);
 	
 	if (pContainer->bIsHorizontal)
-		glTranslatef ( (fX),  (fY - icon->fHeight * icon->fScale * (1 - icon->fGlideScale/2)), - icon->fHeight * (1+myIconsParam.fAmplitude));
+		glTranslatef ( (fX),  (fY - icon->fHeight * icon->fScale * (1 - icon->fGlideScale/2)), - icon->fHeight * fMaxScale);
 	else
-		glTranslatef ( (fY + icon->fHeight * icon->fScale * (1 - icon->fGlideScale/2)),  (fX), - icon->fHeight * (1+myIconsParam.fAmplitude));
+		glTranslatef ( (fY + icon->fHeight * icon->fScale * (1 - icon->fGlideScale/2)),  (fX), - icon->fHeight * fMaxScale);
 }
 
 void cairo_dock_render_one_icon_opengl (Icon *icon, CairoDock *pDock, double fDockMagnitude, gboolean bUseText)
@@ -329,7 +328,7 @@ void cairo_dock_render_one_icon_opengl (Icon *icon, CairoDock *pDock, double fDo
 	if (g_pGradationTexture[pDock->container.bIsHorizontal] == 0)
 	{
 		//g_pGradationTexture[pDock->container.bIsHorizontal] = cairo_dock_load_local_texture (pDock->container.bIsHorizontal ? "texture-gradation-vert.png" : "texture-gradation-horiz.png", GLDI_SHARE_DATA_DIR);
-		g_pGradationTexture[pDock->container.bIsHorizontal] = cairo_dock_load_texture_from_raw_data (gradationTex,
+		g_pGradationTexture[pDock->container.bIsHorizontal] = cairo_dock_create_texture_from_raw_data (gradationTex,
 			pDock->container.bIsHorizontal ? 1:48,
 			pDock->container.bIsHorizontal ? 48:1);
 		cd_debug ("g_pGradationTexture(%d) <- %d", pDock->container.bIsHorizontal, g_pGradationTexture[pDock->container.bIsHorizontal]);
@@ -347,9 +346,9 @@ void cairo_dock_render_one_icon_opengl (Icon *icon, CairoDock *pDock, double fDo
 	
 	glPushMatrix ();
 	if (pDock->container.bIsHorizontal)
-		glTranslatef (fX, fY - icon->fHeight * icon->fScale * (1 - icon->fGlideScale/2), - icon->fHeight * (1+myIconsParam.fAmplitude));
+		glTranslatef (fX, fY - icon->fHeight * icon->fScale * (1 - icon->fGlideScale/2), - icon->fHeight * icon->fScale);
 	else
-		glTranslatef (fY + icon->fHeight * icon->fScale * (1 - icon->fGlideScale/2), fX, - icon->fHeight * (1+myIconsParam.fAmplitude));
+		glTranslatef (fY + icon->fHeight * icon->fScale * (1 - icon->fGlideScale/2), fX, - icon->fHeight * icon->fScale);
 	
 	//\_____________________ On positionne l'icone.
 	glPushMatrix ();
@@ -410,13 +409,15 @@ void cairo_dock_render_one_icon_opengl (Icon *icon, CairoDock *pDock, double fDo
 	//\_____________________ On dessine les infos additionnelles.
 	if (icon->iQuickInfoTexture != 0)
 	{
+		double fMaxScale = cairo_dock_get_icon_max_scale (icon);
+		
 		glPushMatrix ();
 		glRotatef (-icon->fOrientation/G_PI*180., 0., 0., 1.);
-		glTranslatef (0., (- icon->fHeight + icon->iQuickInfoHeight * fRatio) * icon->fScale/2, 0.);
+		glTranslatef (0., (- icon->fHeight + icon->iQuickInfoHeight / fMaxScale * fRatio) * icon->fScale/2, 0.);
 		
 		cairo_dock_draw_texture_with_alpha (icon->iQuickInfoTexture,
-			icon->iQuickInfoWidth * fRatio * icon->fScale,
-			icon->iQuickInfoHeight * fRatio * icon->fScale,
+			icon->iQuickInfoWidth / fMaxScale * fRatio * icon->fScale,
+			icon->iQuickInfoHeight / fMaxScale * fRatio * icon->fScale,
 			icon->fAlpha);
 		
 		glPopMatrix ();
@@ -426,13 +427,24 @@ void cairo_dock_render_one_icon_opengl (Icon *icon, CairoDock *pDock, double fDo
 	
 	//\_____________________ On dessine les etiquettes, avec un alpha proportionnel au facteur d'echelle de leur icone.
 	glPopMatrix ();  // retour au debut de la fonction.
-	if (bUseText && icon->iLabelTexture != 0 && icon->iHideLabel == 0 &&
-		( (icon->fScale > 1.01 && (! myIconsParam.bLabelForPointedIconOnly || icon->bPointed)) ||
-		((myIconsParam.fAmplitude < 0.001 || pDock->fMagnitudeMax < 0.001) && icon->bPointed) ))  // 1.01 car sin(pi) = 1+epsilon :-/  //  && icon->iAnimationState < CAIRO_DOCK_STATE_CLICKED
+	if (bUseText && icon->pTextBuffer != NULL && icon->iHideLabel == 0
+	&& (icon->bPointed || (icon->fScale > 1.01 && ! myIconsParam.bLabelForPointedIconOnly)))  // 1.01 car sin(pi) = 1+epsilon :-/  //  && icon->iAnimationState < CAIRO_DOCK_STATE_CLICKED
 	{
 		glPushMatrix ();
 		glLoadIdentity ();
 		
+		double fMagnitude;
+		if (myIconsParam.bLabelForPointedIconOnly ||pDock->fMagnitudeMax == 0.)
+		{
+			fMagnitude = fDockMagnitude;  // (icon->fScale - 1) / myIconsParam.fAmplitude / sin (icon->fPhase);  // sin (phi ) != 0 puisque fScale > 1.
+		}
+		else
+		{
+			fMagnitude = (icon->fScale - 1) / myIconsParam.fAmplitude;  /// il faudrait diviser par pDock->fMagnitudeMax ...
+			fMagnitude = pow (fMagnitude, myIconsParam.fLabelAlphaThreshold);
+			///fMagnitude *= (fMagnitude * myIconsParam.fLabelAlphaThreshold + 1) / (myIconsParam.fLabelAlphaThreshold + 1);
+		}
+		
 		double dx = .5 * (icon->iTextWidth & 1);  // on decale la texture pour la coller sur la grille des coordonnees entieres.
 		double dy = .5 * (icon->iTextHeight & 1);
 		
@@ -464,62 +476,19 @@ void cairo_dock_render_one_icon_opengl (Icon *icon, CairoDock *pDock, double fDo
 				glRotatef (-icon->fOrientation/G_PI*180., 0., 0., 1.);
 				glTranslatef (icon->fWidth * icon->fScale/2, -icon->fHeight * icon->fScale/2, 0.);
 			}
-			/*if (pDock->container.bIsHorizontal)
-				glTranslatef (floor (fX), floor (fY - icon->fHeight * icon->fScale * (1 - icon->fGlideScale/2)), 0.);
-			else
-				glTranslatef (floor (fY + icon->fHeight * icon->fScale * (1 - icon->fGlideScale/2)), floor (fX), 0.);
-			
-			double fOffsetX = 0.;
-			if (icon->fDrawX + icon->fWidth * icon->fScale/2 - icon->iTextWidth/2 < 0)  // l'etiquette deborde a gauche.
-				fOffsetX = icon->iTextWidth/2 - (icon->fDrawX + icon->fWidth * icon->fScale/2);
-			else if (icon->fDrawX + icon->fWidth * icon->fScale/2 + icon->iTextWidth/2 > pDock->container.iWidth)  // l'etiquette deborde a droite.
-				fOffsetX = pDock->container.iWidth - (icon->fDrawX + icon->fWidth * icon->fScale/2 + icon->iTextWidth/2);
-			if (icon->fOrientation != 0 && ! myIconsParam.bTextAlwaysHorizontal)
+		}
+		else  // horizontal label on a vertical dock -> draw them next to the icon, vertically centered (like the Parabolic view)
+		{
+			const int pad = 3;
+			glTranslatef (pDock->container.bDirectionUp ? 
+					floor (fY - (myDocksParam.iDockLineWidth + myDocksParam.iFrameMargin) * (1 - pDock->fMagnitudeMax) - pad - icon->iTextWidth/2) + dx :
+					floor (fY + icon->fHeight * icon->fScale + (myDocksParam.iDockLineWidth + myDocksParam.iFrameMargin) * (1 - pDock->fMagnitudeMax) + pad + icon->iTextWidth/2) + dx,
+				floor (fX) + dy,
+				0.);
+			if (icon->pSubDock && gldi_container_is_visible (CAIRO_CONTAINER (icon->pSubDock)))
 			{
-				glTranslatef (-icon->fWidth * icon->fScale/2, icon->fHeight * icon->fScale/2, 0.);
-				glRotatef (-icon->fOrientation/G_PI*180., 0., 0., 1.);
-				glTranslatef (icon->fWidth * icon->fScale/2, -icon->fHeight * icon->fScale/2, 0.);
+				fMagnitude /= 3;
 			}
-			
-			if (pDock->container.bIsHorizontal)
-			{
-				glTranslatef (floor (fOffsetX) + dx,
-					floor ((pDock->container.bDirectionUp ? 1:-1) * (icon->fHeight * icon->fScale/2 + myIconsParam.iLabelSize - icon->iTextHeight / 2)) + dy,
-					0.);
-			}
-			else
-			{
-				glTranslatef (floor ((pDock->container.bDirectionUp ? -.5:.5) * (icon->fHeight * icon->fScale + icon->iTextHeight)) + dx,
-					- floor (fOffsetX) + dy,
-					0.);
-				glRotatef (pDock->container.bDirectionUp ? 90 : -90, 0., 0., 1.);
-			}*/
-		}
-		else
-		{
-			fY += icon->fHeight * icon->fScale/2;  // middle of the icon
-			
-			if (fY + icon->iTextWidth/2 > pDock->container.iHeight)  // out from the right
-				fY = pDock->container.iHeight - icon->iTextWidth/2;
-			if (fY - icon->iTextWidth/2 < 0)  // out from the left.
-				fY = icon->iTextWidth/2;
-			
-			double y = (icon->fWidth * icon->fScale + icon->iTextHeight) / 2;
-			if (fX + y + icon->iTextHeight/2 > pDock->container.iWidth)  // le texte deborde par le haut.
-				y = pDock->container.iWidth - fX - icon->iTextHeight/2;
-			glTranslatef (floor (fY) + dx, floor (fX) + floor (y) + dy, 0.);
-		}
-		
-		double fMagnitude;
-		if (myIconsParam.bLabelForPointedIconOnly ||pDock->fMagnitudeMax == 0.)
-		{
-			fMagnitude = fDockMagnitude;  // (icon->fScale - 1) / myIconsParam.fAmplitude / sin (icon->fPhase);  // sin (phi ) != 0 puisque fScale > 1.
-		}
-		else
-		{
-			fMagnitude = (icon->fScale - 1) / myIconsParam.fAmplitude;  /// il faudrait diviser par pDock->fMagnitudeMax ...
-			fMagnitude = pow (fMagnitude, myIconsParam.fLabelAlphaThreshold);
-			///fMagnitude *= (fMagnitude * myIconsParam.fLabelAlphaThreshold + 1) / (myIconsParam.fLabelAlphaThreshold + 1);
 		}
 		
 		_cairo_dock_enable_texture ();
@@ -539,9 +508,9 @@ void cairo_dock_render_hidden_dock_opengl (CairoDock *pDock)
 {
 	//g_print ("%s (%d, %x)\n", __func__, pDock->bIsMainDock, g_pVisibleZoneSurface);
 	//\_____________________ on dessine la zone de rappel.
-	glLoadIdentity ();  // annule l'offset de cachage.
+	///glLoadIdentity ();  // annule l'offset de cachage.
 	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | (pDock->pRenderer->bUseStencil && g_openglConfig.bStencilBufferAvailable ? GL_STENCIL_BUFFER_BIT : 0));
-	cairo_dock_apply_desktop_background_opengl (CAIRO_CONTAINER (pDock));
+	gldi_glx_apply_desktop_background (CAIRO_CONTAINER (pDock));
 	
 	if (g_pVisibleZoneBuffer.iTexture != 0)
 	{
@@ -586,7 +555,10 @@ void cairo_dock_render_hidden_dock_opengl (CairoDock *pDock)
 	Icon *icon;
 	GList *ic = pFirstDrawnElement;
 	double pHiddenBgColor[4];
-	double r = 3;
+	const double r = 4; // corner radius of the background
+	const double gap = 3;  // gap to the screen
+	double dw = (myIconsParam.iIconGap > 2 ? 2 : 0);  // 1px margin around the icons for a better readability (only if icons won't be stuck togather then).
+	double w, h;
 	_cairo_dock_set_blend_alpha ();
 	do
 	{
@@ -595,26 +567,28 @@ void cairo_dock_render_hidden_dock_opengl (CairoDock *pDock)
 		{
 			//g_print ("%s : %d (%d)\n", icon->cName, icon->bIsDemandingAttention, icon->Xid);
 			y = icon->fDrawY;
-			icon->fDrawY = (pDock->container.bDirectionUp ? pDock->container.iHeight - icon->fHeight * icon->fScale : 0.);
+			icon->fDrawY = (pDock->container.bDirectionUp ? pDock->container.iHeight - icon->fHeight * icon->fScale - gap : gap);
 			
 			if (icon->pHiddenBgColor)
 			{
 				glPushMatrix ();
 				memcpy (pHiddenBgColor, icon->pHiddenBgColor, 4*sizeof (gdouble));
 				pHiddenBgColor[3] *= pDock->fPostHideOffset;
+				w = icon->fWidth * icon->fScale;
+				h = icon->fHeight * icon->fScale;
 				if (pDock->container.bIsHorizontal)
 				{
-					glTranslatef (icon->fDrawX + icon->fWidth/2,
-						pDock->container.iHeight - icon->fDrawY - icon->fHeight/2,
+					glTranslatef (icon->fDrawX + w/2,
+						pDock->container.iHeight - icon->fDrawY - h/2,
 						0.);
-					cairo_dock_draw_rounded_rectangle_opengl (icon->fWidth - 2*r, icon->fHeight, r, 0, pHiddenBgColor);
+					cairo_dock_draw_rounded_rectangle_opengl (w - 2*r + dw, h, r, 0, pHiddenBgColor);
 				}
 				else
 				{
-					glTranslatef (icon->fDrawY + icon->fHeight/2,
-						pDock->container.iWidth - icon->fDrawX - icon->fWidth/2,
+					glTranslatef (icon->fDrawY + h/2,
+						pDock->container.iWidth - icon->fDrawX - w/2,
 						0.);
-					cairo_dock_draw_rounded_rectangle_opengl (icon->fHeight - 2*r, icon->fWidth, r, 0, pHiddenBgColor);
+					cairo_dock_draw_rounded_rectangle_opengl (h - 2*r + dw, w, r, 0, pHiddenBgColor);
 				}
 				glPopMatrix ();
 			}
@@ -638,6 +612,7 @@ GLuint cairo_dock_create_texture_from_surface (cairo_surface_t *pImageSurface)
 	GLuint iTexture = 0;
 	int w = cairo_image_surface_get_width (pImageSurface);
 	int h = cairo_image_surface_get_height (pImageSurface);
+	//g_print ("%s (%dx%d)\n", __func__, w, h);
 	
 	cairo_surface_t *pPowerOfwoSurface = pImageSurface;
 	
@@ -663,14 +638,22 @@ GLuint cairo_dock_create_texture_from_surface (cairo_surface_t *pImageSurface)
 		}
 	}
 	
-	glEnable(GL_TEXTURE_2D);
+	_cairo_dock_enable_texture ();
+	_cairo_dock_set_blend_source ();
+	glColor4f (1., 1., 1., 1.);
 	glGenTextures (1, &iTexture);
-	cd_debug ("+ texture %d generee (%x, %dx%d)", iTexture, cairo_image_surface_get_data (pImageSurface), w, h);
+	//g_print ("+ texture %d generee (%p, %dx%d)\n", iTexture, cairo_image_surface_get_data (pImageSurface), w, h);
 	glBindTexture (GL_TEXTURE_2D, iTexture);
-	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
-	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+	
+	glTexParameteri (GL_TEXTURE_2D,
+		GL_TEXTURE_MIN_FILTER,
+		g_bEasterEggs ? GL_LINEAR_MIPMAP_LINEAR : GL_LINEAR);
 	if (g_bEasterEggs)
-		gluBuild2DMipmaps (GL_TEXTURE_2D,
+		glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP, GL_TRUE);
+	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+	
+	if (g_bEasterEggs)
+		gluBuild2DMipmaps (GL_TEXTURE_2D,  /// see for automatic mipmaps generation, or at least how to update the mipmaps...
 			4,
 			w,
 			h,
@@ -690,10 +673,11 @@ GLuint cairo_dock_create_texture_from_surface (cairo_surface_t *pImageSurface)
 	if (pPowerOfwoSurface != pImageSurface)
 		cairo_surface_destroy (pPowerOfwoSurface);
 	glDisable(GL_TEXTURE_2D);
+	glDisable(GL_BLEND);
 	return iTexture;
 }
 
-GLuint cairo_dock_load_texture_from_raw_data (const guchar *pTextureRaw, int iWidth, int iHeight)
+GLuint cairo_dock_create_texture_from_raw_data (const guchar *pTextureRaw, int iWidth, int iHeight)
 {
 	/*cd_debug ("%dx%d\n", iWidth, iHeight);
 	int i;
@@ -718,14 +702,29 @@ GLuint cairo_dock_load_texture_from_raw_data (const guchar *pTextureRaw, int iWi
 	pTextureRaw = pPixelBuffer2;*/
 	GLuint iTexture = 0;
 	
-	glEnable (GL_TEXTURE_2D);
+	_cairo_dock_enable_texture ();
+	_cairo_dock_set_blend_source ();
+	glColor4f (1., 1., 1., 1.);
 	glGenTextures(1, &iTexture);
 	glBindTexture(GL_TEXTURE_2D, iTexture);
 	
-	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+	glTexParameteri (GL_TEXTURE_2D,
+		GL_TEXTURE_MIN_FILTER,
+		g_bEasterEggs ? GL_LINEAR_MIPMAP_LINEAR : GL_LINEAR);
+	if (g_bEasterEggs)
+		glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP, GL_TRUE);
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 	
-	glTexImage2D (GL_TEXTURE_2D, 0, GL_RGBA, iWidth, iHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, pTextureRaw);
+	if (g_bEasterEggs && pTextureRaw)
+		gluBuild2DMipmaps (GL_TEXTURE_2D,
+			4,
+			iWidth,
+			iHeight,
+			GL_RGBA,
+			GL_UNSIGNED_BYTE,
+			pTextureRaw);
+	else
+		glTexImage2D (GL_TEXTURE_2D, 0, GL_RGBA, iWidth, iHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, pTextureRaw);
 	glBindTexture (GL_TEXTURE_2D, 0);
 	glDisable(GL_TEXTURE_2D);
 	return iTexture;
@@ -733,7 +732,11 @@ GLuint cairo_dock_load_texture_from_raw_data (const guchar *pTextureRaw, int iWi
 
 GLuint cairo_dock_create_texture_from_image_full (const gchar *cImageFile, double *fImageWidth, double *fImageHeight)
 {
+	#if (GTK_MAJOR_VERSION < 3 && GTK_MINOR_VERSION < 20)
 	g_return_val_if_fail (GTK_WIDGET_REALIZED (g_pPrimaryContainer->pWidget), 0);
+	#else
+	g_return_val_if_fail (gtk_widget_get_realized (g_pPrimaryContainer->pWidget), 0);
+	#endif
 	double fWidth=0, fHeight=0;
 	if (cImageFile == NULL)
 		return 0;
@@ -770,24 +773,41 @@ void cairo_dock_update_icon_texture (Icon *pIcon)
 {
 	if (pIcon != NULL && pIcon->pIconBuffer != NULL)
 	{
-		glEnable (GL_TEXTURE_2D);
+		_cairo_dock_enable_texture ();
+		_cairo_dock_set_blend_source ();
+		glColor4f (1., 1., 1., 1.);
+		
 		if (pIcon->iIconTexture == 0)
 			glGenTextures (1, &pIcon->iIconTexture);
 		int w = cairo_image_surface_get_width (pIcon->pIconBuffer);
 		int h = cairo_image_surface_get_height (pIcon->pIconBuffer);
 		glBindTexture (GL_TEXTURE_2D, pIcon->iIconTexture);
-		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+		
+		glTexParameteri (GL_TEXTURE_2D,
+			GL_TEXTURE_MIN_FILTER,
+			g_bEasterEggs ? GL_LINEAR_MIPMAP_LINEAR : GL_LINEAR);
+		if (g_bEasterEggs)
+			glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP, GL_TRUE);
 		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 		
-		glTexImage2D (GL_TEXTURE_2D,
-			0,
-			4,  // GL_ALPHA / GL_BGRA
-			w,
-			h,
-			0,
-			GL_BGRA,  // GL_ALPHA / GL_BGRA
-			GL_UNSIGNED_BYTE,
-			cairo_image_surface_get_data (pIcon->pIconBuffer));
+		if (g_bEasterEggs)
+			gluBuild2DMipmaps (GL_TEXTURE_2D,
+				4,
+				w,
+				h,
+				GL_BGRA,
+				GL_UNSIGNED_BYTE,
+				cairo_image_surface_get_data (pIcon->pIconBuffer));
+		else
+			glTexImage2D (GL_TEXTURE_2D,
+				0,
+				4,  // GL_ALPHA / GL_BGRA
+				w,
+				h,
+				0,
+				GL_BGRA,  // GL_ALPHA / GL_BGRA
+				GL_UNSIGNED_BYTE,
+				cairo_image_surface_get_data (pIcon->pIconBuffer));
 		glDisable (GL_TEXTURE_2D);
 	}
 }
@@ -801,13 +821,22 @@ void cairo_dock_update_label_texture (Icon *pIcon)
 	}
 	if (pIcon != NULL && pIcon->pTextBuffer != NULL)
 	{
-		glEnable (GL_TEXTURE_2D);
+		_cairo_dock_enable_texture ();
+		_cairo_dock_set_blend_source ();
+		glColor4f (1., 1., 1., 1.);
+		
 		glGenTextures (1, &pIcon->iLabelTexture);
 		int w = cairo_image_surface_get_width (pIcon->pTextBuffer);
 		int h = cairo_image_surface_get_height (pIcon->pTextBuffer);
 		glBindTexture (GL_TEXTURE_2D, pIcon->iLabelTexture);
-		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+		
+		glTexParameteri (GL_TEXTURE_2D,
+			GL_TEXTURE_MIN_FILTER,
+			g_bEasterEggs ? GL_LINEAR_MIPMAP_LINEAR : GL_LINEAR);
+		if (g_bEasterEggs)
+			glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP, GL_TRUE);
 		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+		
 		glTexImage2D (GL_TEXTURE_2D,
 			0,
 			4,  // GL_ALPHA / GL_BGRA
@@ -830,13 +859,22 @@ void cairo_dock_update_quick_info_texture (Icon *pIcon)
 	}
 	if (pIcon != NULL && pIcon->pQuickInfoBuffer != NULL)
 	{
-		glEnable (GL_TEXTURE_2D);
+		_cairo_dock_enable_texture ();
+		_cairo_dock_set_blend_source ();
+		glColor4f (1., 1., 1., 1.);
+		
 		glGenTextures (1, &pIcon->iQuickInfoTexture);
 		int w = cairo_image_surface_get_width (pIcon->pQuickInfoBuffer);
 		int h = cairo_image_surface_get_height (pIcon->pQuickInfoBuffer);
 		glBindTexture (GL_TEXTURE_2D, pIcon->iQuickInfoTexture);
-		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+		
+		glTexParameteri (GL_TEXTURE_2D,
+			GL_TEXTURE_MIN_FILTER,
+			g_bEasterEggs ? GL_LINEAR_MIPMAP_LINEAR : GL_LINEAR);
+		if (g_bEasterEggs)
+			glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP, GL_TRUE);
 		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+		
 		glTexImage2D (GL_TEXTURE_2D,
 			0,
 			4,  // GL_ALPHA / GL_BGRA
@@ -942,9 +980,6 @@ void cairo_dock_draw_hidden_appli_icon (Icon *pIcon, CairoContainer *pContainer,
 	{
 		cairo_dock_remove_transition_on_icon (pIcon);
 		
-		int iWidth, iHeight;
-		cairo_dock_get_icon_extent (pIcon, &iWidth, &iHeight);
-		
 		GLuint iOriginalTexture;
 		if (pIcon->bIsHidden)
 		{
@@ -1046,16 +1081,16 @@ void cairo_dock_update_animated_image_opengl (CairoAnimatedImage *pAnimatedImage
 
 
 
-typedef void (*GLXBindTexImageProc) (Display *display, GLXDrawable drawable, int buffer, int *attribList);
-typedef void (*GLXReleaseTexImageProc) (Display *display, GLXDrawable drawable, int buffer);
+//typedef void (*GLXBindTexImageProc) (Display *display, GLXDrawable drawable, int buffer, int *attribList);
+//typedef void (*GLXReleaseTexImageProc) (Display *display, GLXDrawable drawable, int buffer);
 
 // Bind redirected window to texture:
 GLuint cairo_dock_texture_from_pixmap (Window Xid, Pixmap iBackingPixmap)
 {
-	if (!g_bEasterEggs)
-		return 0;  /// ca ne marche pas. :-(
+	///if (!g_bEasterEggs)
+	///	return 0;  /// ca ne marche pas. :-(
 	
-	if (! g_openglConfig.bTextureFromPixmapAvailable)
+	if (!iBackingPixmap || ! g_openglConfig.bTextureFromPixmapAvailable)
 		return 0;
 	
 	Display *display = gdk_x11_get_default_xdisplay ();
@@ -1136,7 +1171,11 @@ GLuint cairo_dock_texture_from_pixmap (Window Xid, Pixmap iBackingPixmap)
 	
 	g_openglConfig.bindTexImage (display, glxpixmap, GLX_FRONT_LEFT_EXT, NULL);
 	
-	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+	glTexParameteri (GL_TEXTURE_2D,
+		GL_TEXTURE_MIN_FILTER,
+		g_bEasterEggs ? GL_LINEAR_MIPMAP_LINEAR : GL_LINEAR);
+	if (g_bEasterEggs)
+		glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP, GL_TRUE);
 	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 	
 	// draw using iBackingPixmap as texture
