@@ -24,11 +24,6 @@
 #include <pango/pango.h>
 #include <gtk/gtk.h>
 
-#ifdef HAVE_GLITZ
-#include <gdk/gdkx.h>
-#include <glitz-glx.h>
-#include <cairo-glitz.h>
-#endif
 
 #include "cairo-dock-icon-factory.h"
 #include "cairo-dock-icon-facility.h"
@@ -57,40 +52,7 @@ extern gboolean g_bUseOpenGL;
 
 cairo_t * cairo_dock_create_drawing_context_generic (CairoContainer *pContainer)
 {
-#ifdef HAVE_GLITZ
-	if (pContainer->pGlitzDrawable)
-	{
-		//g_print ("creation d'un contexte lie a une surface glitz\n");
-		glitz_surface_t* pGlitzSurface;
-		cairo_surface_t* pCairoSurface;
-		cairo_t* pCairoContext;
-
-		pGlitzSurface = glitz_surface_create (pContainer->pGlitzDrawable,
-			pContainer->pGlitzFormat,
-			pContainer->iWidth,
-			pContainer->iHeight,
-			0,
-			NULL);
-
-		if (pContainer->pDrawFormat->doublebuffer)
-			glitz_surface_attach (pGlitzSurface,
-				pContainer->pGlitzDrawable,
-				GLITZ_DRAWABLE_BUFFER_BACK_COLOR);
-		else
-			glitz_surface_attach (pGlitzSurface,
-				pContainer->pGlitzDrawable,
-				GLITZ_DRAWABLE_BUFFER_FRONT_COLOR);
-
-		pCairoSurface = cairo_glitz_surface_create (pGlitzSurface);
-		pCairoContext = cairo_create (pCairoSurface);
-
-		cairo_surface_destroy (pCairoSurface);
-		glitz_surface_destroy (pGlitzSurface);
-
-		return pCairoContext;
-	}
-#endif // HAVE_GLITZ
-	return gdk_cairo_create (pContainer->pWidget->window);
+	return gdk_cairo_create (gldi_container_get_gdk_window (pContainer));
 }
 
 cairo_t *cairo_dock_create_drawing_context_on_container (CairoContainer *pContainer)
@@ -334,71 +296,99 @@ void cairo_dock_set_icon_scale_on_context (cairo_t *pCairoContext, Icon *icon, g
 		if (myIconsParam.bConstantSeparatorSize && CAIRO_DOCK_ICON_TYPE_IS_SEPARATOR (icon))
 		{
 			cairo_translate (pCairoContext,
-				1 * icon->fWidthFactor * icon->fWidth * (icon->fScale - 1) / 2,
+				icon->fWidthFactor * icon->fWidth * (icon->fScale - 1) / 2,
 				(bDirectionUp ? 1 * icon->fHeightFactor * icon->fHeight * (icon->fScale - 1) : 0));
 			cairo_scale (pCairoContext,
-				fRatio * icon->fWidthFactor / (1 + myIconsParam.fAmplitude),
-				fRatio * icon->fHeightFactor / (1 + myIconsParam.fAmplitude));
+				icon->fWidth / icon->iImageWidth * icon->fWidthFactor/**fRatio * icon->fWidthFactor / (1 + myIconsParam.fAmplitude)*/,
+				icon->fHeight / icon->iImageHeight * icon->fHeightFactor/**fRatio * icon->fHeightFactor / (1 + myIconsParam.fAmplitude)*/);
 		}
 		else
 			cairo_scale (pCairoContext,
-				fRatio * icon->fWidthFactor * icon->fScale / (1 + myIconsParam.fAmplitude) * icon->fGlideScale,
-				fRatio * icon->fHeightFactor * icon->fScale / (1 + myIconsParam.fAmplitude) * icon->fGlideScale);
+				icon->fWidth / icon->iImageWidth * icon->fWidthFactor * icon->fScale/**fRatio * icon->fWidthFactor * icon->fScale / (1 + myIconsParam.fAmplitude)*/ * icon->fGlideScale,
+				icon->fHeight / icon->iImageHeight * icon->fHeightFactor * icon->fScale/**fRatio * icon->fHeightFactor * icon->fScale / (1 + myIconsParam.fAmplitude)*/ * icon->fGlideScale);
 	}
 	else
 	{
 		if (myIconsParam.bConstantSeparatorSize && CAIRO_DOCK_ICON_TYPE_IS_SEPARATOR (icon))
 		{
 			cairo_translate (pCairoContext,
-				1 * icon->fHeightFactor * icon->fHeight * (icon->fScale - 1) / 2,
+				icon->fHeightFactor * icon->fHeight * (icon->fScale - 1) / 2,
 				(bDirectionUp ? 1 * icon->fWidthFactor * icon->fWidth * (icon->fScale - 1) : 0));
 			cairo_scale (pCairoContext,
-				fRatio * icon->fHeightFactor / (1 + myIconsParam.fAmplitude),
-				fRatio * icon->fWidthFactor / (1 + myIconsParam.fAmplitude));
+				icon->fHeight / icon->iImageWidth * icon->fHeightFactor/**fRatio * icon->fHeightFactor / (1 + myIconsParam.fAmplitude)*/,
+				icon->fWidth / icon->iImageHeight * icon->fWidthFactor/**fRatio * icon->fWidthFactor / (1 + myIconsParam.fAmplitude)*/);
 		}
 		else
 			cairo_scale (pCairoContext,
-				fRatio * icon->fHeightFactor * icon->fScale / (1 + myIconsParam.fAmplitude) * icon->fGlideScale,
-				fRatio * icon->fWidthFactor * icon->fScale / (1 + myIconsParam.fAmplitude) * icon->fGlideScale);
+				icon->fHeight / icon->iImageWidth * icon->fHeightFactor * icon->fScale/**fRatio * icon->fHeightFactor * icon->fScale / (1 + myIconsParam.fAmplitude)*/ * icon->fGlideScale,
+				icon->fWidth / icon->iImageHeight * icon->fWidthFactor * icon->fScale/**fRatio * icon->fWidthFactor * icon->fScale / (1 + myIconsParam.fAmplitude)*/ * icon->fGlideScale);
 		
 	}
 }
 
 
-void cairo_dock_draw_icon_reflect_cairo (Icon *icon, CairoDock *pDock, cairo_t *pCairoContext)
+void cairo_dock_draw_icon_reflect_cairo (Icon *icon, CairoContainer *pContainer, cairo_t *pCairoContext)
 {
-	double fRatio = pDock->container.fRatio;
-	if (pDock->container.bUseReflect && icon->pReflectionBuffer != NULL)  // on dessine les reflets.
+	double fRatio = pContainer->fRatio;
+	if (pContainer->bUseReflect && icon->pIconBuffer != NULL)
 	{
 		cairo_save (pCairoContext);
+		double fScale = (myIconsParam.bConstantSeparatorSize && CAIRO_DOCK_ICON_TYPE_IS_SEPARATOR (icon) ? 1. : icon->fScale);
 		
-		if (pDock->container.bIsHorizontal)
+		if (pContainer->bIsHorizontal)
 		{
-			if (myIconsParam.bConstantSeparatorSize && CAIRO_DOCK_ICON_TYPE_IS_SEPARATOR (icon))
-				cairo_translate (pCairoContext, 0, (pDock->container.bDirectionUp ? icon->fDeltaYReflection + icon->fHeight : -icon->fDeltaYReflection - myIconsParam.fReflectSize * fRatio));
+			cairo_translate (pCairoContext,
+				0,
+				(pContainer->bDirectionUp ?
+					icon->fDeltaYReflection + icon->fHeight * fScale :  // go to bottom of icon
+					-icon->fDeltaYReflection - icon->fHeight * myIconsParam.fReflectHeightRatio));  // go to top of reflect
+			cairo_rectangle (pCairoContext, 0, 0, icon->fWidth * icon->fScale, icon->fHeight * myIconsParam.fReflectHeightRatio);
+			if (pContainer->bDirectionUp)
+				cairo_translate (pCairoContext, 0, icon->fHeight * icon->fHeightFactor * fScale);
 			else
-				cairo_translate (pCairoContext, 0, (pDock->container.bDirectionUp ? icon->fDeltaYReflection + icon->fHeight * icon->fScale : -icon->fDeltaYReflection - myIconsParam.fReflectSize * icon->fScale * fRatio));
+				cairo_translate (pCairoContext, 0, icon->fHeight * icon->fHeightFactor * myIconsParam.fReflectHeightRatio);  // go back to top of icon, since we will flip y axis
 		}
 		else
 		{
-			if (myIconsParam.bConstantSeparatorSize && CAIRO_DOCK_ICON_TYPE_IS_SEPARATOR (icon))
-				cairo_translate (pCairoContext, (pDock->container.bDirectionUp ? icon->fDeltaYReflection + icon->fHeight : -icon->fDeltaYReflection - myIconsParam.fReflectSize * fRatio), 0);
+			cairo_translate (pCairoContext,
+				(pContainer->bDirectionUp ?
+					icon->fDeltaYReflection + icon->fHeight * fScale :  // go to bottom of icon
+					-icon->fDeltaYReflection - icon->fHeight * myIconsParam.fReflectHeightRatio),  // go to top of reflect
+				0);
+			cairo_rectangle (pCairoContext, 0, 0, icon->fHeight * myIconsParam.fReflectHeightRatio, icon->fWidth * icon->fScale);
+			if (pContainer->bDirectionUp)
+				cairo_translate (pCairoContext, icon->fHeight * icon->fHeightFactor * fScale, 0);
 			else
-				cairo_translate (pCairoContext, (pDock->container.bDirectionUp ? icon->fDeltaYReflection + icon->fHeight * icon->fScale : -icon->fDeltaYReflection - myIconsParam.fReflectSize * icon->fScale * fRatio), 0);
+				cairo_translate (pCairoContext, icon->fHeight * icon->fHeightFactor * myIconsParam.fReflectHeightRatio, 0);  // go back to top of icon, since we will flip y axis
 		}
-		cairo_dock_set_icon_scale_on_context (pCairoContext, icon, pDock->container.bIsHorizontal, fRatio, pDock->container.bDirectionUp);
+		cairo_clip (pCairoContext);
 		
-		cairo_set_source_surface (pCairoContext, icon->pReflectionBuffer, 0.0, 0.0);
+		cairo_dock_set_icon_scale_on_context (pCairoContext, icon, pContainer->bIsHorizontal, 1., pContainer->bDirectionUp);
 		
-		if (myBackendsParam.bDynamicReflection && icon->fScale > 1)  // on applique la surface avec un degrade en transparence, ou avec une transparence simple.
+		if (pContainer->bIsHorizontal)
+			cairo_scale (pCairoContext, 1, -1);
+		else
+			cairo_scale (pCairoContext, -1, 1);
+		
+		cairo_set_source_surface (pCairoContext, icon->pIconBuffer, 0.0, 0.0);
+		
+		if (myBackendsParam.bDynamicReflection)  // on applique la surface avec un degrade en transparence, ou avec une transparence simple.
 		{
 			cairo_pattern_t *pGradationPattern;
-			if (pDock->container.bIsHorizontal)
+			if (pContainer->bIsHorizontal)
 			{
-				pGradationPattern = cairo_pattern_create_linear (0.,
-					(pDock->container.bDirectionUp ? 0. : myIconsParam.fReflectSize / fRatio * (1 + myIconsParam.fAmplitude)),
-					0.,
-					(pDock->container.bDirectionUp ? myIconsParam.fReflectSize / fRatio * (1 + myIconsParam.fAmplitude) / icon->fScale : myIconsParam.fReflectSize / fRatio * (1 + myIconsParam.fAmplitude) * (1. - 1./ icon->fScale)));  // de haut en bas.
+				if (pContainer->bDirectionUp)
+					pGradationPattern = cairo_pattern_create_linear (
+						0,
+						icon->iImageHeight,
+						0,
+						icon->iImageHeight * (1 - myIconsParam.fReflectHeightRatio));
+				else
+					pGradationPattern = cairo_pattern_create_linear (
+						0,
+						0.,
+						0,
+						icon->iImageHeight * myIconsParam.fReflectHeightRatio);
 				g_return_if_fail (cairo_pattern_status (pGradationPattern) == CAIRO_STATUS_SUCCESS);
 				
 				cairo_pattern_set_extend (pGradationPattern, CAIRO_EXTEND_NONE);
@@ -407,20 +397,28 @@ void cairo_dock_draw_icon_reflect_cairo (Icon *icon, CairoDock *pDock, cairo_t *
 					0.,
 					0.,
 					0.,
-					1.);
+					icon->fAlpha * myIconsParam.fAlbedo);
 				cairo_pattern_add_color_stop_rgba (pGradationPattern,
 					1.,
 					0.,
 					0.,
 					0.,
-					1 - (icon->fScale - 1) / myIconsParam.fAmplitude);  // astuce pour ne pas avoir a re-creer la surface de la reflection.
+					0.);
 			}
 			else
 			{
-				pGradationPattern = cairo_pattern_create_linear ((pDock->container.bDirectionUp ? 0. : myIconsParam.fReflectSize / fRatio * (1 + myIconsParam.fAmplitude)),
-					0.,
-					(pDock->container.bDirectionUp ? myIconsParam.fReflectSize / fRatio * (1 + myIconsParam.fAmplitude) / icon->fScale : myIconsParam.fReflectSize / fRatio * (1 + myIconsParam.fAmplitude) * (1. - 1./ icon->fScale)),
-					0.);
+				if (pContainer->bDirectionUp)
+					pGradationPattern = cairo_pattern_create_linear (
+						icon->iImageWidth,
+						0.,
+						icon->iImageWidth * (1-myIconsParam.fReflectHeightRatio),
+						0);
+				else
+					pGradationPattern = cairo_pattern_create_linear (
+						0,
+						0.,
+						icon->iImageWidth * myIconsParam.fReflectHeightRatio,
+						0);
 				g_return_if_fail (cairo_pattern_status (pGradationPattern) == CAIRO_STATUS_SUCCESS);
 				
 				cairo_pattern_set_extend (pGradationPattern, CAIRO_EXTEND_NONE);
@@ -429,26 +427,22 @@ void cairo_dock_draw_icon_reflect_cairo (Icon *icon, CairoDock *pDock, cairo_t *
 					0.,
 					0.,
 					0.,
-					1.);
+					icon->fAlpha * myIconsParam.fAlbedo);
 				cairo_pattern_add_color_stop_rgba (pGradationPattern,
 					1.,
 					0.,
 					0.,
 					0.,
-					1. - (icon->fScale - 1) / myIconsParam.fAmplitude);  // astuce pour ne pas avoir a re-creer la surface de la reflection.
+					0.);
 			}
 			cairo_set_operator (pCairoContext, CAIRO_OPERATOR_OVER);
-			cairo_translate (pCairoContext, 0, 0);
 			cairo_mask (pCairoContext, pGradationPattern);
 
 			cairo_pattern_destroy (pGradationPattern);
 		}
 		else
 		{
-			if (icon->fAlpha == 1)
-				cairo_paint (pCairoContext);
-			else
-				cairo_paint_with_alpha (pCairoContext, icon->fAlpha);
+			cairo_paint_with_alpha (pCairoContext, icon->fAlpha * myIconsParam.fAlbedo);
 		}
 		cairo_restore (pCairoContext);
 	}
@@ -461,7 +455,7 @@ void cairo_dock_draw_icon_cairo (Icon *icon, CairoDock *pDock, cairo_t *pCairoCo
 	{
 		cairo_save (pCairoContext);
 		
-		cairo_dock_set_icon_scale_on_context (pCairoContext, icon, pDock->container.bIsHorizontal, pDock->container.fRatio, pDock->container.bDirectionUp);
+		cairo_dock_set_icon_scale_on_context (pCairoContext, icon, pDock->container.bIsHorizontal, 1., pDock->container.bDirectionUp);
 		cairo_set_source_surface (pCairoContext, icon->pIconBuffer, 0.0, 0.0);
 		if (icon->fAlpha == 1)
 			cairo_paint (pCairoContext);
@@ -471,7 +465,7 @@ void cairo_dock_draw_icon_cairo (Icon *icon, CairoDock *pDock, cairo_t *pCairoCo
 		cairo_restore (pCairoContext);
 	}
 	//\_____________________ On dessine son reflet.
-	cairo_dock_draw_icon_reflect_cairo (icon, pDock, pCairoContext);
+	cairo_dock_draw_icon_reflect_cairo (icon, CAIRO_CONTAINER (pDock), pCairoContext);
 }
 
 gboolean cairo_dock_render_icon_notification (gpointer pUserData, Icon *icon, CairoDock *pDock, gboolean *bHasBeenRendered, cairo_t *pCairoContext)
@@ -561,12 +555,23 @@ void cairo_dock_render_one_icon (Icon *icon, CairoDock *pDock, cairo_t *pCairoCo
 	cairo_restore (pCairoContext);  // retour juste apres la translation (fDrawX, fDrawY).
 	
 	//\_____________________ On dessine les etiquettes, avec un alpha proportionnel au facteur d'echelle de leur icone.
-	if (bUseText && icon->pTextBuffer != NULL && icon->iHideLabel == 0 &&
-		( (icon->fScale > 1.01 && (! myIconsParam.bLabelForPointedIconOnly || icon->bPointed)) ||
-		((myIconsParam.fAmplitude < 0.001 || pDock->fMagnitudeMax < 0.001) && icon->bPointed) ))  // 1.01 car sin(pi) = 1+epsilon :-/  //  && icon->iAnimationState < CAIRO_DOCK_STATE_CLICKED
+	if (bUseText && icon->pTextBuffer != NULL && icon->iHideLabel == 0
+	&& (icon->bPointed || (icon->fScale > 1.01 && ! myIconsParam.bLabelForPointedIconOnly)))  // 1.01 car sin(pi) = 1+epsilon :-/  //  && icon->iAnimationState < CAIRO_DOCK_STATE_CLICKED
 	{
 		cairo_save (pCairoContext);
 		
+		double fMagnitude;
+		if (myIconsParam.bLabelForPointedIconOnly || pDock->fMagnitudeMax == 0.)
+		{
+			fMagnitude = fDockMagnitude;  // (icon->fScale - 1) / myIconsParam.fAmplitude / sin (icon->fPhase);  // sin (phi ) != 0 puisque fScale > 1.
+		}
+		else
+		{
+			fMagnitude = (icon->fScale - 1) / myIconsParam.fAmplitude;  /// il faudrait diviser par pDock->fMagnitudeMax ...
+			fMagnitude = pow (fMagnitude, myIconsParam.fLabelAlphaThreshold);
+			///fMagnitude *= (fMagnitude * myIconsParam.fLabelAlphaThreshold + 1) / (myIconsParam.fLabelAlphaThreshold + 1);
+		}
+		
 		//int iLabelSize = icon->iTextHeight;
 		int iLabelSize = myIconsParam.iLabelSize;
 		//g_print ("%d / %d\n", icon->iTextHeight, myIconsParam.iLabelSize),
@@ -592,16 +597,19 @@ void cairo_dock_render_one_icon (Icon *icon, CairoDock *pDock, cairo_t *pCairoCo
 				floor (fOffsetX),
 				floor (bDirectionUp ? -iLabelSize : icon->fHeight * icon->fScale));
 		}
-		else if (myIconsParam.bTextAlwaysHorizontal)
+		else if (myIconsParam.bTextAlwaysHorizontal)  // horizontal label on a vertical dock -> draw them next to the icon, vertically centered (like the Parabolic view)
 		{
-			if (icon->fDrawY + fOffsetX + icon->iTextWidth > pDock->container.iHeight)
-				fOffsetX = pDock->container.iHeight - icon->fDrawY - icon->iTextWidth;
-			if (icon->fDrawY + fOffsetX < 0)
-				fOffsetX = - icon->fDrawY;
+			const int pad = 3;
 			cairo_set_source_surface (pCairoContext,
 				icon->pTextBuffer,
-				floor (fOffsetX),
-				0.);
+				bDirectionUp ? 
+					floor (0 - (myDocksParam.iDockLineWidth + myDocksParam.iFrameMargin) * (1 - pDock->fMagnitudeMax) - pad - icon->iTextWidth):
+					floor (0 + icon->fHeight * icon->fScale + (myDocksParam.iDockLineWidth + myDocksParam.iFrameMargin) * (1 - pDock->fMagnitudeMax) + pad),
+				floor (0 + icon->fWidth * icon->fScale/2 - icon->iTextHeight/2));
+			if (icon->pSubDock && gldi_container_is_visible (CAIRO_CONTAINER (icon->pSubDock)))
+			{
+				fMagnitude /= 3;
+			}
 		}
 		else
 		{
@@ -611,17 +619,7 @@ void cairo_dock_render_one_icon (Icon *icon, CairoDock *pDock, cairo_t *pCairoCo
 				floor (bDirectionUp ? fOffsetX - icon->fWidth * icon->fScale : fOffsetX),
 				-floor (bDirectionUp ? iLabelSize : icon->fHeight * icon->fScale + iLabelSize));
 		}
-		double fMagnitude;
-		if (myIconsParam.bLabelForPointedIconOnly || pDock->fMagnitudeMax == 0.)
-		{
-			fMagnitude = fDockMagnitude;  // (icon->fScale - 1) / myIconsParam.fAmplitude / sin (icon->fPhase);  // sin (phi ) != 0 puisque fScale > 1.
-		}
-		else
-		{
-			fMagnitude = (icon->fScale - 1) / myIconsParam.fAmplitude;  /// il faudrait diviser par pDock->fMagnitudeMax ...
-			fMagnitude = pow (fMagnitude, myIconsParam.fLabelAlphaThreshold);
-			///fMagnitude *= (fMagnitude * myIconsParam.fLabelAlphaThreshold + 1) / (myIconsParam.fLabelAlphaThreshold + 1);
-		}
+		
 		if (fMagnitude > .1)
 			cairo_paint_with_alpha (pCairoContext, fMagnitude);
 		cairo_restore (pCairoContext);  // retour juste apres la translation (fDrawX, fDrawY).
@@ -630,13 +628,14 @@ void cairo_dock_render_one_icon (Icon *icon, CairoDock *pDock, cairo_t *pCairoCo
 	//\_____________________ On dessine les infos additionnelles.
 	if (icon->pQuickInfoBuffer != NULL)
 	{
+		double fMaxScale = cairo_dock_get_icon_max_scale (icon);
 		cairo_translate (pCairoContext,
-			(- icon->iQuickInfoWidth * fRatio + icon->fWidthFactor * icon->fWidth) / 2 * icon->fScale,
-			(icon->fHeight - icon->iQuickInfoHeight * fRatio) * icon->fScale);
+			(- icon->iQuickInfoWidth / fMaxScale * fRatio + icon->fWidthFactor * icon->fWidth) / 2 * icon->fScale,
+			(icon->fHeight - icon->iQuickInfoHeight / fMaxScale * fRatio) * icon->fScale);
 		
 		cairo_scale (pCairoContext,
-			fRatio * icon->fScale / (1 + myIconsParam.fAmplitude) * 1,
-			fRatio * icon->fScale / (1 + myIconsParam.fAmplitude) * 1);
+			fRatio * icon->fScale / fMaxScale,
+			fRatio * icon->fScale / fMaxScale);
 		
 		cairo_set_source_surface (pCairoContext,
 			icon->pQuickInfoBuffer,
@@ -652,85 +651,33 @@ void cairo_dock_render_one_icon (Icon *icon, CairoDock *pDock, cairo_t *pCairoCo
 }
 
 
-void cairo_dock_render_one_icon_in_desklet (Icon *icon, cairo_t *pCairoContext, gboolean bUseReflect, gboolean bUseText, int iWidth)
+void cairo_dock_render_one_icon_in_desklet (Icon *icon, CairoContainer *pContainer, cairo_t *pCairoContext, gboolean bUseText)
 {
 	//\_____________________ On dessine l'icone en fonction de son placement, son angle, et sa transparence.
 	//g_print ("%s (%.2f;%.2f x %.2f)\n", __func__, icon->fDrawX, icon->fDrawY, icon->fScale);
-	cairo_translate (pCairoContext, icon->fDrawX, icon->fDrawY);
-	cairo_save (pCairoContext);
-	cairo_scale (pCairoContext, icon->fWidthFactor * icon->fScale, icon->fHeightFactor * icon->fScale);
-	if (icon->fOrientation != 0)
-		cairo_rotate (pCairoContext, icon->fOrientation);
-	
-	double fAlpha = icon->fAlpha;
-	
-	if (bUseReflect && icon->pReflectionBuffer != NULL)  // on dessine les reflets.
+	if (icon->pIconBuffer != NULL)
 	{
-		if (icon->pIconBuffer != NULL)
-			cairo_set_source_surface (pCairoContext, icon->pIconBuffer, 0.0, 0.0);
-		if (fAlpha == 1)
-			cairo_paint (pCairoContext);
-		else
-			cairo_paint_with_alpha (pCairoContext, fAlpha);
-
-		cairo_restore (pCairoContext);  // retour juste apres la translation (fDrawX, fDrawY).
-
 		cairo_save (pCairoContext);
-		cairo_translate (pCairoContext, 0, - icon->fDeltaYReflection + icon->fHeight * icon->fScale);
+		
+		cairo_translate (pCairoContext, icon->fDrawX, icon->fDrawY);
 		cairo_scale (pCairoContext, icon->fWidthFactor * icon->fScale, icon->fHeightFactor * icon->fScale);
-		
-		cairo_set_source_surface (pCairoContext, icon->pReflectionBuffer, 0.0, 0.0);
-		
-		if (myBackendsParam.bDynamicReflection && icon->fScale != 1)
-		{
-			cairo_pattern_t *pGradationPattern = cairo_pattern_create_linear (0.,
-				0.,
-				0.,
-				myIconsParam.fReflectSize / icon->fScale);  // de haut en bas.
-			g_return_if_fail (cairo_pattern_status (pGradationPattern) == CAIRO_STATUS_SUCCESS);
-			
-			cairo_pattern_set_extend (pGradationPattern, CAIRO_EXTEND_NONE);
-			cairo_pattern_add_color_stop_rgba (pGradationPattern,
-				0.,
-				0.,
-				0.,
-				0.,
-				1.);  // astuce pour ne pas avoir a re-creer la surface de la reflection.
-			cairo_pattern_add_color_stop_rgba (pGradationPattern,
-				1.,
-				0.,
-				0.,
-				0.,
-				0.);
-			
-			cairo_save (pCairoContext);
-			cairo_set_operator (pCairoContext, CAIRO_OPERATOR_OVER);
-			cairo_translate (pCairoContext, 0, 0);
-			cairo_mask (pCairoContext, pGradationPattern);
-			cairo_restore (pCairoContext);
-			
-			cairo_pattern_destroy (pGradationPattern);
-		}
-		else
-		{
-			if (fAlpha == 1)
-				cairo_paint (pCairoContext);
-			else
-				cairo_paint_with_alpha (pCairoContext, fAlpha);
-		}
-	}
-	else  // on dessine l'icone tout simplement.
-	{
-		if (icon->pIconBuffer != NULL)
-			cairo_set_source_surface (pCairoContext, icon->pIconBuffer, 0.0, 0.0);
-		if (fAlpha == 1)
+		if (icon->fOrientation != 0)
+			cairo_rotate (pCairoContext, icon->fOrientation);
+
+		cairo_set_source_surface (pCairoContext, icon->pIconBuffer, 0.0, 0.0);
+		if (icon->fAlpha == 1)
 			cairo_paint (pCairoContext);
 		else
-			cairo_paint_with_alpha (pCairoContext, fAlpha);
+			cairo_paint_with_alpha (pCairoContext, icon->fAlpha);
+		
+		cairo_restore (pCairoContext);  // retour juste apres la translation (fDrawX, fDrawY).
+		
+		if (pContainer->bUseReflect)
+		{
+			cairo_dock_draw_icon_reflect_cairo (icon, pContainer, pCairoContext);
+		}
 	}
 	
-	cairo_restore (pCairoContext);  // retour juste apres la translation (fDrawX, fDrawY).
-	
 	//\_____________________ On dessine les etiquettes, avec un alpha proportionnel au facteur d'echelle de leur icone.
 	if (bUseText && icon->pTextBuffer != NULL)
 	{
@@ -738,8 +685,8 @@ void cairo_dock_render_one_icon_in_desklet (Icon *icon, cairo_t *pCairoContext,
 		double fOffsetX = (icon->fWidthFactor * icon->fWidth * icon->fScale - icon->iTextWidth) / 2;
 		if (fOffsetX < - icon->fDrawX)
 			fOffsetX = - icon->fDrawX;
-		else if (icon->fDrawX + fOffsetX + icon->iTextWidth > iWidth)
-			fOffsetX = iWidth - icon->iTextWidth - icon->fDrawX;
+		else if (icon->fDrawX + fOffsetX + icon->iTextWidth > pContainer->iWidth)
+			fOffsetX = pContainer->iWidth - icon->iTextWidth - icon->fDrawX;
 		if (icon->fOrientation != 0)
 		{
 			cairo_rotate (pCairoContext, icon->fOrientation);
@@ -759,8 +706,6 @@ void cairo_dock_render_one_icon_in_desklet (Icon *icon, cairo_t *pCairoContext,
 	if (icon->pQuickInfoBuffer != NULL)
 	{
 		cairo_translate (pCairoContext,
-			//-icon->fQuickInfoXOffset + icon->fWidth / 2,
-			//icon->fHeight - icon->fQuickInfoYOffset);
 			(- icon->iQuickInfoWidth + icon->fWidth) / 2 * icon->fScale,
 			(icon->fHeight - icon->iQuickInfoHeight) * icon->fScale);
 		
@@ -781,7 +726,7 @@ void cairo_dock_render_one_icon_in_desklet (Icon *icon, cairo_t *pCairoContext,
 void cairo_dock_draw_string (cairo_t *pCairoContext, CairoDock *pDock, double fStringLineWidth, gboolean bIsLoop, gboolean bForceConstantSeparator)
 {
 	bForceConstantSeparator = bForceConstantSeparator || myIconsParam.bConstantSeparatorSize;
-	GList *ic, *pFirstDrawnElement = (pDock->pFirstDrawnElement != NULL ? pDock->pFirstDrawnElement : pDock->icons);
+	GList *ic, *pFirstDrawnElement = pDock->icons;
 	if (pFirstDrawnElement == NULL || fStringLineWidth <= 0)
 		return ;
 
@@ -872,7 +817,6 @@ void cairo_dock_draw_string (cairo_t *pCairoContext, CairoDock *pDock, double fS
 
 void cairo_dock_render_icons_linear (cairo_t *pCairoContext, CairoDock *pDock)
 {
-	//GList *pFirstDrawnElement = (pDock->pFirstDrawnElement != NULL ? pDock->pFirstDrawnElement : pDock->icons);
 	GList *pFirstDrawnElement = cairo_dock_get_first_drawn_element_linear (pDock->icons);
 	if (pFirstDrawnElement == NULL)
 		return;
@@ -973,28 +917,33 @@ void cairo_dock_render_hidden_dock (cairo_t *pCairoContext, CairoDock *pDock)
 	Icon *icon;
 	GList *ic = pFirstDrawnElement;
 	double pHiddenBgColor[4];
-	double r = 3;
+	const double r = 4; // corner radius of the background
+	const double gap = 3;  // gap to the screen
+	double dw = (myIconsParam.iIconGap > 2 ? 2 : 0);  // 1px margin around the icons for a better readability (only if icons won't be stuck togather then).
+	double w, h;
 	do
 	{
 		icon = ic->data;
 		if (icon->bIsDemandingAttention || icon->bAlwaysVisible)
 		{
 			y = icon->fDrawY;
-			icon->fDrawY = (pDock->container.bDirectionUp ? pDock->container.iHeight - icon->fHeight * icon->fScale : 0.);
+			icon->fDrawY = (pDock->container.bDirectionUp ? pDock->container.iHeight - icon->fHeight * icon->fScale  - gap: gap);
 			
 			if (icon->pHiddenBgColor)
 			{
 				cairo_save (pCairoContext);
 				cairo_set_source_rgba (pCairoContext, icon->pHiddenBgColor[0], icon->pHiddenBgColor[1], icon->pHiddenBgColor[2], icon->pHiddenBgColor[3] * pDock->fPostHideOffset);
+				w = icon->fWidth * icon->fScale;
+				h = icon->fHeight * icon->fScale;
 				if (pDock->container.bIsHorizontal)
 				{
-					cairo_translate (pCairoContext, icon->fDrawX, icon->fDrawY);
-					cairo_dock_draw_rounded_rectangle (pCairoContext, r, 0, icon->fWidth - 2*r, icon->fHeight);
+					cairo_translate (pCairoContext, icon->fDrawX - dw / 2, icon->fDrawY);
+					cairo_dock_draw_rounded_rectangle (pCairoContext, r, 0, w - 2*r + dw, h);
 				}
 				else
 				{
-					cairo_translate (pCairoContext, icon->fDrawY, icon->fDrawX);
-					cairo_dock_draw_rounded_rectangle (pCairoContext, r, 0, icon->fHeight - 2*r, icon->fWidth);
+					cairo_translate (pCairoContext, icon->fDrawY - dw / 2, icon->fDrawX);
+					cairo_dock_draw_rounded_rectangle (pCairoContext, r, 0, h - 2*r + dw, w);
 				}
 				cairo_fill (pCairoContext);
 				cairo_restore (pCairoContext);
