@@ -51,7 +51,7 @@ struct _CairoGuiManager {
 
 // signals
 typedef enum {
-	NOTIFICATION_SHOW_MODULE_INSTANCE_GUI,
+	NOTIFICATION_SHOW_MODULE_INSTANCE_GUI = NB_NOTIFICATIONS_OBJECT,
 	NOTIFICATION_STATUS_MESSAGE,
 	NOTIFICATION_GET_GROUP_KEY_WIDGET,
 	NB_NOTIFICATIONS_GUI
