@@ -23,7 +23,8 @@
 #include <glib.h>
 
 #include "cairo-dock-struct.h"
-#include "cairo-dock-notifications.h"
+#include "cairo-dock-object.h"
+
 G_BEGIN_DECLS
 
 /**
@@ -53,13 +54,11 @@ typedef enum {
 	CAIRO_DOCK_NB_ICON_TYPES
 	} CairoDockIconTrueType;
 
-/// Definition of the groups of icons.
+/// Available groups of icons.
 typedef enum {
-	CAIRO_DOCK_LAUNCHER = 0,
+	CAIRO_DOCK_LAUNCHER = 0,  // launchers and applets, and applis if mixed
 	CAIRO_DOCK_SEPARATOR12,
 	CAIRO_DOCK_APPLI,
-	CAIRO_DOCK_SEPARATOR23,
-	CAIRO_DOCK_APPLET,
 	CAIRO_DOCK_NB_GROUPS
 	} CairoDockIconGroup;
 
@@ -87,15 +86,15 @@ struct _IconInterface {
 /// Definition of an Icon.
 struct _Icon {
 	//\____________ Definition.
-	/// list of available notifications.
-	GPtrArray *pNotificationsTab;
+	/// object
+	GldiObject object;
+	gpointer pDataSlot[CAIRO_DOCK_NB_DATA_SLOT];
 	/// type of the icon.
 	CairoDockIconTrueType iTrueType;
 	/// group of the icon.
 	CairoDockIconGroup iGroup;
 	/// interface
 	IconInterface iface;
-	gpointer pDataSlot[CAIRO_DOCK_NB_DATA_SLOT];
 	
 	//\____________ properties.
 	// generic.
@@ -150,6 +149,7 @@ struct _Icon {
 	gint iLastCheckTime;
 	gchar *cInitialName;
 	gchar *cLastAttentionDemand;
+	gint iAge;  // age of the window (a mere growing integer).
 	Pixmap iBackingPixmap;
 	//Damage iDamageHandle;
 	
@@ -162,7 +162,7 @@ struct _Icon {
 	gint iImageWidth, iImageHeight;  // taille de la surface/texture telle qu'elle a ete creee.
 	cairo_surface_t* pIconBuffer;
 	GLuint iIconTexture;
-	cairo_surface_t* pReflectionBuffer;
+	cairo_surface_t* pReflectionBuffer_deprecated;
 	
 	gint iTextWidth, iTextHeight;
 	cairo_surface_t* pTextBuffer;
@@ -197,7 +197,8 @@ struct _Icon {
 	guint iSidRedrawSubdockContent;
 	guint iSidLoadImage;
 	guint iSidDoubleClickDelay;
-	CairoContainer *pContainerForLoad;
+	/// container where the icon is currently.
+	CairoContainer *pContainer;
 	gint iNbDoubleClickListeners;
 	
 	gchar **pMimeTypes;
@@ -206,7 +207,11 @@ struct _Icon {
 	gchar *cWmClass;
 	GList *pOverlays;
 	gdouble *pHiddenBgColor;
-	//gpointer reserved[0];
+	
+	gint iThumbnailX, iThumbnailY;  // X icon geometry for apps
+	gint iThumbnailWidth, iThumbnailHeight;
+	gboolean bDamaged;  // TRUE when the icon couldn't draw its surface, because the Gl context was not yet ready.
+	gpointer reserved[1];
 };
 
 typedef void (*CairoIconContainerLoadFunc) (void);
@@ -285,7 +290,6 @@ void cairo_dock_free_icon_buffers (Icon *icon);
 */
 void cairo_dock_add_reflection_to_icon (Icon *pIcon, CairoContainer *pContainer);
 
-void cairo_dock_set_icon_size (CairoContainer *pContainer, Icon *icon);
 
 /**Fill the image buffer (surface & texture) of a given icon, according to its type. Set its size if necessary, and fills the reflection buffer for cairo.
 *@param icon the icon.
@@ -302,9 +306,8 @@ void cairo_dock_load_icon_text (Icon *icon, CairoDockLabelDescription *pTextDesc
 /**Fill the quick-info buffer (surface & texture) of a given icon, according to a text description.
 *@param icon the icon.
 *@param pTextDescription desctiption of the text rendering.
-*@param fMaxScale maximum zoom.
 */
-void cairo_dock_load_icon_quickinfo (Icon *icon, CairoDockLabelDescription *pTextDescription, double fMaxScale);
+void cairo_dock_load_icon_quickinfo (Icon *icon, CairoDockLabelDescription *pTextDescription);
 
 /** Fill all the buffers (surfaces & textures) of a given icon, according to its type. Set its size accordingly, and fills the reflection buffer for cairo. Label and quick-info are loaded with the current global text description.
 *@param pIcon the icon.
@@ -312,10 +315,7 @@ void cairo_dock_load_icon_quickinfo (Icon *icon, CairoDockLabelDescription *pTex
 */
 void cairo_dock_load_icon_buffers (Icon *pIcon, CairoContainer *pContainer);
 
-void cairo_dock_trigger_load_icon_buffers (Icon *pIcon, CairoContainer *pContainer);
-
-void cairo_dock_reload_buffers_in_dock (CairoDock *pDock, gboolean bReloadAppletsToo, gboolean bRecursive);
-#define cairo_dock_load_buffers_in_one_dock(pDock) cairo_dock_reload_buffers_in_dock (pDock, TRUE, FALSE)
+void cairo_dock_trigger_load_icon_buffers (Icon *pIcon);
 
 void cairo_dock_reload_icon_image (Icon *icon, CairoContainer *pContainer);
 
