@@ -23,15 +23,8 @@
 
 #include <gtk/gtk.h>
 
-#ifdef HAVE_GLITZ
-#include <gdk/gdkx.h>
-#include <glitz-glx.h>
-#include <cairo-glitz.h>
-#endif
-
-#include <gtk/gtkgl.h>
-
 #include "gldi-config.h"
+#include "cairo-dock-notifications.h"
 #include "cairo-dock-icon-factory.h"
 #include "cairo-dock-module-factory.h"  // cairo_dock_deinstanciate_module
 #include "cairo-dock-log.h"
@@ -62,7 +55,7 @@ CairoIconsManager myIconsMgr;
 CairoDockImageBuffer g_pIconBackgroundBuffer;
 GLuint g_pGradationTexture[2]={0, 0};
 
-// dependancies
+// dependencies
 extern CairoDock *g_pMainDock;
 extern gchar *g_cCurrentThemePath;
 extern gboolean g_bUseOpenGL;
@@ -78,9 +71,11 @@ static int s_iNbNonStickyLaunchers = 0;
 static GtkIconTheme *s_pIconTheme = NULL;
 static gboolean s_bUseLocalIcons = FALSE;
 static gboolean s_bUseDefaultTheme = TRUE;
+static guint s_iSidReloadTheme = 0;
 
 static void _cairo_dock_unload_icon_textures (void);
 static void _cairo_dock_unload_icon_theme (void);
+static void _on_icon_theme_changed (GtkIconTheme *pIconTheme, gpointer data);
 
 
 void cairo_dock_free_icon (Icon *icon)
@@ -102,8 +97,8 @@ void cairo_dock_free_icon (Icon *icon)
 		cairo_dock_deinhibite_class (icon->cClass, icon);
 	if (icon->pModuleInstance != NULL)
 		cairo_dock_deinstanciate_module (icon->pModuleInstance);
-	cairo_dock_notify_on_object (&myIconsMgr, NOTIFICATION_STOP_ICON, icon);
 	cairo_dock_notify_on_object (icon, NOTIFICATION_STOP_ICON, icon);
+	cairo_dock_notify_on_object (icon, NOTIFICATION_DESTROY, icon);
 	cairo_dock_remove_transition_on_icon (icon);
 	cairo_dock_remove_data_renderer_on_icon (icon);
 	
@@ -141,7 +136,7 @@ static CairoDock *_cairo_dock_insert_floating_icon_in_dock (Icon *icon, CairoDoc
 	g_return_val_if_fail (pParentDock != NULL, NULL);
 	
 	//\_________________ On l'insere dans son dock parent (sans animation, puisqu'on n'anime pas non plus son enlevement).
-	cairo_dock_insert_icon_in_dock (icon, pParentDock, CAIRO_DOCK_UPDATE_DOCK_SIZE, ! CAIRO_DOCK_ANIMATE_ICON);
+	cairo_dock_insert_icon_in_dock (icon, pParentDock, ! CAIRO_DOCK_ANIMATE_ICON);
 	cd_message (" insertion de %s complete (%.2f %.2fx%.2f) dans %s", icon->cName, icon->fInsertRemoveFactor, icon->fWidth, icon->fHeight, icon->cParentDockName);
 	
 	return pParentDock;
@@ -154,14 +149,14 @@ static CairoDock * _cairo_dock_detach_launcher(Icon *pIcon)
 		return NULL;
 
 	gchar *cParentDockName = g_strdup(pIcon->cParentDockName);
-	cairo_dock_detach_icon_from_dock (pIcon, pParentDock, TRUE); // this will set cParentDockName to NULL
+	cairo_dock_detach_icon_from_dock (pIcon, pParentDock); // this will set cParentDockName to NULL
 	
 	pIcon->cParentDockName = cParentDockName; // put it back !
 
-	cairo_dock_update_dock_size (pParentDock);
+	///cairo_dock_update_dock_size (pParentDock);
 	return pParentDock;
 }
-static void _cairo_dock_hide_show_launchers_on_other_desktops (Icon *icon, CairoContainer *pContainer, CairoDock *pMainDock)
+static void _cairo_dock_hide_show_launchers_on_other_desktops (Icon *icon, CairoDock *pMainDock)
 {
 	if (CAIRO_DOCK_ICON_TYPE_IS_LAUNCHER (icon) || CAIRO_DOCK_ICON_TYPE_IS_CONTAINER (icon))
 	{
@@ -202,10 +197,10 @@ void cairo_dock_hide_show_launchers_on_other_desktops (CairoDock *pDock)
 {
 	if (s_iNbNonStickyLaunchers <= 0)
 		return ;
-	// first we detach what shouldn't be show on this viewport
-	cairo_dock_foreach_icons_of_type (pDock->icons, CAIRO_DOCK_LAUNCHER, (CairoDockForeachIconFunc)_cairo_dock_hide_show_launchers_on_other_desktops, pDock);
+	// first we detach what shouldn't be shown on this viewport
+	g_list_foreach (pDock->icons, (GFunc)_cairo_dock_hide_show_launchers_on_other_desktops, pDock);
 	// then we reattach what was eventually missing
-	cairo_dock_foreach_icons_of_type (s_pFloatingIconsList, CAIRO_DOCK_LAUNCHER, (CairoDockForeachIconFunc)_cairo_dock_hide_show_launchers_on_other_desktops, pDock);
+	g_list_foreach (s_pFloatingIconsList, (GFunc)_cairo_dock_hide_show_launchers_on_other_desktops, pDock);
 }
 
 static gboolean _on_change_current_desktop_viewport_notification (gpointer data)
@@ -248,11 +243,28 @@ void cairo_dock_set_specified_desktop_for_icon (Icon *pIcon, int iSpecificDeskto
  /// ICON THEME ///
 //////////////////
 
-gchar *cairo_dock_search_icon_s_path (const gchar *cFileName)
+/*
+ * GTK_ICON_SIZE_MENU:          16
+ * GTK_ICON_SIZE_SMALL_TOOLBAR: 18
+ * GTK_ICON_SIZE_BUTTON:        20
+ * GTK_ICON_SIZE_LARGE_TOOLBAR: 24
+ * GTK_ICON_SIZE_DND:           32
+ * GTK_ICON_SIZE_DIALOG:        48
+ */
+gint cairo_dock_search_icon_size (GtkIconSize iIconSize)
+{
+	gint iWidth, iHeight;
+	if (! gtk_icon_size_lookup (iIconSize, &iWidth, &iHeight))
+		return CAIRO_DOCK_DEFAULT_ICON_SIZE;
+
+	return MAX (iWidth, iHeight);
+}
+
+gchar *cairo_dock_search_icon_s_path (const gchar *cFileName, gint iDesiredIconSize)
 {
 	g_return_val_if_fail (cFileName != NULL, NULL);
 	
-	//\_______________________ cas faciles : l'entree est deja un chemin.
+	//\_______________________ easy cases: we receive a path.
 	if (*cFileName == '~')
 	{
 		return g_strdup_printf ("%s%s", g_getenv ("HOME"), cFileName+1);
@@ -311,7 +323,7 @@ gchar *cairo_dock_search_icon_s_path (const gchar *cFileName)
 		}
 		pIconInfo = gtk_icon_theme_lookup_icon (s_pIconTheme,
 			sIconPath->str,
-			128,
+			iDesiredIconSize, // GTK_ICON_LOOKUP_FORCE_SIZE if size < 30 ?? -> icons can be different // a lot of themes now use only svg files.
 			GTK_ICON_LOOKUP_FORCE_SVG);
 		if (pIconInfo != NULL)
 		{
@@ -336,7 +348,7 @@ gchar *cairo_dock_search_icon_s_path (const gchar *cFileName)
 		{
 			*(str+1) = '\0';
 			cd_debug (" on cherche '%s'...\n", sIconPath->str);
-			gchar *cPath = cairo_dock_search_icon_s_path (sIconPath->str);
+			gchar *cPath = cairo_dock_search_icon_s_path (sIconPath->str, iDesiredIconSize);
 			if (cPath != NULL)
 			{
 				bFileFound = TRUE;
@@ -357,87 +369,59 @@ gchar *cairo_dock_search_icon_s_path (const gchar *cFileName)
 	return cIconPath;
 }
 
+void cairo_dock_add_path_to_icon_theme (const gchar *cThemePath)
+{
+	g_signal_handlers_block_matched (s_pIconTheme,
+		(GSignalMatchType) G_SIGNAL_MATCH_FUNC,
+		0, 0, NULL, _on_icon_theme_changed, NULL);
+	gtk_icon_theme_append_search_path (s_pIconTheme,
+		cThemePath);  /// TODO: does it check for unicity ?...
+	g_signal_handlers_unblock_matched (s_pIconTheme,
+		(GSignalMatchType) G_SIGNAL_MATCH_FUNC,
+		0, 0, NULL, _on_icon_theme_changed, NULL);  // will do nothing if the callback has not been connected
+}
+
+void cairo_dock_remove_path_from_icon_theme (const gchar *cThemePath)
+{
+	g_signal_handlers_block_matched (s_pIconTheme,
+		(GSignalMatchType) G_SIGNAL_MATCH_FUNC,
+		0, 0, NULL, _on_icon_theme_changed, NULL);
+	
+	gchar **paths = NULL;
+	gint iNbPaths = 0;
+	gtk_icon_theme_get_search_path (s_pIconTheme, &paths, &iNbPaths);
+	int i;
+	for (i = 0; i < iNbPaths; i++)  // on cherche sa position dans le tableau.
+	{
+		if (strcmp (paths[i], cThemePath))
+			break;
+	}
+	if (i < iNbPaths)  // trouve
+	{
+		g_free (paths[i]);
+		for (i = i+1; i < iNbPaths; i++)  // on decale tous les suivants vers l'arriere.
+		{
+			paths[i-1] = paths[i];
+		}
+		paths[i-1] = NULL;
+		gtk_icon_theme_set_search_path (s_pIconTheme, (const gchar **)paths, iNbPaths - 1);
+	}
+	g_strfreev (paths);
+	
+	g_signal_handlers_unblock_matched (s_pIconTheme,
+		(GSignalMatchType) G_SIGNAL_MATCH_FUNC,
+		0, 0, NULL, _on_icon_theme_changed, NULL);  // will do nothing if the callback has not been connected
+}
+
 
   //////////////////
  /// GET CONFIG ///
 //////////////////
 
-static const gchar * s_cIconTypeNames[(CAIRO_DOCK_NB_GROUPS+1)/2] = {"launchers", "applications", "applets"};
-
 static gboolean get_config (GKeyFile *pKeyFile, CairoIconsParam *pIcons)
 {
 	gboolean bFlushConfFileNeeded = FALSE;
 	
-	//\___________________ Ordre des icones.
-	int i;
-	for (i = 0; i < CAIRO_DOCK_NB_GROUPS; i ++)
-		pIcons->tIconTypeOrder[i] = i;
-	gsize length=0;
-	
-	int iSeparateIcons = 0;
-	if (! g_key_file_has_key (pKeyFile, "Icons", "separate_icons", NULL))  // old parameters.
-	{
-		if (!g_key_file_get_boolean (pKeyFile, "Icons", "mix applets with launchers", NULL))
-		{
-			if (!g_key_file_get_boolean (pKeyFile, "Icons", "mix applis with launchers", NULL))
-				iSeparateIcons = 3;
-			else
-				iSeparateIcons = 2;
-		}
-		else if (!g_key_file_get_boolean (pKeyFile, "Icons", "mix applis with launchers", NULL))
-			iSeparateIcons = 1;
-	}
-	pIcons->iSeparateIcons = cairo_dock_get_integer_key_value (pKeyFile, "Icons", "separate_icons", &bFlushConfFileNeeded, iSeparateIcons , NULL, NULL);
-	
-	cairo_dock_get_integer_list_key_value (pKeyFile, "Icons", "icon's type order", &bFlushConfFileNeeded, pIcons->iIconsTypesList, 3, NULL, "Cairo Dock", NULL);  // on le recupere meme si on ne separe pas les icones, pour le panneau de conf simple.
-	if (pIcons->iIconsTypesList[0] == 0 && pIcons->iIconsTypesList[1] == 0)  // old format.
-	{
-		cd_debug ("icon's type order : old format\n");
-		gchar **cIconsTypesList = cairo_dock_get_string_list_key_value (pKeyFile, "Icons", "icon's type order", &bFlushConfFileNeeded, &length, NULL, "Cairo Dock", NULL);
-		
-		if (cIconsTypesList != NULL && length > 0)
-		{
-			cd_debug (" conversion ...\n");
-			unsigned int i, j;
-			for (i = 0; i < length; i ++)
-			{
-				cd_debug (" %d) %s\n", i, cIconsTypesList[i]);
-				for (j = 0; j < ((CAIRO_DOCK_NB_GROUPS + 1) / 2); j ++)
-				{
-					if (strcmp (cIconsTypesList[i], s_cIconTypeNames[j]) == 0)
-					{
-						cd_debug ("   => %d\n", j);
-						pIcons->tIconTypeOrder[2*j] = 2 * i;
-					}
-				}
-			}
-		}
-		g_strfreev (cIconsTypesList);
-		
-		pIcons->iIconsTypesList[0] = pIcons->tIconTypeOrder[2*0]/2;
-		pIcons->iIconsTypesList[1] = pIcons->tIconTypeOrder[2*1]/2;
-		pIcons->iIconsTypesList[2] = pIcons->tIconTypeOrder[2*2]/2;
-		cd_debug ("mise a jour avec {%d;%d;%d}\n", pIcons->iIconsTypesList[0], pIcons->iIconsTypesList[1], pIcons->iIconsTypesList[2]);
-		g_key_file_set_integer_list (pKeyFile, "Icons", "icon's type order", pIcons->iIconsTypesList, 3);
-		bFlushConfFileNeeded = TRUE;
-	}
-	
-	for (i = 0; i < 3; i ++)
-		pIcons->tIconTypeOrder[2*pIcons->iIconsTypesList[i]] = 2*i;
-	if (pIcons->iSeparateIcons == 0)
-	{
-		for (i = 0; i < 3; i ++)
-			pIcons->tIconTypeOrder[2*i] = 0;
-	}
-	else if (pIcons->iSeparateIcons == 1)
-	{
-		pIcons->tIconTypeOrder[CAIRO_DOCK_APPLET] = pIcons->tIconTypeOrder[CAIRO_DOCK_LAUNCHER];
-	}
-	else if (pIcons->iSeparateIcons == 2)
-	{
-		pIcons->tIconTypeOrder[CAIRO_DOCK_APPLI] = pIcons->tIconTypeOrder[CAIRO_DOCK_LAUNCHER];
-	}
-	
 	//\___________________ Reflets.
 	pIcons->fReflectHeightRatio = cairo_dock_get_double_key_value (pKeyFile, "Icons", "field depth", &bFlushConfFileNeeded, 0.7, NULL, NULL);
 
@@ -491,37 +475,21 @@ static gboolean get_config (GKeyFile *pKeyFile, CairoIconsParam *pIcons)
 		g_free (cLauncherBackgroundImageName);
 	}
 		
-	//\___________________ Parametres des lanceurs.
-	cairo_dock_get_size_key_value_helper (pKeyFile, "Icons", "launcher ", bFlushConfFileNeeded, pIcons->tIconAuthorizedWidth[CAIRO_DOCK_LAUNCHER], pIcons->tIconAuthorizedHeight[CAIRO_DOCK_LAUNCHER]);
-	if (pIcons->tIconAuthorizedWidth[CAIRO_DOCK_LAUNCHER] == 0)
-		pIcons->tIconAuthorizedWidth[CAIRO_DOCK_LAUNCHER] = 48;
-	if (pIcons->tIconAuthorizedHeight[CAIRO_DOCK_LAUNCHER] == 0)
-		pIcons->tIconAuthorizedHeight[CAIRO_DOCK_LAUNCHER] = 48;
-	
-	//\___________________ Parametres des applis.
-	cairo_dock_get_size_key_value_helper (pKeyFile, "Icons", "appli ", bFlushConfFileNeeded, pIcons->tIconAuthorizedWidth[CAIRO_DOCK_APPLI], pIcons->tIconAuthorizedHeight[CAIRO_DOCK_APPLI]);
-	if (pIcons->tIconAuthorizedWidth[CAIRO_DOCK_APPLI] == 0)
-		pIcons->tIconAuthorizedWidth[CAIRO_DOCK_APPLET] = pIcons->tIconAuthorizedWidth[CAIRO_DOCK_LAUNCHER];
-	if (pIcons->tIconAuthorizedHeight[CAIRO_DOCK_APPLI] == 0)
-		pIcons->tIconAuthorizedHeight[CAIRO_DOCK_APPLET] = pIcons->tIconAuthorizedHeight[CAIRO_DOCK_LAUNCHER];
-	
-	//\___________________ Parametres des applets.
-	cairo_dock_get_size_key_value_helper (pKeyFile, "Icons", "applet ", bFlushConfFileNeeded, pIcons->tIconAuthorizedWidth[CAIRO_DOCK_APPLET], pIcons->tIconAuthorizedHeight[CAIRO_DOCK_APPLET]);
-	if (pIcons->tIconAuthorizedWidth[CAIRO_DOCK_APPLET] == 0)
-		pIcons->tIconAuthorizedWidth[CAIRO_DOCK_APPLET] = pIcons->tIconAuthorizedWidth[CAIRO_DOCK_LAUNCHER];
-	if (pIcons->tIconAuthorizedHeight[CAIRO_DOCK_APPLET] == 0)
-		pIcons->tIconAuthorizedHeight[CAIRO_DOCK_APPLET] = pIcons->tIconAuthorizedHeight[CAIRO_DOCK_LAUNCHER];
+	//\___________________ icons size
+	cairo_dock_get_size_key_value_helper (pKeyFile, "Icons", "launcher ", bFlushConfFileNeeded, pIcons->iIconWidth, pIcons->iIconHeight);
+	if (pIcons->iIconWidth == 0)
+		pIcons->iIconWidth = 48;
+	if (pIcons->iIconHeight == 0)
+		pIcons->iIconHeight = 48;
 	
 	//\___________________ Parametres des separateurs.
-	cairo_dock_get_size_key_value_helper (pKeyFile, "Icons", "separator ", bFlushConfFileNeeded, pIcons->tIconAuthorizedWidth[CAIRO_DOCK_SEPARATOR12], pIcons->tIconAuthorizedHeight[CAIRO_DOCK_SEPARATOR12]);
-	if (pIcons->tIconAuthorizedWidth[CAIRO_DOCK_SEPARATOR12] == 0)
-		pIcons->tIconAuthorizedWidth[CAIRO_DOCK_SEPARATOR12] = pIcons->tIconAuthorizedWidth[CAIRO_DOCK_LAUNCHER];
-	if (pIcons->tIconAuthorizedHeight[CAIRO_DOCK_SEPARATOR12] == 0)
-		pIcons->tIconAuthorizedHeight[CAIRO_DOCK_SEPARATOR12] = pIcons->tIconAuthorizedHeight[CAIRO_DOCK_LAUNCHER];
-	pIcons->tIconAuthorizedHeight[CAIRO_DOCK_SEPARATOR12] = MIN (pIcons->tIconAuthorizedHeight[CAIRO_DOCK_SEPARATOR12], pIcons->tIconAuthorizedHeight[CAIRO_DOCK_LAUNCHER]);
-	
-	pIcons->tIconAuthorizedWidth[CAIRO_DOCK_SEPARATOR23] = pIcons->tIconAuthorizedWidth[CAIRO_DOCK_SEPARATOR12];
-	pIcons->tIconAuthorizedHeight[CAIRO_DOCK_SEPARATOR23] = pIcons->tIconAuthorizedHeight[CAIRO_DOCK_SEPARATOR12];
+	cairo_dock_get_size_key_value_helper (pKeyFile, "Icons", "separator ", bFlushConfFileNeeded, pIcons->iSeparatorWidth, pIcons->iSeparatorHeight);
+	if (pIcons->iSeparatorWidth == 0)
+		pIcons->iSeparatorWidth = pIcons->iIconWidth;
+	if (pIcons->iSeparatorHeight == 0)
+		pIcons->iSeparatorHeight = pIcons->iIconHeight;
+	if (pIcons->iSeparatorHeight > pIcons->iIconHeight)
+		pIcons->iSeparatorHeight = pIcons->iIconHeight;
 	
 	pIcons->iSeparatorType = cairo_dock_get_integer_key_value (pKeyFile, "Icons", "separator type", &bFlushConfFileNeeded, -1, NULL, NULL);
 	if (pIcons->iSeparatorType >= CAIRO_DOCK_NB_SEPARATOR_TYPES)  // nouveau parametre, avant il etait dans dock-rendering.
@@ -574,27 +542,7 @@ static gboolean get_config (GKeyFile *pKeyFile, CairoIconsParam *pIcons)
 
 	pIcons->bConstantSeparatorSize = cairo_dock_get_boolean_key_value (pKeyFile, "Icons", "force size", &bFlushConfFileNeeded, TRUE, "Separators", NULL);
 	
-	
-	pIcons->fReflectSize = 0;
-	for (i = 0; i < CAIRO_DOCK_NB_GROUPS; i ++)
-	{
-		if (pIcons->tIconAuthorizedHeight[i] > 0)
-			pIcons->fReflectSize = MAX (pIcons->fReflectSize, pIcons->tIconAuthorizedHeight[i]);
-	}
-	if (pIcons->fReflectSize == 0)  // on n'a pas trouve de hauteur, on va essayer avec la largeur.
-	{
-		for (i = 0; i < CAIRO_DOCK_NB_GROUPS; i ++)
-		{
-			if (pIcons->tIconAuthorizedWidth[i] > 0)
-				pIcons->fReflectSize = MAX (pIcons->fReflectSize, pIcons->tIconAuthorizedWidth[i]);
-		}
-		if (pIcons->fReflectSize > 0)
-			pIcons->fReflectSize = MIN (48, pIcons->fReflectSize);
-		else
-			pIcons->fReflectSize = 48;
-	}
-	pIcons->fReflectSize *= pIcons->fReflectHeightRatio;
-	
+	///pIcons->fReflectSize = pIcons->iIconHeight * pIcons->fReflectHeightRatio;
 	
 	//\___________________ labels font
 	CairoIconsParam *pLabels = pIcons;
@@ -665,6 +613,7 @@ static gboolean get_config (GKeyFile *pKeyFile, CairoIconsParam *pIcons)
 	
 	//\___________________ quick-info
 	memcpy (&pLabels->quickInfoTextDescription, &pLabels->iconTextDescription, sizeof (CairoDockLabelDescription));
+	pLabels->quickInfoTextDescription.iMargin = 1;  // to minimize the surface of the quick-info (0 would be too much).
 	pLabels->quickInfoTextDescription.cFont = g_strdup (pLabels->iconTextDescription.cFont);
 	pLabels->quickInfoTextDescription.iSize = 12;
 	pLabels->quickInfoTextDescription.iWeight = PANGO_WEIGHT_HEAVY;
@@ -745,14 +694,17 @@ static void reset_config (CairoIconsParam *pIcons)
  /// LOAD ///
 ////////////
 
-static void _cairo_dock_load_icons_background_surface (const gchar *cImagePath, double fMaxScale)
+static void _cairo_dock_load_icons_background_surface (const gchar *cImagePath)
 {
 	cairo_dock_unload_image_buffer (&g_pIconBackgroundBuffer);
 	
-	int iSize = MAX (myIconsParam.tIconAuthorizedWidth[CAIRO_DOCK_LAUNCHER], myIconsParam.tIconAuthorizedWidth[CAIRO_DOCK_APPLI]);
+	int iSize = myIconsParam.iIconWidth;
 	if (iSize == 0)
 		iSize = 48;
+	
+	double fMaxScale = cairo_dock_get_max_scale (g_pMainDock);
 	iSize *= fMaxScale;
+	
 	cairo_dock_load_image_buffer (&g_pIconBackgroundBuffer,
 		cImagePath,
 		iSize,
@@ -767,16 +719,31 @@ static void _load_renderer (const gchar *cRenderername, CairoIconContainerRender
 }
 static void _cairo_dock_load_icon_textures (void)
 {
-	double fMaxScale = cairo_dock_get_max_scale (g_pMainDock);
-	
-	_cairo_dock_load_icons_background_surface (myIconsParam.cBackgroundImagePath, fMaxScale);
+	_cairo_dock_load_icons_background_surface (myIconsParam.cBackgroundImagePath);
 	
 	cairo_dock_foreach_icon_container_renderer ((GHFunc)_load_renderer, NULL);
 }
+static void _reload_in_desklet (CairoDesklet *pDesklet, gpointer data)
+{
+	if (CAIRO_DOCK_IS_APPLET (pDesklet->pIcon))
+	{
+		cairo_dock_reload_module_instance (pDesklet->pIcon->pModuleInstance, FALSE);
+	}
+}
+static gboolean _on_icon_theme_changed_idle (gpointer data)
+{
+	cd_debug ("");
+	cairo_dock_foreach_desklet ((CairoDockForeachDeskletFunc) _reload_in_desklet, NULL);
+	cairo_dock_reload_buffers_in_all_docks (FALSE);
+	s_iSidReloadTheme = 0;
+	return FALSE;
+}
 static void _on_icon_theme_changed (GtkIconTheme *pIconTheme, gpointer data)
 {
 	cd_message ("theme has changed");
-	cairo_dock_reload_buffers_in_all_docks (TRUE);  // TRUE <=> y compris les applets.
+	// Reload the icons in idle, because this signal is triggered directly by 'gtk_icon_theme_set_search_path()'; so we may end reloading an applet in the middle of its work (ex.: Status-Notifier when the watcher terminates)
+	if (s_iSidReloadTheme == 0)
+		s_iSidReloadTheme = g_idle_add (_on_icon_theme_changed_idle, NULL);
 }
 static void _cairo_dock_load_icon_theme (void)
 {
@@ -799,11 +766,11 @@ static void _cairo_dock_load_icon_theme (void)
 
 static void load (void)
 {
-	_cairo_dock_load_icon_textures ();
-	
 	cairo_dock_create_icon_fbo ();
 	
 	_cairo_dock_load_icon_theme ();
+	
+	_cairo_dock_load_icon_textures ();
 }
 
 
@@ -827,28 +794,17 @@ static void _insert_separators (const gchar *cDockName, CairoDock *pDock, gpoint
 			cairo_dock_load_icon_image (icon, CAIRO_CONTAINER (pDock));
 		}
 	}
-	cairo_dock_insert_separators_in_dock (pDock);
+	cairo_dock_insert_automatic_separators_in_dock (pDock);
 }
 static void _calculate_icons (const gchar *cDockName, CairoDock *pDock, gpointer data)
 {
 	cairo_dock_calculate_dock_icons (pDock);
 }
-static void _reorder_icons (const gchar *cDockName, CairoDock *pDock, gpointer data)
-{
-	cairo_dock_remove_automatic_separators (pDock);
-
-	if (GPOINTER_TO_INT (data) && pDock->bIsMainDock)
-	{
-		cairo_dock_reorder_classes ();  // on re-ordonne les applis a cote des lanceurs/applets.
-	}
-	pDock->icons = g_list_sort (pDock->icons, (GCompareFunc) cairo_dock_compare_icons_order);
-}
 
 static void _reload_one_label (Icon *pIcon, CairoContainer *pContainer, CairoIconsParam *pLabels)
 {
 	cairo_dock_load_icon_text (pIcon, &pLabels->iconTextDescription);
-	double fMaxScale = cairo_dock_get_max_scale (pContainer);
-	cairo_dock_load_icon_quickinfo (pIcon, &pLabels->quickInfoTextDescription, fMaxScale);
+	cairo_dock_load_icon_quickinfo (pIcon, &pLabels->quickInfoTextDescription);
 }
 static void _cairo_dock_resize_one_dock (gchar *cDockName, CairoDock *pDock, gpointer data)
 {
@@ -857,28 +813,11 @@ static void _cairo_dock_resize_one_dock (gchar *cDockName, CairoDock *pDock, gpo
 
 static void reload (CairoIconsParam *pPrevIcons, CairoIconsParam *pIcons)
 {
-	double fMaxScale = cairo_dock_get_max_scale (g_pMainDock);
 	gboolean bInsertSeparators = FALSE;
 	
-	gboolean bGroupOrderChanged;
-	if (pPrevIcons->tIconTypeOrder[CAIRO_DOCK_LAUNCHER] != pIcons->tIconTypeOrder[CAIRO_DOCK_LAUNCHER] ||
-		pPrevIcons->tIconTypeOrder[CAIRO_DOCK_APPLI] != pIcons->tIconTypeOrder[CAIRO_DOCK_APPLI] ||
-		pPrevIcons->tIconTypeOrder[CAIRO_DOCK_APPLET] != pIcons->tIconTypeOrder[CAIRO_DOCK_APPLET] ||
-		pPrevIcons->iSeparateIcons != pIcons->iSeparateIcons)
-		bGroupOrderChanged = TRUE;
-	else
-		bGroupOrderChanged = FALSE;
-	
-	if (bGroupOrderChanged)
-	{
-		bInsertSeparators = TRUE;  // on enleve les separateurs avant de re-ordonner.
-		cairo_dock_foreach_docks ((GHFunc)_reorder_icons, GINT_TO_POINTER (pPrevIcons->iSeparateIcons && ! pIcons->iSeparateIcons));
-	}
-	
-	if ((pPrevIcons->iSeparateIcons && ! pIcons->iSeparateIcons) ||
-		cairo_dock_strings_differ (pPrevIcons->cSeparatorImage, pIcons->cSeparatorImage) ||
-		pPrevIcons->tIconAuthorizedWidth[CAIRO_DOCK_SEPARATOR12] != pIcons->tIconAuthorizedWidth[CAIRO_DOCK_SEPARATOR12] ||
-		pPrevIcons->tIconAuthorizedHeight[CAIRO_DOCK_SEPARATOR12] != pIcons->tIconAuthorizedHeight[CAIRO_DOCK_SEPARATOR12] ||
+	if (cairo_dock_strings_differ (pPrevIcons->cSeparatorImage, pIcons->cSeparatorImage) ||
+		pPrevIcons->iSeparatorWidth != pIcons->iSeparatorWidth ||
+		pPrevIcons->iSeparatorHeight != pIcons->iSeparatorHeight ||
 		pPrevIcons->fAmplitude != pIcons->fAmplitude)
 	{
 		bInsertSeparators = TRUE;
@@ -899,28 +838,24 @@ static void reload (CairoIconsParam *pPrevIcons, CairoIconsParam *pIcons)
 		pPrevIcons->fAmplitude != pIcons->fAmplitude)
 	{
 		bIconBackgroundImagesChanged = TRUE;
-		_cairo_dock_load_icons_background_surface (pIcons->cBackgroundImagePath, fMaxScale);
+		_cairo_dock_load_icons_background_surface (pIcons->cBackgroundImagePath);
 	}
 	
 	///cairo_dock_create_icon_pbuffer ();
 	cairo_dock_destroy_icon_fbo ();
 	cairo_dock_create_icon_fbo ();
 	
-	if (pPrevIcons->tIconAuthorizedWidth[CAIRO_DOCK_LAUNCHER] != pIcons->tIconAuthorizedWidth[CAIRO_DOCK_LAUNCHER] ||
-		pPrevIcons->tIconAuthorizedHeight[CAIRO_DOCK_LAUNCHER] != pIcons->tIconAuthorizedHeight[CAIRO_DOCK_LAUNCHER] ||
-		pPrevIcons->tIconAuthorizedWidth[CAIRO_DOCK_APPLI] != pIcons->tIconAuthorizedWidth[CAIRO_DOCK_APPLI] ||
-		pPrevIcons->tIconAuthorizedHeight[CAIRO_DOCK_APPLI] != pIcons->tIconAuthorizedHeight[CAIRO_DOCK_APPLI] ||
-		pPrevIcons->tIconAuthorizedWidth[CAIRO_DOCK_APPLET] != pIcons->tIconAuthorizedWidth[CAIRO_DOCK_APPLET] ||
-		pPrevIcons->tIconAuthorizedHeight[CAIRO_DOCK_APPLET] != pIcons->tIconAuthorizedHeight[CAIRO_DOCK_APPLET] ||
-		pPrevIcons->tIconAuthorizedWidth[CAIRO_DOCK_SEPARATOR12] != pIcons->tIconAuthorizedWidth[CAIRO_DOCK_SEPARATOR12] ||
-		pPrevIcons->tIconAuthorizedHeight[CAIRO_DOCK_SEPARATOR12] != pIcons->tIconAuthorizedHeight[CAIRO_DOCK_SEPARATOR12] ||
+	if (pPrevIcons->iIconWidth != pIcons->iIconWidth ||
+		pPrevIcons->iIconHeight != pIcons->iIconHeight ||
+		pPrevIcons->iSeparatorWidth != pIcons->iSeparatorWidth ||
+		pPrevIcons->iSeparatorHeight != pIcons->iSeparatorHeight ||
 		pPrevIcons->fAmplitude != pIcons->fAmplitude ||
 		(!g_bUseOpenGL && pPrevIcons->fReflectHeightRatio != pIcons->fReflectHeightRatio) ||
 		(!g_bUseOpenGL && pPrevIcons->fAlbedo != pIcons->fAlbedo) ||
 		bThemeChanged ||
 		bIconBackgroundImagesChanged)  // oui on ne fait pas dans la finesse.
 	{
-		cairo_dock_reload_buffers_in_all_docks (TRUE);  // TRUE <=> y compris les applets.
+		cairo_dock_reload_buffers_in_all_docks (TRUE);
 	}
 	
 	if (bInsertSeparators)
@@ -928,8 +863,8 @@ static void reload (CairoIconsParam *pPrevIcons, CairoIconsParam *pIcons)
 		cairo_dock_foreach_docks ((GHFunc)_insert_separators, NULL);
 	}
 	
-	if (pPrevIcons->tIconAuthorizedWidth[CAIRO_DOCK_LAUNCHER] != pIcons->tIconAuthorizedWidth[CAIRO_DOCK_LAUNCHER] ||
-		pPrevIcons->tIconAuthorizedHeight[CAIRO_DOCK_LAUNCHER] != pIcons->tIconAuthorizedHeight[CAIRO_DOCK_LAUNCHER] ||
+	if (pPrevIcons->iIconWidth != pIcons->iIconWidth ||
+		pPrevIcons->iIconHeight != pIcons->iIconHeight ||
 		pPrevIcons->fAmplitude != pIcons->fAmplitude)
 	{
 		_cairo_dock_unload_icon_textures ();
@@ -951,10 +886,6 @@ static void reload (CairoIconsParam *pPrevIcons, CairoIconsParam *pIcons)
 	{
 		cairo_dock_foreach_docks ((GHFunc) _cairo_dock_resize_one_dock, NULL);
 	}
-	if (pPrevLabels->bTextAlwaysHorizontal != pLabels->bTextAlwaysHorizontal)
-	{
-		cairo_dock_reload_buffers_in_all_docks (TRUE);  // les modules aussi.
-	}
 }
 
 
