@@ -45,6 +45,8 @@ struct _CairoDockImageBuffer {
 	gint iHeight;
 	gdouble fZoomX;
 	gdouble fZoomY;
+	gint iNbFrames;  // nb frames in the case of an animated image.
+	gint iCurrentFrame;
 	} ;
 
 void cairo_dock_free_label_description (CairoDockLabelDescription *pTextDescription);
@@ -68,6 +70,7 @@ CairoDockLabelDescription *cairo_dock_duplicate_label_description (CairoDockLabe
 gchar *cairo_dock_search_image_s_path (const gchar *cImageFile);
 #define cairo_dock_generate_file_path cairo_dock_search_image_s_path
 
+
 /** Load an image into an ImageBuffer with a given transparency. If the image is given by its sole name, it is taken in the root folder of the current theme.
 *@param pImage an ImageBuffer.
 *@param cImageFile name of a file
@@ -92,6 +95,9 @@ void cairo_dock_load_image_buffer_full (CairoDockImageBuffer *pImage, const gcha
 *@param iHeight height of the surface
 */
 void cairo_dock_load_image_buffer_from_surface (CairoDockImageBuffer *pImage, cairo_surface_t *pSurface, int iWidth, int iHeight);
+
+void cairo_dock_load_image_buffer_from_texture (CairoDockImageBuffer *pImage, GLuint iTexture);
+
 /** Create and load an image into an ImageBuffer. If the image is given by its sole name, it is taken in the root folder of the current theme.
 *@param cImageFile name of a file
 *@param iWidth width it should be loaded.
@@ -101,6 +107,12 @@ void cairo_dock_load_image_buffer_from_surface (CairoDockImageBuffer *pImage, ca
 */
 CairoDockImageBuffer *cairo_dock_create_image_buffer (const gchar *cImageFile, int iWidth, int iHeight, CairoDockLoadImageModifier iLoadModifier);
 
+#define cairo_dock_image_buffer_is_animated(pImage) ((pImage)->iNbFrames > 0)
+#define cairo_dock_image_buffer_next_frame(pImage) do {\
+	(pImage)->iCurrentFrame ++;\
+	if ((pImage)->iCurrentFrame >= (pImage)->iNbFrames)\
+		(pImage)->iCurrentFrame = 0; } while (0)
+
 /** Reset an ImageBuffer's ressources. It can be used to load another image then.
 *@param pImage an ImageBuffer.
 */
@@ -111,5 +123,13 @@ void cairo_dock_unload_image_buffer (CairoDockImageBuffer *pImage);
 void cairo_dock_free_image_buffer (CairoDockImageBuffer *pImage);
 
 
+void cairo_dock_apply_image_buffer_surface_with_offset (CairoDockImageBuffer *pImage, cairo_t *pCairoContext, double x, double y, double fAlpha);
+
+#define cairo_dock_apply_image_buffer_surface(pImage, pCairoContext) cairo_dock_apply_image_buffer_surface_with_offset (pImage, pCairoContext, 0., 0., 1.)
+
+void cairo_dock_apply_image_buffer_texture_with_offset (CairoDockImageBuffer *pImage, double x, double y);
+
+#define cairo_dock_apply_image_buffer_texture(pImage) cairo_dock_apply_image_buffer_texture_with_offset (pImage, 0., 0.)
+
 G_END_DECLS
 #endif
