@@ -21,13 +21,11 @@
 #include <math.h>
 #include <stdlib.h>
 
-#include <gdk/gdkx.h>
-#include <gtk/gtkgl.h>
-
 #include <X11/extensions/Xrender.h>
 #include <GL/glx.h>
 #include <GL/glxext.h>
 #include <GL/glu.h>
+#include <gdk/gdkx.h>
 
 #include "cairo-dock-log.h"
 #include "cairo-dock-icon-facility.h"
@@ -38,14 +36,19 @@
 #include "cairo-dock-X-manager.h"
 #include "cairo-dock-opengl.h"
 
+// public (manager, config, data)
 CairoDockGLConfig g_openglConfig;
 gboolean g_bUseOpenGL = FALSE;
 
+// dependancies
 extern CairoContainer *g_pPrimaryContainer;
 extern CairoDockDesktopGeometry g_desktopGeometry;
 extern CairoDockDesktopBackground *g_pFakeTransparencyDesktopBg;
 extern gboolean g_bEasterEggs;
 
+// private
+static gboolean s_bInitialized = FALSE;
+
 static inline gboolean _check_extension (const char *extName, const gchar *cExtensions)
 {
 	g_return_val_if_fail (cExtensions != NULL, FALSE);
@@ -84,11 +87,46 @@ static gboolean _check_client_glx_extension (const char *extName)
 	Display *display = gdk_x11_get_default_xdisplay ();
 	int screen = 0;
 	//const gchar *glxExtensions = glXQueryExtensionsString (display, screen);
-	const gchar *glxExtensions = glXGetClientString (display,GLX_EXTENSIONS);
+	const gchar *glxExtensions = glXGetClientString (display, GLX_EXTENSIONS);
 	return _check_extension (extName, glxExtensions);
 }
 
 
+static void _post_initialize_opengl_backend (GtkWidget *pWidget, CairoContainer *pContainer)  // initialisation necessitant un contexte opengl.
+{
+	g_return_if_fail (!s_bInitialized);
+	
+	if (! gldi_glx_make_current (pContainer))
+		return ;
+	
+	s_bInitialized = TRUE;
+	g_openglConfig.bNonPowerOfTwoAvailable = _check_gl_extension ("GL_ARB_texture_non_power_of_two");
+	g_openglConfig.bFboAvailable = _check_gl_extension ("GL_EXT_framebuffer_object");
+	if (!g_openglConfig.bFboAvailable)
+		cd_warning ("No FBO support, some applets will be invisible if placed inside the dock.");
+	
+	g_openglConfig.bNonPowerOfTwoAvailable = _check_gl_extension ("GL_ARB_texture_non_power_of_two");
+	g_openglConfig.bAccumBufferAvailable = _check_gl_extension ("GL_SUN_slice_accum");
+	
+	GLfloat fMaximumAnistropy = 0.;
+	if (_check_gl_extension ("GL_EXT_texture_filter_anisotropic"))
+	{
+		glGetFloatv(GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT, &fMaximumAnistropy);
+		glTexParameterf (GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, fMaximumAnistropy);
+	}
+	
+	cd_debug ("OpenGL config summary :\n - bNonPowerOfTwoAvailable : %d\n - bFboAvailable : %d\n - direct rendering : %d\n - bTextureFromPixmapAvailable : %d\n - bAccumBufferAvailable : %d\n - Anisotroy filtering level max : %.1f\n - OpenGL version: %s\n - OpenGL vendor: %s\n - OpenGL renderer: %s\n\n",
+		g_openglConfig.bNonPowerOfTwoAvailable,
+		g_openglConfig.bFboAvailable,
+		!g_openglConfig.bIndirectRendering,
+		g_openglConfig.bTextureFromPixmapAvailable,
+		g_openglConfig.bAccumBufferAvailable,
+		fMaximumAnistropy,
+		glGetString (GL_VERSION),
+		glGetString (GL_VENDOR),
+		glGetString (GL_RENDERER));
+}
+
 static inline XVisualInfo *_get_visual_from_fbconfigs (GLXFBConfig *pFBConfigs, int iNumOfFBConfigs, Display *XDisplay)
 {
 	XRenderPictFormat *pPictFormat;
@@ -135,18 +173,18 @@ gboolean cairo_dock_initialize_opengl_backend (gboolean bForceOpenGL)  // taken
 	GLXFBConfig *pFBConfigs;
 	XRenderPictFormat *pPictFormat = NULL;
 	int doubleBufferAttributes[] = {
-		GLX_DRAWABLE_TYPE, 	GLX_WINDOW_BIT,
+		GLX_DRAWABLE_TYPE, 		GLX_WINDOW_BIT,
 		GLX_RENDER_TYPE, 		GLX_RGBA_BIT,
-		GLX_DOUBLEBUFFER, 	True,
-		GLX_RED_SIZE, 		1,
+		GLX_DOUBLEBUFFER, 		True,
+		GLX_RED_SIZE, 			1,
 		GLX_GREEN_SIZE, 		1,
-		GLX_BLUE_SIZE, 		1,
+		GLX_BLUE_SIZE, 			1,
 		GLX_DEPTH_SIZE, 		1,
 		GLX_ALPHA_SIZE, 		1,
-		GLX_STENCIL_SIZE, 	1,
+		GLX_STENCIL_SIZE, 		1,
 		/// a tester ...
 		GLX_SAMPLE_BUFFERS_ARB, 1,
-		GLX_SAMPLES_ARB, 4,
+		GLX_SAMPLES_ARB, 		4,
 		/*GL_MULTISAMPLEBUFFERS, 	1,
 		GL_MULTISAMPLESAMPLES, 	2,*/
 		None};
@@ -181,118 +219,84 @@ gboolean cairo_dock_initialize_opengl_backend (gboolean bForceOpenGL)  // taken
 		pVisInfo = _get_visual_from_fbconfigs (pFBConfigs, iNumOfFBConfigs, XDisplay);
 		if (pFBConfigs)
 			XFree (pFBConfigs);
-	}
-	
-	//\_________________ Si toujours rien, on essaye avec la methode de gdkgl.
-	if (pVisInfo == NULL)
-	{
-		cd_warning ("still couldn't find an appropriate visual ourself, trying something else, this may not work with some drivers ...");
-		g_openglConfig.pGlConfig = gdk_gl_config_new_by_mode (GDK_GL_MODE_RGB |
-			GDK_GL_MODE_ALPHA |
-			GDK_GL_MODE_DEPTH |
-			GDK_GL_MODE_DOUBLE |
-			GDK_GL_MODE_STENCIL);
-		bStencilBufferAvailable = TRUE;
-		bAlphaAvailable = TRUE;
 		
-		if (g_openglConfig.pGlConfig == NULL)
+		// if still no luck, and user insisted on using OpenGL, give up transparency.
+		if (pVisInfo == NULL && bForceOpenGL)
 		{
-			cd_warning ("no luck, trying without double-buffer and stencil ...");
-			g_openglConfig.pGlConfig = gdk_gl_config_new_by_mode (GDK_GL_MODE_RGB |
-				GDK_GL_MODE_ALPHA |
-				GDK_GL_MODE_DEPTH);
-			bStencilBufferAvailable = FALSE;
-			bAlphaAvailable = TRUE;
-		}
-	}
-	
-	//\_________________ Si toujours rien et si l'utilisateur n'en demord pas, on en cherche un sans canal alpha.
-	if (pVisInfo == NULL && g_openglConfig.pGlConfig == NULL && bForceOpenGL)
-	{
-		cd_warning ("we could not get an ARGB-visual, trying to get an RGB one (fake transparency will be used in return) ...");
-		g_openglConfig.bAlphaAvailable = FALSE;
-		doubleBufferAttributes[14] = None;
-		int i, iNumOfFBConfigs;
-		pFBConfigs = glXChooseFBConfig (XDisplay,
-			DefaultScreen (XDisplay),
-			doubleBufferAttributes,
-			&iNumOfFBConfigs);
-		bStencilBufferAvailable = FALSE;
-		bAlphaAvailable = FALSE;
-		cd_debug ("got %d FBConfig(s) without alpha channel", iNumOfFBConfigs);
-		for (i = 0; i < iNumOfFBConfigs; i++)
-		{
-			pVisInfo = glXGetVisualFromFBConfig (XDisplay, pFBConfigs[i]);
-			if (!pVisInfo)
-			{
-				cd_warning ("this FBConfig has no visual.");
-				XFree (pVisInfo);
-				pVisInfo = NULL;
-			}
-			else
-				break;
-		}
-		if (pFBConfigs)
-			XFree (pFBConfigs);
-		
-		if (pVisInfo == NULL)
-		{
-			cd_warning ("still no visual, this is the last chance");
-			pVisInfo = glXChooseVisual (XDisplay,
+			cd_warning ("we could not get an ARGB-visual, trying to get an RGB one (fake transparency will be used in return) ...");
+			g_openglConfig.bAlphaAvailable = FALSE;
+			doubleBufferAttributes[14] = None;
+			int i, iNumOfFBConfigs;
+			pFBConfigs = glXChooseFBConfig (XDisplay,
 				DefaultScreen (XDisplay),
-				doubleBufferAttributes);
+				doubleBufferAttributes,
+				&iNumOfFBConfigs);
+			bStencilBufferAvailable = FALSE;
+			bAlphaAvailable = FALSE;
+			cd_debug ("got %d FBConfig(s) without alpha channel", iNumOfFBConfigs);
+			for (i = 0; i < iNumOfFBConfigs; i++)
+			{
+				pVisInfo = glXGetVisualFromFBConfig (XDisplay, pFBConfigs[i]);
+				if (!pVisInfo)
+				{
+					cd_warning ("this FBConfig has no visual.");
+					XFree (pVisInfo);
+					pVisInfo = NULL;
+				}
+				else
+					break;
+			}
+			if (pFBConfigs)
+				XFree (pFBConfigs);
+			
+			if (pVisInfo == NULL)
+			{
+				cd_warning ("still no visual, this is the last chance");
+				pVisInfo = glXChooseVisual (XDisplay,
+					DefaultScreen (XDisplay),
+					doubleBufferAttributes);
+			}
 		}
+		
 	}
+	
 	g_openglConfig.bStencilBufferAvailable = bStencilBufferAvailable;
 	g_openglConfig.bAlphaAvailable = bAlphaAvailable;
 	
 	//\_________________ Si rien de chez rien, on quitte.
-	if (pVisInfo == NULL && g_openglConfig.pGlConfig == NULL)
+	if (pVisInfo == NULL)
 	{
 		cd_warning ("couldn't find a suitable GLX Visual, OpenGL can't be used.\n (sorry to say that, but your graphic card and/or its driver is crappy)");
 		return FALSE;
 	}
 	
-	//\_________________ On en deduit une config opengl valide pour les widgets gtk.
-	if (pVisInfo != NULL)
-	{
-		cd_message ("ok, got a visual");
-		/// The function below sets a new colormap on the root window, and Qt doesn't like that !...
-		/// link with https://bugzilla.redhat.com/show_bug.cgi?id=440340 ?
-		g_openglConfig.pGlConfig = gdk_x11_gl_config_new_from_visualid (pVisInfo->visualid);
-		/// as a workaround, we remove this colormap.
-		Display *dpy = gdk_x11_display_get_xdisplay (gdk_display_get_default ());
-		XDeleteProperty ( dpy, DefaultRootWindow (dpy), XInternAtom (dpy, "RGB_DEFAULT_MAP", 0) ); // (XA_RGB_DEFAULT_MAP)
-		XFree (pVisInfo);
-	}
-	g_return_val_if_fail (g_openglConfig.pGlConfig != NULL, FALSE);
-	g_bUseOpenGL = TRUE;
+	//\_________________ create a context for this visual. All other context will share ressources with it, and it will be the default context in case no other context exist.
+	Display *dpy = gdk_x11_display_get_xdisplay (gdk_display_get_default ());
+	g_openglConfig.context = glXCreateContext (dpy, pVisInfo, NULL, TRUE);
+	g_return_val_if_fail (g_openglConfig.context != 0, FALSE);
 	
-	//\_________________ On regarde si le rendu doit etre indirect ou pas.
-	/**if (glXQueryVersion(XDisplay, &g_openglConfig.iGlxMajor, &g_openglConfig.iGlxMinor) == False)
-	{
-		cd_warning ("GLX not available !\nFear the worst !");
-	}
-	else
-	{
-		if (g_openglConfig.iGlxMajor <= 1 && g_openglConfig.iGlxMinor < 3)  // on a besoin de GLX >= 1.3 pour les pbuffers; mais si on a l'extension GLX_SGIX_pbuffer et un version inferieure, ca marche quand meme.
-		{
-			if (! _check_client_glx_extension ("GLX_SGIX_pbuffer"))
-			{
-				cd_warning ("No pbuffer extension in GLX.\n this might affect the drawing of some applets which are inside a dock");
-			}
-			else
-			{
-				cd_warning ("GLX version too old (%d.%d).\nCairo-Dock needs at least GLX 1.3. Indirect rendering will be toggled on/off as a workaround.", g_openglConfig.iGlxMajor, g_openglConfig.iGlxMinor);
-				g_openglConfig.bPBufferAvailable = TRUE;
-				g_openglConfig.bIndirectRendering = ! bToggleIndirectRendering;
-			}
-		}
-		else
-		{
-			g_openglConfig.bPBufferAvailable = TRUE;
-		}
-	}*/
+	//\_________________ create a colormap based on this visual.
+	GdkScreen *screen = gdk_screen_get_default ();
+	GdkVisual *pGdkVisual = gdk_x11_screen_lookup_visual (screen, pVisInfo->visualid);
+	#if (GTK_MAJOR_VERSION < 3)
+	g_openglConfig.xcolormap = XCreateColormap (dpy, DefaultRootWindow (dpy), pVisInfo->visual, AllocNone);
+	g_openglConfig.pColormap = gdk_x11_colormap_foreign_new (pGdkVisual, g_openglConfig.xcolormap);
+	#else
+	g_openglConfig.pGdkVisual = pGdkVisual;
+	g_return_val_if_fail (g_openglConfig.pGdkVisual != NULL, FALSE);
+	#endif
+	g_openglConfig.pVisInfo = pVisInfo;
+	
+	/**
+	/// The function below sets a new colormap on the root window, and Qt doesn't like that !...
+	/// link with https://bugzilla.redhat.com/show_bug.cgi?id=440340 ?
+	g_openglConfig.pGlConfig = gdk_x11_gl_config_new_from_visualid (pVisInfo->visualid);
+	/// as a workaround, we remove this colormap.
+	Display *dpy = gdk_x11_display_get_xdisplay (gdk_display_get_default ());
+	XDeleteProperty ( dpy, DefaultRootWindow (dpy), XInternAtom (dpy, "RGB_DEFAULT_MAP", 0) ); // (XA_RGB_DEFAULT_MAP)
+	XFree (pVisInfo);
+	g_return_val_if_fail (g_openglConfig.pGlConfig != NULL, FALSE);*/
+	g_bUseOpenGL = TRUE;
 	
 	//\_________________ on verifier les capacites des textures.
 	g_openglConfig.bTextureFromPixmapAvailable = _check_client_glx_extension ("GLX_EXT_texture_from_pixmap");
@@ -322,20 +326,12 @@ void cairo_dock_create_icon_fbo (void)  // it has been found that you get a spee
 	
 	glGenFramebuffersEXT(1, &g_openglConfig.iFboId);
 	
-	int iWidth = 0, iHeight = 0;
-	int i;
-	for (i = 0; i < CAIRO_DOCK_NB_GROUPS; i += 2)
-	{
-		iWidth = MAX (iWidth, myIconsParam.tIconAuthorizedWidth[i]);
-		iHeight = MAX (iHeight, myIconsParam.tIconAuthorizedHeight[i]);
-	}
-	if (iWidth == 0)
-		iWidth = 48;
-	if (iHeight == 0)
-		iHeight = 48;
+	int iWidth = myIconsParam.iIconWidth;
+	int iHeight = myIconsParam.iIconHeight;
+	
 	iWidth *= (1 + myIconsParam.fAmplitude);
 	iHeight *= (1 + myIconsParam.fAmplitude);
-	g_openglConfig.iRedirectedTexture = cairo_dock_load_texture_from_raw_data (NULL, iWidth, iHeight);
+	g_openglConfig.iRedirectedTexture = cairo_dock_create_texture_from_raw_data (NULL, iWidth, iHeight);
 }
 
 void cairo_dock_destroy_icon_fbo (void)
@@ -349,135 +345,19 @@ void cairo_dock_destroy_icon_fbo (void)
 	g_openglConfig.iRedirectedTexture = 0;
 }
 
-/**static GLXPbuffer _cairo_dock_create_pbuffer (int iWidth, int iHeight, GLXContext *pContext)
-{
-	Display *XDisplay = gdk_x11_get_default_xdisplay ();
-	
-	GLXFBConfig *pFBConfigs;
-	XRenderPictFormat *pPictFormat = NULL;
-	int visAttribs[] = {
-		GLX_DRAWABLE_TYPE, 	GLX_PBUFFER_BIT | GLX_WINDOW_BIT,
-		GLX_RENDER_TYPE, 		GLX_RGBA_BIT,
-		GLX_RED_SIZE, 		1,
-		GLX_GREEN_SIZE, 		1,
-		GLX_BLUE_SIZE, 		1,
-		GLX_ALPHA_SIZE, 		1,
-		GLX_DEPTH_SIZE, 		1,
-		None};
-	
-	XVisualInfo *pVisInfo = NULL;
-	int i, iNumOfFBConfigs = 0;
-	pFBConfigs = glXChooseFBConfig (XDisplay,
-		DefaultScreen (XDisplay),
-		visAttribs,
-		&iNumOfFBConfigs);
-	if (iNumOfFBConfigs == 0)
-	{
-		cd_warning ("No suitable visual could be found for pbuffer\n this might affect the drawing of some applets which are inside a dock");
-		*pContext = 0;
-		return 0;
-	}
-	cd_debug (" -> %d FBConfig(s) pour le pbuffer", iNumOfFBConfigs);
-	
-	int pbufAttribs [] = {
-		GLX_PBUFFER_WIDTH, iWidth,
-		GLX_PBUFFER_HEIGHT, iHeight,
-		GLX_LARGEST_PBUFFER, True,
-		None};
-	GLXPbuffer pbuffer = glXCreatePbuffer (XDisplay, pFBConfigs[0], pbufAttribs);
-	
-	pVisInfo = glXGetVisualFromFBConfig (XDisplay, pFBConfigs[0]);
-	
-	GdkGLContext *pGlContext = gtk_widget_get_gl_context (g_pMainDock->container.pWidget);
-	GLXContext mainContext = GDK_GL_CONTEXT_GLXCONTEXT (pGlContext);
-	*pContext = glXCreateContext (XDisplay, pVisInfo, mainContext, ! g_openglConfig.bIndirectRendering);
-	
-	XFree (pVisInfo);
-	XFree (pFBConfigs);
-	
-	return pbuffer;
-}
-
-void cairo_dock_create_icon_pbuffer (void)
-{
-	if (! g_openglConfig.pGlConfig || ! g_openglConfig.bPBufferAvailable)
-		return ;
-	int iWidth = 0, iHeight = 0;
-	int i;
-	for (i = 0; i < CAIRO_DOCK_NB_GROUPS; i += 2)
-	{
-		iWidth = MAX (iWidth, myIconsParam.tIconAuthorizedWidth[i]);
-		iHeight = MAX (iHeight, myIconsParam.tIconAuthorizedHeight[i]);
-	}
-	if (iWidth == 0)
-		iWidth = 48;
-	if (iHeight == 0)
-		iHeight = 48;
-	iWidth *= (1 + myIconsParam.fAmplitude);
-	iHeight *= (1 + myIconsParam.fAmplitude);
-	
-	cd_debug ("%s (%dx%d)", __func__, iWidth, iHeight);
-	if (g_openglConfig.iIconPbufferWidth != iWidth || g_openglConfig.iIconPbufferHeight != iHeight)
-	{
-		Display *XDisplay = gdk_x11_get_default_xdisplay ();
-		if (g_openglConfig.iconPbuffer != 0)
-		{
-			glXDestroyPbuffer (XDisplay, g_openglConfig.iconPbuffer);
-			glXDestroyContext (XDisplay, g_openglConfig.iconContext);
-			g_openglConfig.iconContext = 0;
-		}
-		g_openglConfig.iconPbuffer = _cairo_dock_create_pbuffer (iWidth, iHeight, &g_openglConfig.iconContext);
-		g_openglConfig.iIconPbufferWidth = iWidth;
-		g_openglConfig.iIconPbufferHeight = iHeight;
-		
-		cd_debug ("activating pbuffer, usually buggy drivers will crash here ...");
-		if (g_openglConfig.iconPbuffer != 0 && g_openglConfig.iconContext != 0 && glXMakeCurrent (XDisplay, g_openglConfig.iconPbuffer, g_openglConfig.iconContext))
-		{
-			glMatrixMode(GL_PROJECTION);
-			glLoadIdentity();
-			glOrtho(0, iWidth, 0, iHeight, 0.0, 500.0);
-			glMatrixMode (GL_MODELVIEW);
-			
-			glLoadIdentity();
-			gluLookAt (0., 0., 3.,
-				0., 0., 0.,
-				0.0f, 1.0f, 0.0f);
-			
-			glClearColor (0.0f, 0.0f, 0.0f, 0.0f);
-			glClearDepth (1.0f);
-		}
-		cd_debug (" ok, they are fine enough.\n");
-	}
-}
-
-void cairo_dock_destroy_icon_pbuffer (void)
-{
-	Display *XDisplay = gdk_x11_get_default_xdisplay ();
-	if (g_openglConfig.iconPbuffer != 0)
-	{
-		glXDestroyPbuffer (XDisplay, g_openglConfig.iconPbuffer);
-		g_openglConfig.iconPbuffer = 0;
-	}
-	if (g_openglConfig.iconContext != 0)
-	{
-		glXDestroyContext (XDisplay, g_openglConfig.iconContext);
-		g_openglConfig.iconContext = 0;
-	}
-	g_openglConfig.iIconPbufferWidth = 0;
-	g_openglConfig.iIconPbufferHeight = 0;
-}*/
 
 gboolean cairo_dock_begin_draw_icon (Icon *pIcon, CairoContainer *pContainer, gint iRenderingMode)
 {
+	//g_print ("%s (%s, %d)\n", __func__, pIcon->cName, iRenderingMode);
 	int iWidth, iHeight;
+	/// TODO: test without FBO and dock when iRenderingMode == 2
 	if (CAIRO_DOCK_IS_DESKLET (pContainer))
 	{
-		GdkGLContext *pGlContext = gtk_widget_get_gl_context (pContainer->pWidget);
-		GdkGLDrawable *pGlDrawable = gtk_widget_get_gl_drawable (pContainer->pWidget);
-		if (! gdk_gl_drawable_gl_begin (pGlDrawable, pGlContext))
+		if (! gldi_glx_make_current (pContainer))
+		{
+			pIcon->bDamaged = TRUE;
 			return FALSE;
-		
-		///cairo_dock_set_ortho_view (pContainer);
+		}
 		iWidth = pContainer->iWidth;
 		iHeight = pContainer->iHeight;
 		glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
@@ -485,13 +365,20 @@ gboolean cairo_dock_begin_draw_icon (Icon *pIcon, CairoContainer *pContainer, gi
 	else if (g_openglConfig.iFboId != 0)
 	{
 		// on attache la texture au FBO.
+		if (pContainer->iWidth == 1 && pContainer->iHeight == 1)  // container not yet fully resized
+		{
+			pIcon->bDamaged = TRUE;
+			return FALSE;
+		}
 		cairo_dock_get_icon_extent (pIcon, &iWidth, &iHeight);
 		if (pContainer == NULL)
 			pContainer = g_pPrimaryContainer;
-		GdkGLContext *pGlContext = gtk_widget_get_gl_context (pContainer->pWidget);
-		GdkGLDrawable *pGlDrawable = gtk_widget_get_gl_drawable (pContainer->pWidget);
-		if (! gdk_gl_drawable_gl_begin (pGlDrawable, pGlContext))
+		if (! gldi_glx_make_current (pContainer))
+		{
+			cd_warning ("couldn't set the opengl context");
+			pIcon->bDamaged = TRUE;
 			return FALSE;
+		}
 		glBindFramebufferEXT (GL_FRAMEBUFFER_EXT, g_openglConfig.iFboId);  // on redirige sur notre FBO.
 		g_openglConfig.bRedirected = (iRenderingMode == 2);
 		glFramebufferTexture2DEXT (GL_FRAMEBUFFER_EXT,
@@ -510,25 +397,13 @@ gboolean cairo_dock_begin_draw_icon (Icon *pIcon, CairoContainer *pContainer, gi
 				GL_TEXTURE_2D,
 				0,
 				0);
+			pIcon->bDamaged = TRUE;
 			return FALSE;
 		}
 		
-		// on se positionne au milieu.
-		/**cairo_dock_set_ortho_view (pContainer);
-		glLoadIdentity ();
-		glTranslatef (iWidth/2, iHeight/2, - iHeight/2);*/
-		
 		if (iRenderingMode != 1)
 			glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 	}
-	/**else if (g_openglConfig.iconContext != 0)
-	{
-		Display *XDisplay = gdk_x11_get_default_xdisplay ();
-		if (! glXMakeCurrent (XDisplay, g_openglConfig.iconPbuffer, g_openglConfig.iconContext))
-			return FALSE;
-		glLoadIdentity ();
-		glTranslatef (g_openglConfig.iIconPbufferWidth/2, g_openglConfig.iIconPbufferHeight/2, - g_openglConfig.iIconPbufferHeight/2);
-	}*/
 	else
 		return FALSE;
 	
@@ -604,24 +479,12 @@ void cairo_dock_end_draw_icon (Icon *pIcon, CairoContainer *pContainer)
 			0);  // on detache la texture (precaution).
 		//glGenerateMipmapEXT(GL_TEXTURE_2D);  // si on utilise les mipmaps, il faut les generer explicitement avec les FBO.
 	}
-	/**else
-	{
-		x = (g_openglConfig.iIconPbufferWidth - iWidth)/2;
-		y = (g_openglConfig.iIconPbufferHeight - iHeight)/2;
-	}*/
-		
-	///if (CAIRO_DOCK_IS_DESKLET (pContainer))
+	
 	if (pContainer && g_openglConfig.bSetPerspective)
 	{
 		cairo_dock_set_perspective_view (pContainer);
 		g_openglConfig.bSetPerspective = FALSE;
 	}
-	
-	if (pContainer)
-	{
-		GdkGLDrawable *pGlDrawable = gtk_widget_get_gl_drawable (pContainer->pWidget);
-		gdk_gl_drawable_gl_end (pGlDrawable);
-	}
 }
 
 
@@ -703,7 +566,7 @@ void cairo_dock_set_ortho_view_for_icon (Icon *pIcon, CairoContainer *pContainer
 }
 
 
-void cairo_dock_apply_desktop_background_opengl (CairoContainer *pContainer)
+void gldi_glx_apply_desktop_background (CairoContainer *pContainer)
 {
 	if (! myContainersParam.bUseFakeTransparency || ! g_pFakeTransparencyDesktopBg || g_pFakeTransparencyDesktopBg->iTexture == 0)
 		return ;
@@ -757,105 +620,115 @@ void cairo_dock_apply_desktop_background_opengl (CairoContainer *pContainer)
 }
 
 
-
-
-static void _cairo_dock_post_initialize_opengl_backend (GtkWidget* pWidget, gpointer data)  // initialisation necessitant un contexte opengl.
+gboolean gldi_glx_make_current (CairoContainer *pContainer)
 {
-	static gboolean bChecked=FALSE;
-	if (bChecked)
-		return;
-	//g_print ("*** Check for cairo-desklet\n");
-	GdkGLContext* pGlContext = gtk_widget_get_gl_context (pWidget);
-	GdkGLDrawable* pGlDrawable = gtk_widget_get_gl_drawable (pWidget);
-	if (!gdk_gl_drawable_gl_begin (pGlDrawable, pGlContext))
-		return ;
-	
-	bChecked = TRUE;
-	g_openglConfig.bNonPowerOfTwoAvailable = _check_gl_extension ("GL_ARB_texture_non_power_of_two");
-	g_openglConfig.bFboAvailable = _check_gl_extension ("GL_EXT_framebuffer_object");
-	if (!g_openglConfig.bFboAvailable)
-		cd_warning ("No FBO support, some applets will be invisible if placed inside the dock.");
-	
-	g_openglConfig.bNonPowerOfTwoAvailable = _check_gl_extension ("GL_ARB_texture_non_power_of_two");
-	g_openglConfig.bAccumBufferAvailable = _check_gl_extension ("GL_SUN_slice_accum");
-	
-	GLfloat fMaximumAnistropy = 0.;
-	if (_check_gl_extension ("GL_EXT_texture_filter_anisotropic"))
-	{
-		glGetFloatv(GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT, &fMaximumAnistropy);
-		glTexParameterf (GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, fMaximumAnistropy);
-	}
-	
-	cd_debug ("OpenGL config summary :\n - bNonPowerOfTwoAvailable : %d\n - bFboAvailable : %d\n - direct rendering : %d\n - bTextureFromPixmapAvailable : %d\n - bAccumBufferAvailable : %d\n - Anisotroy filtering level max : %.1f\n - OpenGL version: %s\n - OpenGL vendor: %s\n - OpenGL renderer: %s\n\n",
-		g_openglConfig.bNonPowerOfTwoAvailable,
-		g_openglConfig.bFboAvailable,
-		!g_openglConfig.bIndirectRendering,
-		g_openglConfig.bTextureFromPixmapAvailable,
-		g_openglConfig.bAccumBufferAvailable,
-		fMaximumAnistropy,
-		glGetString (GL_VERSION),
-		glGetString (GL_VENDOR),
-		glGetString (GL_RENDERER));
-	
-	gdk_gl_drawable_gl_end (pGlDrawable);
+	Display *dpy = gdk_x11_display_get_xdisplay (gdk_display_get_default ());
+	Window Xid = gldi_container_get_Xid (pContainer);
+
+	return glXMakeCurrent (dpy, Xid, pContainer->glContext);
 }
 
+gboolean gldi_glx_begin_draw_container_full (CairoContainer *pContainer, gboolean bClear)
+{
+	if (! gldi_glx_make_current (pContainer))
+		return FALSE;
+	
+	if (bClear)
+	{
+		glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
+		gldi_glx_apply_desktop_background (pContainer);
+	}
+	glLoadIdentity ();
+	
+	return TRUE;
+}
 
-static void _reset_opengl_context (GtkWidget* pWidget, gpointer data)
+void gldi_glx_end_draw_container (CairoContainer *pContainer)
+{
+	Display *dpy = gdk_x11_display_get_xdisplay (gdk_display_get_default ());
+	Window Xid = gldi_container_get_Xid (pContainer);
+
+	glXSwapBuffers (dpy, Xid);
+}
+
+static void _init_opengl_context (GtkWidget* pWidget, CairoContainer *pContainer)
 {
 	if (! g_bUseOpenGL)
 		return ;
 	
-	GdkGLContext* pGlContext = gtk_widget_get_gl_context (pWidget);
-	GdkGLDrawable* pGlDrawable = gtk_widget_get_gl_drawable (pWidget);
-	if (!gdk_gl_drawable_gl_begin (pGlDrawable, pGlContext))
-		return ;
+	if (! gldi_glx_make_current (pContainer))
+		return;
 	
+	//g_print ("INIT OPENGL ctx\n");
 	glClearColor (0.0f, 0.0f, 0.0f, 0.0f);
 	glClearDepth (1.0f);
 	glClearStencil (0);
 	glHint (GL_LINE_SMOOTH_HINT, GL_NICEST);
 	
 	/// a tester ...
-	if (g_bEasterEggs)
-		glEnable (GL_MULTISAMPLE_ARB);
+	///if (g_bEasterEggs)
+	///	glEnable (GL_MULTISAMPLE_ARB);
 	
+	// set once and for all
 	glTexEnvi (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);  // GL_MODULATE / GL_DECAL /  GL_BLEND
 	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
 	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
 	glTexParameteri (GL_TEXTURE_2D,
 		GL_TEXTURE_MIN_FILTER,
-		GL_LINEAR_MIPMAP_LINEAR);
+		g_bEasterEggs ? GL_LINEAR_MIPMAP_LINEAR : GL_LINEAR);
+	if (g_bEasterEggs)
+		glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP, GL_TRUE);
 	glTexParameteri (GL_TEXTURE_2D,
 		GL_TEXTURE_MAG_FILTER,
 		GL_LINEAR);
-	
-	gdk_gl_drawable_gl_end (pGlDrawable);
 }
-void cairo_dock_set_gl_capabilities (GtkWidget *pWindow)
+void gldi_glx_init_container (CairoContainer *pContainer)
 {
-	gboolean bFirstContainer = (! g_pPrimaryContainer || ! g_pPrimaryContainer->pWidget);
-	GdkGLContext *pMainGlContext = (bFirstContainer ? NULL : gtk_widget_get_gl_context (g_pPrimaryContainer->pWidget));  // NULL si on est en train de creer la fenetre du main dock, ce qui nous convient.
-	gtk_widget_set_gl_capability (pWindow,
-		g_openglConfig.pGlConfig,
-		pMainGlContext,  // on partage les ressources entre les contextes.
-		! g_openglConfig.bIndirectRendering,  // TRUE <=> direct connection to the graphics system.
-		GDK_GL_RGBA_TYPE);
-	if (bFirstContainer)  // c'est donc le 1er container cree, on finit l'initialisation du backend opengl.
-		g_signal_connect (G_OBJECT (pWindow),
+	// Set the visual we found during the init
+	#if (GTK_MAJOR_VERSION < 3)
+	gtk_widget_set_colormap (pContainer->pWidget, g_openglConfig.pColormap);
+	#else
+	gtk_widget_set_visual (pContainer->pWidget, g_openglConfig.pGdkVisual);
+	#endif
+	
+	// create a GL context for this container (this way, we can set the viewport once and for all).
+	Display *dpy = gdk_x11_display_get_xdisplay (gdk_display_get_default ());
+	///gboolean bFirstContainer = (! g_pPrimaryContainer || ! g_pPrimaryContainer->pWidget);
+	///GLXContext context = (bFirstContainer ? NULL : g_pPrimaryContainer->glContext);
+	GLXContext context = g_openglConfig.context;
+	pContainer->glContext = glXCreateContext (dpy, g_openglConfig.pVisInfo, context, TRUE);
+	
+	// handle the double buffer manually.
+	gtk_widget_set_double_buffered (pContainer->pWidget, FALSE);
+	
+	// finish the initialisation of the opengl backend, now that we have a context (and a window to bind it to)
+	if (! s_bInitialized)
+		g_signal_connect (G_OBJECT (pContainer->pWidget),
 			"realize",
-			G_CALLBACK (_cairo_dock_post_initialize_opengl_backend),
-			NULL);
-	g_signal_connect_after (G_OBJECT (pWindow),
+			G_CALLBACK (_post_initialize_opengl_backend),
+			pContainer);
+	
+	// when the window will be realised, initialise its GL context.
+	g_signal_connect_after (G_OBJECT (pContainer->pWidget),
 		"realize",
-		G_CALLBACK (_reset_opengl_context),
-		NULL);
+		G_CALLBACK (_init_opengl_context),
+		pContainer);
 }
 
-void cairo_dock_set_default_gl_context (void)
+void gldi_glx_finish_container (CairoContainer *pContainer)
 {
-	GdkGLContext *pGlContext = gtk_widget_get_gl_context (g_pPrimaryContainer->pWidget);
-	GdkGLDrawable *pGlDrawable = gtk_widget_get_gl_drawable (g_pPrimaryContainer->pWidget);
-	if (gdk_gl_drawable_gl_begin (pGlDrawable, pGlContext))
-		gdk_gl_drawable_gl_end (pGlDrawable);
+	if (pContainer->glContext != 0)
+	{
+		Display *dpy = gdk_x11_display_get_xdisplay (gdk_display_get_default ());
+		
+		if (glXGetCurrentContext() == pContainer->glContext)
+		{
+			if (g_pPrimaryContainer != NULL && pContainer != g_pPrimaryContainer)
+				gldi_glx_make_current (g_pPrimaryContainer);
+			else
+				glXMakeCurrent (dpy, 0, g_openglConfig.context);
+		}
+		
+		glXDestroyContext (dpy, pContainer->glContext);
+	}
 }
