@@ -30,12 +30,6 @@
 #include <glib/gi18n.h>
 //#include <X11/extensions/Xdamage.h>
 
-#ifdef HAVE_GLITZ
-#include <gdk/gdkx.h>
-#include <glitz-glx.h>
-#include <cairo-glitz.h>
-#endif
-
 #include <GL/gl.h>
 #include <GL/glx.h>
 
@@ -398,13 +392,23 @@ typedef struct _CairoDockWMBackend CairoDockWMBackend;
 
 typedef struct _CairoDockImageBuffer CairoDockImageBuffer;
 
-typedef struct _CairoEmblem CairoEmblem;
+typedef struct _CairoOverlay CairoOverlay;
+
+typedef struct _CairoKeyBinding CairoKeyBinding;
 
 typedef struct _GldiManager GldiManager;
 
+typedef struct _GldiObject GldiObject;
+
 typedef void (* _CairoDockForeachIconFunc) (Icon *icon, CairoContainer *pContainer, gpointer data);
 typedef _CairoDockForeachIconFunc CairoDockForeachIconFunc;
 
+#if (GTK_MAJOR_VERSION < 3)
+#define GldiShape GdkBitmap
+#else  // GTK3
+#define GldiShape cairo_region_t
+#endif
+
 
 #define CAIRO_DOCK_NB_DATA_SLOT 12
 
