@@ -85,6 +85,7 @@ void cairo_dock_calculate_constrainted_size (double *fImageWidth, double *fImage
 	gboolean bFillSpace = iLoadingModifier & CAIRO_DOCK_FILL_SPACE;
 	gboolean bKeepRatio = iLoadingModifier & CAIRO_DOCK_KEEP_RATIO;
 	gboolean bNoZoomUp = iLoadingModifier & CAIRO_DOCK_DONT_ZOOM_IN;
+	gboolean bAnimated = iLoadingModifier & CAIRO_DOCK_ANIMATED_IMAGE;
 	gint iOrientation = iLoadingModifier & CAIRO_DOCK_ORIENTATION_MASK;
 	if (iOrientation > CAIRO_DOCK_ORIENTATION_VFLIP)  // inversion x/y
 	{
@@ -92,6 +93,33 @@ void cairo_dock_calculate_constrainted_size (double *fImageWidth, double *fImage
 		*fImageWidth = *fImageHeight;
 		*fImageHeight = tmp;
 	}
+	
+	if (bAnimated)
+	{
+		if (*fImageWidth > *fImageHeight)
+		{
+			if (((int)*fImageWidth) % ((int)*fImageHeight) == 0)  // w = k*h
+			{
+				iWidthConstraint = *fImageWidth / *fImageHeight * iHeightConstraint;
+			}
+			else if (*fImageWidth > 2 * *fImageHeight)  // if we're pretty sure this image is an animated one, try to be smart, to handle the case of non-square frames.
+			{
+				// assume we have wide frames => w > h
+				int w = *fImageHeight + 1, h = *fImageHeight;
+				do
+				{
+					if ((int)*fImageWidth % w == 0)
+					{
+						iWidthConstraint = *fImageWidth / w * iHeightConstraint;
+						//g_print ("frame: %d, %d\n", w, iWidthConstraint);
+						break;
+					}
+					w ++;
+				} while (w < (*fImageWidth) / 2);
+			}
+		}
+	}
+	
 	if (bKeepRatio)
 	{
 		cairo_dock_calculate_size_constant_ratio (fImageWidth,
@@ -569,7 +597,7 @@ cairo_surface_t *cairo_dock_create_surface_from_icon (const gchar *cImageFile, d
 	if (*cImageFile == '/')
 		cIconPath = (gchar *)cImageFile;
 	else
-		cIconPath = cairo_dock_search_icon_s_path (cImageFile);
+		cIconPath = cairo_dock_search_icon_s_path (cImageFile, (gint) MAX (fImageWidth, fImageHeight));
 		
 	cairo_surface_t *pSurface = cairo_dock_create_surface_from_image (cIconPath,
 		1.,
@@ -677,7 +705,7 @@ cairo_surface_t * cairo_dock_rotate_surface (cairo_surface_t *pSurface, double f
 }
 
 
-static cairo_surface_t * cairo_dock_create_reflection_surface_horizontal (cairo_surface_t *pSurface, double fImageWidth, double fImageHeight, double fReflectSize, double fAlbedo, gboolean bDirectionUp)
+/**static cairo_surface_t * cairo_dock_create_reflection_surface_horizontal (cairo_surface_t *pSurface, double fImageWidth, double fImageHeight, double fReflectSize, double fAlbedo, gboolean bDirectionUp)
 {
 	//g_print ("%s (%d)\n", __func__, bDirectionUp);
 
@@ -779,14 +807,14 @@ cairo_surface_t * cairo_dock_create_reflection_surface (cairo_surface_t *pSurfac
 		return cairo_dock_create_reflection_surface_horizontal (pSurface, fImageWidth, fImageHeight, fReflectSize, fAlbedo, bDirectionUp);
 	else
 		return cairo_dock_create_reflection_surface_vertical (pSurface, fImageWidth, fImageHeight, fReflectSize, fAlbedo, bDirectionUp);
-}
+}*/
 
 
 static void _cairo_dock_limit_string_width (gchar *cLine, PangoLayout *pLayout, gboolean bUseMarkup, int iMaxWidth)
 {
 	//g_print ("%s (%s)\n", __func__, cLine);
 	// on insere des retours chariot pour tenir dans la largeur donnee.
-	PangoRectangle ink, log;
+	PangoRectangle log;
 	gchar *sp, *last_sp=NULL;
 	double w;
 	int iNbLines = 0;
@@ -807,8 +835,8 @@ static void _cairo_dock_limit_string_width (gchar *cLine, PangoLayout *pLayout,
 			pango_layout_set_markup (pLayout, str, -1);
 		else
 			pango_layout_set_text (pLayout, str, -1);
-		pango_layout_get_pixel_extents (pLayout, &ink, &log);
-		//g_print ("%s => w:%d/%d, x:%d/%d\n", str, log.width, ink.width, log.x, ink.x);
+		pango_layout_get_pixel_extents (pLayout, NULL, &log);
+		//g_print ("%s => w:%d, x:%d\n", str, log.width, log.x);
 		w = log.width + log.x;
 		
 		if (w > iMaxWidth)  // on deborde.
@@ -847,7 +875,7 @@ static void _cairo_dock_limit_string_width (gchar *cLine, PangoLayout *pLayout,
 		pango_layout_set_markup (pLayout, str, -1);
 	else
 		pango_layout_set_text (pLayout, str, -1);
-	pango_layout_get_pixel_extents (pLayout, &ink, &log);
+	pango_layout_get_pixel_extents (pLayout, NULL, &log);
 	w = log.width + log.x;
 	if (w > iMaxWidth)  // on deborde.
 	{
@@ -856,7 +884,7 @@ static void _cairo_dock_limit_string_width (gchar *cLine, PangoLayout *pLayout,
 	}
 }
 
-cairo_surface_t *cairo_dock_create_surface_from_text_full (const gchar *cText, CairoDockLabelDescription *pLabelDescription, double fMaxScale, int iMaxWidth, int *iTextWidth, int *iTextHeight, double *fTextXOffset, double *fTextYOffset)
+cairo_surface_t *cairo_dock_create_surface_from_text_full (const gchar *cText, CairoDockLabelDescription *pLabelDescription, double fMaxScale, int iMaxWidth, int *iTextWidth, int *iTextHeight)
 {
 	g_return_val_if_fail (cText != NULL && pLabelDescription != NULL, NULL);
 	cairo_t *pSourceContext = _get_source_context ();
@@ -864,7 +892,7 @@ cairo_surface_t *cairo_dock_create_surface_from_text_full (const gchar *cText, C
 	
 	//\_________________ On ecrit le texte dans un calque Pango.
 	PangoLayout *pLayout = pango_cairo_create_layout (pSourceContext);
-	PangoRectangle ink, log;
+	PangoRectangle log;
 	
 	PangoFontDescription *pDesc = pango_font_description_new ();
 	pango_font_description_set_absolute_size (pDesc, fMaxScale * pLabelDescription->iSize * PANGO_SCALE);
@@ -874,22 +902,18 @@ cairo_surface_t *cairo_dock_create_surface_from_text_full (const gchar *cText, C
 	pango_layout_set_font_description (pLayout, pDesc);
 	pango_font_description_free (pDesc);
 	
-	pango_layout_set_text (pLayout, "|", -1);  // donne la hauteur max des lettres.
-	pango_layout_get_pixel_extents (pLayout, &ink, &log);
-	int iMinSize = ink.height;  // hauteur min = au moins la plus grande lettre pour une uniformite des labels.
-	
 	if (pLabelDescription->bUseMarkup)
 		pango_layout_set_markup (pLayout, cText, -1);
 	else
 		pango_layout_set_text (pLayout, cText, -1);
 	
 	//\_________________ On insere des retours chariot si necessaire.
-	pango_layout_get_pixel_extents (pLayout, &ink, &log);
+	pango_layout_get_pixel_extents (pLayout, NULL, &log);
 	
 	if (pLabelDescription->fMaxRelativeWidth != 0)
 	{
 		int iMaxLineWidth = pLabelDescription->fMaxRelativeWidth * g_desktopGeometry.iScreenWidth[CAIRO_DOCK_HORIZONTAL];
-		int w = ink.width;
+		int w = log.width;
 		//g_print ("text width : %d / %d\n", w, iMaxLineWidth);
 		if (w > iMaxLineWidth)  // le texte est trop long.
 		{
@@ -910,7 +934,7 @@ cairo_surface_t *cairo_dock_create_surface_from_text_full (const gchar *cText, C
 				pango_layout_set_markup (pLayout, cCutText, -1);
 			else
 				pango_layout_set_text (pLayout, cCutText, -1);
-			pango_layout_get_pixel_extents (pLayout, &ink, &log);
+			pango_layout_get_pixel_extents (pLayout, NULL, &log);
 			g_strfreev (cLines);
 			g_free (cCutText);
 		}
@@ -918,18 +942,18 @@ cairo_surface_t *cairo_dock_create_surface_from_text_full (const gchar *cText, C
 	
 	//\_________________ On cree une surface aux dimensions du texte.
 	gboolean bDrawBackground = (pLabelDescription->fBackgroundColor != NULL && pLabelDescription->fBackgroundColor[3] > 0);
-	double fRadius = fMaxScale * MAX (pLabelDescription->iMargin, MIN (6, pLabelDescription->iSize/3));  // permet d'avoir un rayon meme si on n'a pas de marge.
+	double fRadius = fMaxScale * MAX (pLabelDescription->iMargin, MIN (6, pLabelDescription->iSize/4));  // permet d'avoir un rayon meme si on n'a pas de marge.
 	int iOutlineMargin = 2*pLabelDescription->iMargin + (pLabelDescription->bOutlined ? 2 : 0);  // outlined => +1 tout autour des lettres.
-	double fZoomX = ((iMaxWidth != 0 && ink.width + iOutlineMargin > iMaxWidth) ? (double)iMaxWidth / (ink.width + iOutlineMargin) : 1.);
+	double fZoomX = ((iMaxWidth != 0 && log.width + iOutlineMargin > iMaxWidth) ? (double)iMaxWidth / (log.width + iOutlineMargin) : 1.);
 	
-	*iTextWidth = (ink.width + iOutlineMargin) * fZoomX;  // le texte + la marge de chaque cote.
+	*iTextWidth = (log.width + iOutlineMargin) * fZoomX;  // le texte + la marge de chaque cote.
 	if (bDrawBackground)  // quand on trace le cadre, on evite qu'avec des petits textes genre "1" on obtienne un fond tout rond.
 	{
 		*iTextWidth = MAX (*iTextWidth, 2 * fRadius + 10);
 		if (iMaxWidth != 0 && *iTextWidth > iMaxWidth)
 			*iTextWidth = iMaxWidth;
 	}
-	*iTextHeight = MAX (iMinSize, ink.height) + iOutlineMargin + 0; // +1 car certaines polices "debordent".
+	*iTextHeight = log.height + iOutlineMargin;
 	
 	cairo_surface_t* pNewSurface = cairo_dock_create_blank_surface (
 		*iTextWidth,
@@ -948,12 +972,12 @@ cairo_surface_t *cairo_dock_create_surface_from_text_full (const gchar *cText, C
 		cairo_restore(pCairoContext);
 	}
 	
-	//g_print ("%s : ink = %d;%d\n", cText, (int) ink.x, (int) ink.y);
-	int dx = (*iTextWidth - ink.width * fZoomX)/2;  // pour se centrer.
-	int dy = (*iTextHeight - ink.height)/2;  // pour se centrer.
+	//g_print ("%s : log = %d;%d\n", cText, (int) log.x, (int) log.y);
+	int dx = (*iTextWidth - log.width * fZoomX)/2;  // pour se centrer.
+	int dy = (*iTextHeight - log.height)/2;  // pour se centrer.
 	cairo_translate (pCairoContext,
-		-ink.x*fZoomX + dx,
-		-ink.y + dy);  // meme remarque pour le +1.
+		-log.x*fZoomX + dx,
+		-log.y + dy);
 	
 	//\_________________ On dessine les contours du texte.
 	if (pLabelDescription->bOutlined)
@@ -986,13 +1010,13 @@ cairo_surface_t *cairo_dock_create_surface_from_text_full (const gchar *cText, C
 	{
 		if (pLabelDescription->bVerticalPattern)
 			pGradationPattern = cairo_pattern_create_linear (0.,
-				ink.y + 0,  // meme remarque pour le +1.
+				log.y,
 				0.,
-				ink.y + 0 + ink.height);
+				log.y + log.height);
 		else
-			pGradationPattern = cairo_pattern_create_linear (ink.x + 0,
+			pGradationPattern = cairo_pattern_create_linear (log.x,
 				0.,
-				ink.x + 0 + ink.width,
+				log.x + log.width,
 				0.);
 		g_return_val_if_fail (cairo_pattern_status (pGradationPattern) == CAIRO_STATUS_SUCCESS, NULL);
 		cairo_pattern_set_extend (pGradationPattern, CAIRO_EXTEND_NONE);
@@ -1022,18 +1046,8 @@ cairo_surface_t *cairo_dock_create_surface_from_text_full (const gchar *cText, C
 	
 	cairo_destroy (pCairoContext);
 	
-	/* set_device_offset is buggy, doesn't work for positive offsets. so we use explicit offsets... so unfortunate.
-	cairo_surface_set_device_offset (pNewSurface,
-					 log.width / 2. - ink.x,
-					 log.height     - ink.y);*/
-	if (fTextXOffset != NULL)
-		*fTextXOffset = (log.width * fZoomX / 2. - ink.x) / fMaxScale;
-	if (fTextYOffset != NULL)
-		*fTextYOffset = - (pLabelDescription->iSize - (log.height - ink.y)) / fMaxScale ;  // en tenant compte de l'ecart du bas du texte.
-	// *fTextYOffset = - (ink.y) / fMaxScale;  // pour tenir compte de l'ecart du bas du texte.
-	
-	*iTextWidth = *iTextWidth / fMaxScale;
-	*iTextHeight = *iTextHeight / fMaxScale;
+	*iTextWidth = *iTextWidth/** / fMaxScale*/;
+	*iTextHeight = *iTextHeight/** / fMaxScale*/;
 	
 	g_object_unref (pLayout);
 	return pNewSurface;
