@@ -40,7 +40,10 @@
 #include "cairo-dock-log.h"
 #include "cairo-dock-packages.h"
 #include "cairo-dock-core.h"
+#include "cairo-dock-dock-facility.h"
+#include "cairo-dock-opengl.h"
 #include "cairo-dock-themes-manager.h"
+#include "cairo-dock-global-variables.h"
 
 #define CAIRO_DOCK_MODIFIED_THEME_FILE ".cairo-dock-need-save"
 
@@ -57,6 +60,7 @@ gchar *g_cConfFile = NULL;  // le chemin du fichier de conf.
 static gchar *s_cLocalThemeDirPath = NULL;
 static gchar *s_cDistantThemeDirName = NULL;
 
+extern gboolean g_bUseOpenGL;
 extern CairoDock *g_pMainDock;
 
 #define CAIRO_DOCK_LOCAL_EXTRAS_DIR "extras"
@@ -187,33 +191,10 @@ gboolean cairo_dock_export_current_theme (const gchar *cNewThemeName, gboolean b
 	g_string_printf (sCommand, "rm -f \"%s/last-modif\"", cNewThemePath);
 	r = system (sCommand->str);
 	
-	/// TODO: check vertical docks, and handle opengl ...
-	if (g_pMainDock && g_pMainDock->pRenderer)
-	{
-		cairo_surface_t *pSurface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32,
-			g_pMainDock->iActiveWidth,
-			g_pMainDock->iActiveHeight);
-		cairo_t *pCairoContext = cairo_create (pSurface);
-		
-		if (!g_pMainDock->container.bIsHorizontal)
-		{
-			cairo_translate (pCairoContext, g_pMainDock->iMaxDockWidth/2, g_pMainDock->iMaxDockHeight/2);
-			cairo_rotate (pCairoContext, -G_PI/2);
-			
-			if (!g_pMainDock->container.bDirectionUp)
-			{
-				
-			}
-			cairo_translate (pCairoContext, -g_pMainDock->iMaxDockHeight/2, -g_pMainDock->iMaxDockWidth/2);
-		}
-		g_pMainDock->pRenderer->render (pCairoContext, g_pMainDock);
-		
-		gchar *cPreviewPath = g_strdup_printf ("%s/preview", cNewThemePath);
-		cairo_surface_write_to_png (pSurface, cPreviewPath);
-		g_free (cPreviewPath);
-		cairo_destroy (pCairoContext);
-		cairo_surface_destroy (pSurface);
-	}
+	//\___________________ make a preview of the current main dock.
+	gchar *cPreviewPath = g_strdup_printf ("%s/preview", cNewThemePath);
+	cairo_dock_make_preview (g_pMainDock, cPreviewPath);
+	g_free (cPreviewPath);
 	
 	//\___________________ Le theme n'est plus en etat 'modifie'.
 	g_free (cNewThemePath);
@@ -235,7 +216,7 @@ gboolean cairo_dock_package_current_theme (const gchar *cThemeName)
 	
 	cd_message ("building theme package ...");
 	int r;
-	if (g_file_test (GLDI_SHARE_DATA_DIR"/../../bin/cairo-dock-package-theme", G_FILE_TEST_EXISTS))
+	if (g_file_test (GLDI_BIN_DIR"/cairo-dock-package-theme", G_FILE_TEST_EXISTS))
 	{
 		gchar *cCommand;
 		const gchar *cTerm = g_getenv ("TERM");
@@ -258,23 +239,18 @@ gchar *cairo_dock_depackage_theme (const gchar *cPackagePath)
 	gchar *cNewThemePath = NULL;
 	if (*cPackagePath == '/' || strncmp (cPackagePath, "file://", 7) == 0)  // paquet en local.
 	{
-		cd_debug (" paquet local\n");
+		cd_debug (" paquet local");
 		gchar *cFilePath = (*cPackagePath == '/' ? g_strdup (cPackagePath) : g_filename_from_uri (cPackagePath, NULL, NULL));
 		cNewThemePath = cairo_dock_uncompress_file (cFilePath, g_cThemesDirPath, NULL);
 		g_free (cFilePath);
 	}
 	else  // paquet distant.
 	{
-		cd_debug (" paquet distant\n");
-		gchar *str = strrchr (cPackagePath, '/');
-		if (str != NULL)
+		cd_debug (" paquet distant");
+		cNewThemePath = cairo_dock_download_archive (cPackagePath, g_cThemesDirPath);
+		if (cNewThemePath == NULL)
 		{
-			*str = '\0';
-			cNewThemePath = cairo_dock_download_file (cPackagePath, "", str+1, g_cThemesDirPath, NULL);
-			if (cNewThemePath == NULL)
-			{
-				cairo_dock_show_temporary_dialog_with_icon_printf (_("Could not access remote file %s/%s. Maybe the server is down.\nPlease retry later or contact us at glx-dock.org."), NULL, NULL, 0, NULL, cPackagePath, str+1);
-			}
+			cairo_dock_show_temporary_dialog_with_icon_printf (_("Could not access remote file %s. Maybe the server is down.\nPlease retry later or contact us at glx-dock.org."), NULL, NULL, 0, NULL, cPackagePath);
 		}
 	}
 	return cNewThemePath;
