@@ -31,12 +31,6 @@
 #include <X11/Xutil.h>
 #include <gdk/gdkx.h>
 
-#ifdef HAVE_GLITZ
-#include <gdk/gdkx.h>
-#include <glitz-glx.h>
-#include <cairo-glitz.h>
-#endif
-#include <gtk/gtkgl.h>
 #include <GL/glu.h>
 
 #include "cairo-dock-animations.h"  // definition of CairoDockHidingEffect
